/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_AUDIO_RESOURCES_H
#define MYSDL_AUDIO_RESOURCES_H

#include <ogg/ogg.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include "../mysdl.h"

#include "core.h"

namespace mysdl
{
	namespace audio
	{
		class SampleBank;

		class SoundProperties
		{
			public:
				Vector3D position;
				ALfloat volume;
				ALfloat rolloff;
				ALfloat pitch;
				bool loop;

				SoundProperties();
				void setStereoPosition(GLfloat stereoPosition);
		};

		class Sample : public NamedResource<SampleBank>
		{
			public:
				std::string filename;
				int priority;
				SoundProperties properties;

				Sample();
				Sample(const xml::Node *root,const std::string &baseDir);
				virtual ~Sample();

				virtual void getFromNode(const xml::Node *node,const std::string &baseDir);
				virtual void reload();
				virtual void refresh();
				virtual void release();
		};

		class StreamedSample : public Sample
		{
			public:
				long bufferSize;

				StreamedSample();
				StreamedSample(const xml::Node *root,const std::string &baseDir);

				void getFromNode(const xml::Node *node,const std::string &baseDir);
		};

		class CachedSample : public Sample
		{
			public:
				ALuint bufferName;

				ALsizei format;
				ALsizei size;
				ALsizei frequency;

				CachedSample();
				CachedSample(const xml::Node *root,const std::string &baseDir);
				virtual ~CachedSample();

				void getFromNode(const xml::Node *node,const std::string &baseDir);

				/**
					* Loads the sample from the file, using the current
					* filename.
					*/
				void load();

				/**
					* Reloads the sample from the file.
					*/
				void reload();

				/**
					* Reloads the sample from the file if necessary.
					*/
				void refresh();

				/**
					* Releases the sample.
					*/
				void release();
		};

		class SampleBank : public NamedResourceBank<Sample>
		{
			public:
				~SampleBank();

				/**
				* Reads a list of samples from a DOM node, adding them to
				* the bank.
				*
				* @param root		DOM node containing sample bank data.
				* @param baseDir Base directory for sample files.
				*/
				void getFromNode(const xml::Node *root,const std::string &baseDir);

				/**
				* Loads a list of samples from an XML file, adding them to
				* the bank.
				*
				* @param filename SampleBank filename.
				*/
				void load(const std::string &filename);

				/**
				* Reloads all samples in the bank.
				*/
				void reload();

				/**
				* Refreshes all samples in the bank.
				*/
				void refresh();

				/**
				* Releases all samples in the bank.
				*/
				void release();
		};
	};
};

#endif
