/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_AUDIO_SOURCES_H
#define MYSDL_AUDIO_SOURCES_H

#include <queue>

#include <ogg/ogg.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>

#include "../mysdl.h"

#include "core.h"
#include "loaders.h"
#include "resources.h"

namespace mysdl
{
	namespace audio
	{
		enum SourceState {ssInitialized,ssPlaying,ssPaused,ssStopped};

		class Sound;
		class Source;
		class SourcePool;
		struct CompareSourcePointers;

		class SoundHandler
		{
			public:
				virtual ~SoundHandler() {};
				virtual void soundInit(Sound *sound);
				virtual void soundUpdate(Sound *sound,GLfloat ticks);
				virtual void soundStreamed(Sound *sound);
				virtual void soundDone(Sound *sound);
		};

		/**
		 * An audio source. Audio sources are managed by SourcePools. When playing a sample,
		 * one or more audio sources from one pool are occupied and used for playback. When
		 * the sample is finished they are automatically returned to the pool.
		 */
		class Source
		{
			friend class SourcePool;
			friend class Sound;
			friend class CachedSound;
			friend class StreamedSound;
			friend class CompareSourcePointers;

			private:
				SourcePool *owner;
				SourceState state;
				int priority;
				Sound *sound;
				unsigned long stamp;

			public:
				ALuint sourceName;

				Source(SourcePool *owner,ALuint sourceName);
				~Source();

				bool operator<(const Source &source) const;
				void dropSound();
		};

		class Sound
		{
			friend class SourcePool;
			friend class Source;

			protected:
				SourcePool *owner;
				const Sample *sample;
				SoundHandler *handler;
				Source *source;
//				bool loop;
				SoundProperties properties;
				//bool ignoreStops;

				Sound(SourcePool *owner,const Sample *sample,SoundHandler *handler=0);
				virtual ~Sound();

			public:
				Source *getSource();
				virtual void perform(GLfloat ticks);
				//virtual void ready()=0;
				virtual void play();
				virtual void stop();

				void setVolume(ALfloat volume); // remember internal volume and stuff
		};

		/**
		 * A currently playing sound. Is automatically created when a sample is played via a
		 * source pool. A sample manages the connection between a sample and one or more sources
		 * and stores current "local" properties like volumne and position for the sources.
		 */
		class CachedSound : public Sound
		{
			friend class SourcePool;
//			friend class Source;

			private:
				CachedSound(SourcePool *owner,const CachedSample *sample,SoundHandler *handler=0);
//				~CachedSound();

			public:
				//void ready();
				void play();
// 				void enqueue();
		};

		class StreamedSound : public Sound
		{
			protected:
				FileLoader *loader;
				ALsizei bufferSize;
				bool fileComplete;
				ALuint buffers[2];
// 				int queueCount;

				void readBuffer(ALuint bufferName);
				void stream();
				void perform(GLfloat ticks);

			public:
				StreamedSound(SourcePool *owner,const StreamedSample *sample,SoundHandler *handler=0);
				~StreamedSound();
				//void ready();
				void play();
// 				void enqueue();
		};

		struct CompareSourcePointers
		{
			public:
				bool operator()(const Source *s1,const Source *s2) const
				{
					return(*s1<*s2);
				};
		};

		/**
		 * A pool of audio sources. The pool manages the creation and deletion of the sources. When playing
		 * an audio sample, the required number of sources is requested from the pool, which tries to return
		 * currently unused sources if possible. If not enough sources are currently unused, the occupied
		 * sources with the lowest priority stop playing their current sample and are instead used for the
		 * new sample.
		 */
		class SourcePool
		{
			friend class Source;
			friend class Sound;
			friend class StreamedSound;
			friend class CachedSound;

			protected:
				std::priority_queue<Source *,std::vector<Source *>,CompareSourcePointers> sources;
				std::list<Sound *> sounds;
				std::list<Sound *> deleteSounds;
				unsigned long stampCount;
				ALfloat volume;

			public:
				SourcePool(int size=0);
				~SourcePool();

				int getSize();
				void resize(unsigned int size);
// 				Sound *ready(Source *source,Sample *sample,const SoundProperties *properties=0,/*bool loop=false,*/SoundHandler *handler=0,int priority=-1);
// 				Sound *ready(Sample *sample,const SoundProperties *properties=0,/*bool loop=false,*/SoundHandler *handler=0,int priority=-1);
				Sound *play(Sample *sample,const SoundProperties *properties=0,/*bool loop=false,*/SoundHandler *handler=0,int priority=-1);
				Source *requestSource(int priority);
				void deleteExpiredSounds();

				void perform(GLfloat ticks);
				void stopAll();

				void setPlayingVolumes(ALfloat volume);
				void setVolume(ALfloat volume);
				ALfloat getVolume();
		};
	};
};

#endif
