/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_CONTROLS_H
#define MYSDL_CONTROLS_H

#include <list>
#include <map>
#include <set>
#include <string>

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include "mysdl.h"
#include "resources.h"

namespace mysdl
{
	enum KeyState {ctrlDefault,ctrlDown,ctrlUp,ctrlPressed};

	template<> std::string toStr<KeyState>(KeyState var);
	template<> KeyState strTo<KeyState>(std::string var);

	typedef Sint32 ControlId;

	class ControlQueue;
	class InputEvaluator;
	class Trigger;
	class DigitalTrigger;
	class Axis;

	class Evaluator
	{
		public:
			bool enabled;

			Evaluator();
			virtual ~Evaluator();

			virtual void perform(GLfloat ticks,ControlQueue *queue);
			virtual bool evaluate(const SDL_Event &event,ControlQueue *queue)=0;
			virtual void flush();
	};

	class ControlQueue
	{
		protected:
			std::list<Trigger *> eventQueue;

		public:
			std::list<Evaluator *> evaluators;

			Trigger *activateTrigger;
			Trigger *deactivateTrigger;
			
			ControlQueue();
			virtual ~ControlQueue();

			virtual void operator<<(const SDL_Event &event);
			virtual void operator<<(Trigger *trigger);
			virtual void operator>>(Trigger *&result);

			virtual void perform(GLfloat ticks);
			virtual void flush();
			virtual bool isEmpty();

			bool readEvents();
	};

	class Translator
	{
		public:
			Trigger *trigger;

			Translator(Trigger *trigger);
			virtual ~Translator();

			virtual bool translate(const SDL_Event &event,ControlQueue *queue)=0;
			virtual Translator *clone(Trigger *trigger)=0;
			virtual xml::Node *buildTranslatorTree()=0;
	};

	class Trigger : public NamedResource<InputEvaluator>
	{
		public:
			ControlId id;
			std::string humanName;
			int data;
			std::string dataString;
			std::list<Translator *> translators;
			bool digital;
			GLfloat max;
			KeyState defaultKeyState;
			bool queue;
			bool enabled;
			GLfloat value;

			Trigger(InputEvaluator *evaluator,ControlId id,const std::string &name,const std::string &humanName,KeyState defaultKeyState=ctrlDown);
			Trigger(InputEvaluator *evaluator,ControlId id,const std::string &name,const std::string &humanName,GLfloat max);
			~Trigger();

			void trigger(ControlQueue *queue);
			void trigger(ControlQueue *queue,Translator *translator,GLfloat value);
			void reset();
			bool isActive();
			void setValue(ControlQueue *queue,Translator *translator,GLfloat value);
			void getFromNode(const xml::Node *root,const std::string &baseDir="");
			Trigger *clone(InputEvaluator *evaluator);

			void clearTranslators();
			void clearKeyTranslators(GLfloat value=0.0);
			void clearJoyButtonTranslators(GLfloat value=0.0);
			void clearJoyAxisTranslators(short int direction=0);
			void clearMouseButtonTranslators(GLfloat value=0.0);
			void clearMouseAxisTranslators(short int direction=0);
			void getTranslatorsFromNode(const xml::Node *root);
			xml::Node *buildTranslatorTree();

			std::string keysToString(GLfloat value=0.0);
			std::string joyToString(short int direction=0,GLfloat value=0.0);
			std::string joyButtonsToString(GLfloat value=0.0);
			std::string joyAxisToString(short int direction=0);
			// TODO: mouse too
	};

	class KeyTranslator : public Translator
	{
		public:
			SDLKey key;
			KeyState state;
			GLfloat value;

			KeyTranslator(Trigger *trigger,SDLKey key,KeyState state=ctrlDefault,GLfloat value=1);
			virtual ~KeyTranslator();

			bool translate(const SDL_Event &event,ControlQueue *queue);
			Translator *clone(Trigger *trigger);
			xml::Node *buildTranslatorTree();
	};

	class JoyTranslator : public Translator
	{
		public:
			Sint16 joy;

			JoyTranslator(Trigger *trigger,Sint16 joy);
	};

	class JoyButtonTranslator : public JoyTranslator
	{
		public:
			Sint16 button;
			KeyState state;
			GLfloat value;

			JoyButtonTranslator(Trigger *trigger,Sint16 joy,Sint16 button,KeyState state=ctrlDefault,GLfloat value=1);
			virtual ~JoyButtonTranslator();

			bool translate(const SDL_Event &event,ControlQueue *queue);
			Translator *clone(Trigger *trigger);
			xml::Node *buildTranslatorTree();
	};

	class JoyAxisTranslator : public JoyTranslator
	{
		public:
			Uint8 axis;
			short int direction;
			Sint16 minPhysical;
			Sint16 maxPhysical;

			JoyAxisTranslator(Trigger *trigger,Sint16 joy,Uint8 axis,Sint8 direction,Sint16 minPhysical,Sint16 maxPhysical);
			virtual ~JoyAxisTranslator();

			bool translate(const SDL_Event &event,ControlQueue *queue);
			Translator *clone(Trigger *trigger);
			xml::Node *buildTranslatorTree();
	};

	class MouseButtonTranslator : public Translator
	{
		public:
			Uint8 button;
			KeyState state;
			GLfloat value;

			MouseButtonTranslator(Trigger *trigger,Uint8 button,KeyState state=ctrlDefault,GLfloat value=1);
			virtual ~MouseButtonTranslator();

			bool translate(const SDL_Event &event,ControlQueue *queue);
			Translator *clone(Trigger *trigger);
			xml::Node *buildTranslatorTree();
	};

	class MouseAxisTranslator : public Translator
	{
		public:
			Uint8 axis;
			short int direction;
			Sint16 minPhysical;
			Sint16 maxPhysical;

			MouseAxisTranslator(Trigger *trigger,Uint8 axis,Sint8 direction,Sint16 minPhysical,Sint16 maxPhysical);
			virtual ~MouseAxisTranslator();

			bool translate(const SDL_Event &event,ControlQueue *queue);
			Translator *clone(Trigger *trigger);
			xml::Node *buildTranslatorTree();
	};

	class InputEvaluator : public Evaluator,public NamedResourceBank<Trigger>
	{
		friend class Trigger;
		friend class Translator;
		friend class KeyTranslator;
		friend class JoyButtonTranslator;
		friend class JoyAxisTranslator;
		friend class MouseButtonTranslator;
		friend class MouseAxisTranslator;

		private:
			std::list<KeyTranslator *> translatorsKey;
			std::list<JoyButtonTranslator *> translatorsJoyButton;
			std::list<JoyAxisTranslator *> translatorsJoyAxis;
			std::list<MouseButtonTranslator *> translatorsMouseButton;
			std::list<MouseAxisTranslator *> translatorsMouseAxis;
			std::set<Trigger *> activeTriggers;

			bool handleKeyEvent(const SDL_Event &e,ControlQueue *queue);
			bool handleJoyButtonEvent(const SDL_Event &e,ControlQueue *queue);
			bool handleJoyAxisEvent(const SDL_Event &e,ControlQueue *queue);
			bool handleMouseButtonEvent(const SDL_Event &e,ControlQueue *queue);
			bool handleMouseAxisEvent(const SDL_Event &e,ControlQueue *queue);

		public:
			InputEvaluator();
			virtual ~InputEvaluator();

			void clear();
			virtual bool evaluate(const SDL_Event &event,ControlQueue *queue);
			virtual void perform(GLfloat ticks,ControlQueue *queue);
			virtual void flush();
			std::string getNodeName();
			void getFromNode(const xml::Node *root,const std::string &baseDir="");
			InputEvaluator *clone();
			void copyFrom(InputEvaluator *source);
			void setDataOnAllTriggers(int data);
			void getTranslatorsFromNode(const xml::Node *root);
			void addTranslatorsToTree(xml::Node *root);
	};

	extern ControlQueue queue;

	std::map<std::string,SDLKey> &getNameKeyMap();
	std::map<SDLKey,std::string> &getKeyNameMap();
};

#endif
