/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_DEBUG_H
#define MYSDL_DEBUG_H

#include <stdarg.h>

#include <list>
#include <map>
#include <string>

#include "xml.h"

namespace mysdl
{
	namespace debug
	{
		#ifdef MYSDL_DEBUG_ALL
			#define MYSDL_DEBUG_DEBUG
			//#define MYSDL_DEBUG_STACK
			//#define MYSDL_DEBUG_PROFILE
		#endif

		#ifndef MYSDL_DEBUG_DEBUG
			#ifndef MYSDL_DEBUG_INFO
				#ifndef MYSDL_DEBUG_WARNING
					#ifndef MYSDL_DEBUG_ERROR
						#ifndef MYSDL_DEBUG_CRITICAL
							#define MYSDL_DEBUG_INFO
						#endif
					#endif
				#endif
			#endif
		#endif

		#ifdef MYSDL_DEBUG_DEBUG
			#define MYSDL_DEBUG_INFO
		#endif

		#ifdef MYSDL_DEBUG_INFO
			#define MYSDL_DEBUG_WARNING
		#endif

		#ifdef MYSDL_DEBUG_WARNING
			#define MYSDL_DEBUG_ERROR
		#endif

		#ifdef MYSDL_DEBUG_ERROR
			#define MYSDL_DEBUG_CRITICAL
		#endif

		#ifdef MYSDL_NO_DEBUG_STACK
			#undefine MYSDL_DEBUG_STACK
		#endif

		#ifndef MYSDL_DEBUG_DEFAULT
			#define MYSDL_DEBUG_DEFAULT dlInfo;
		#endif

		#ifdef MYSDL_NO_DEBUG
			#undef MYSDL_DEBUG
			#undef MYSDL_DEBUG_CRITICAL
			#undef MYSDL_DEBUG_ERROR
			#undef MYSDL_DEBUG_WARNING
			#undef MYSDL_DEBUG_INFO
			//#undef MYSDL_DEBUG_DEBUG
			//#undef MYSDL_DEBUG_STACK
		#endif

		/**
		 * Initialises the profiler. TODO: Do we need the profiler??
		 */
		void init();

		/**
		 * Available debug levels.
		 *
		 * dlNone:     Not a debug level itself. Used to disable all debug levels.
		 * dlCritical: For critical error messages.
		 * dlError:    For non-critical error messages.
		 * dlwarning:  For warning messages.
		 * dlInfo:     For informative messages.
		 * dlDebug:    For debug messages.
		 */
		enum DebugLevel {dlNone=0,dlCritical,dlError,dlWarning,dlInfo,dlDebug};

		/** The default debug level, i.e. the debug level that is used if no
		 *  other debug level is specified by the application. */
		const DebugLevel DEF_DEBUG_LEVEL=MYSDL_DEBUG_DEFAULT;

		/**
		 * A debug group. For example, a game consisting of a resource manager, a menu
		 * and the actual game logic might contain debug groups called "core", "resources",
		 * "menu" and "game".
		 */
		class Group
		{
			public:
				/** Name of the group. Will be displayed along with every message for this group. */
				std::string name;

				/** The debug level of the group. Depending on the specified output debug level,
				 *  messages from this group may or may not be displayed. */
				DebugLevel level;

				/**
				 * Creates a new debug group, using the default debug level and the name
				 * "undefined".
				 */
				Group();

				/**
				 * Creates a new debug group.
				 *
				 * @param name  Name for the debug group.
				 * @param level Debug level for this group. If this is not specified,
				 *              the default debug level will be used.
				 */
				Group(const std::string name,DebugLevel level=DEF_DEBUG_LEVEL);
		};

		/**
		 * A collection of debug groups. Debug levels for the groups stored in the
		 * collection can be read from XML nodes and files to provide an easy way
		 * to customise the debug settings of an application without the need to
		 * recompile the source code.
		 */
		class Groups : public std::map<int,Group>
		{
			public:
				/**
				 * Reads the debug levels for the groups stored in the collection
				 * from the specified XML node.
				 *
				 * @param root XML node containing debug settings.
				 */
				void getSettingsFromNode(const xml::Node *root);

				/**
				 * Reads the debug levels for the groups stored in the collection
				 * from the specified file.
				 *
				 * @param filename Name of a file containing debug settings.
				 */
				void loadSettings(const char *filename);
		};

		/** Available debug groups. Add new debug groups to this collection, otherwise
		 *  they cannot be used. */
		extern Groups groups;

		/** Stack of debug messages, used to print a stack trace
		*  after access violations. */
		extern std::list<std::string> messages;

		/**
		 * Prints a critical debug message. If the debug level at compile time does is not
		 * at least "critical", this will be implemented as an emtpy inline function which
		 * does nothing.
		 *
		 * @param group Group for which the debugmessage will be printed.
		 * @param fmt   Format string ala printf(), followed by an arbitrary number of
		 *              parameters.
		 */
		#ifdef MYSDL_DEBUG_CRITICAL
			void critical(int group,const char *fmt,...);
		#else
			inline void critical(int group,const char *fmt,...)
			{
			}
		#endif

		/**
		 * Prints an error debug message. If the debug level at compile time does is not
		 * at least "error", this will be implemented as an emtpy inline function which
		 * does nothing.
		 *
		 * @param group Group for which the debugmessage will be printed.
		 * @param fmt   Format string ala printf(), followed by an arbitrary number of
		 *              parameters.
		 */
		#ifdef MYSDL_DEBUG_ERROR
			void error(int group,const char *fmt,...);
		#else
			inline void error(int group,const char *fmt,...)
			{
			}
		#endif

		/**
		 * Prints a warning debug message. If the debug level at compile time does is not
		 * at least "warning", this will be implemented as an emtpy inline function which
		 * does nothing.
		 *
		 * @param group Group for which the debugmessage will be printed.
		 * @param fmt   Format string ala printf(), followed by an arbitrary number of
		 *              parameters.
		 */
		#ifdef MYSDL_DEBUG_WARNING
			void warning(int group,const char *fmt,...);
		#else
			inline void warning(int group,const char *fmt,...)
			{
			}
		#endif

		/**
		 * Prints an info debug message. If the debug level at compile time does is not
		 * at least "info", this will be implemented as an emtpy inline function which
		 * does nothing.
		 *
		 * @param group Group for which the debugmessage will be printed.
		 * @param fmt   Format string ala printf(), followed by an arbitrary number of
		 *              parameters.
		 */
		#ifdef MYSDL_DEBUG_INFO
			void info(int group,const char *fmt,...);
		#else
			inline void info(int group,const char *fmt,...)
			{
			}
		#endif

		/**
		 * Prints a debug debug message. If the debug level at compile time does is not
		 * at least "debug", this will be implemented as an emtpy inline function which
		 * does nothing.
		 *
		 * @param group Group for which the debugmessage will be printed.
		 * @param fmt   Format string ala printf(), followed by an arbitrary number of
		 *              parameters.
		 */
		#ifdef MYSDL_DEBUG_DEBUG
			void debug(int group,const char *fmt,...);
		#else
			inline void debug(int group,const char *fmt,...)
			{
			}
		#endif

		//void message(DebugLevel level,int group,const char *fmt,...);

		#ifdef MYSDL_DEBUG_STACK
			/**
			 * Adds a debug message to the debug message stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will be implemented as an empty inline message
			 * which does nothing.
			 *
			 * @param msg Debug message.
			 */
			//void addMessage(const std::string msg);

			/**
			 * Removes the newest debug message from the stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will be implemented as an empty inline message
			 * which does nothing.
		 	*/
			//void removeMessage();

			/**
			 * Reads the newest debug message from the stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will return a simple string informing the
			 * user that the debug stack subsystem is disabled.
			 *
			 * @returns Newest debug message.
			 */
			//const std::string getLatestMessage();

			/**
			 * Reads and removes the newest debug message from the stack. If
			 * MYSDL_DEBUG_STACK is not set at compile time, this will be implemented as
			 * an empty inline message which returns <code>false</code>.
			 *
			 * @param s Will contain the debug message.
			 *
			 * @returns <code>true</code> if the stack contained a
			 *          debug message. If <code>false</code> is
			 *          returned, s was not changed.
			 */
			//bool popLatestMessage(std::string &s);
		#else
			/**
			 * Adds a debug message to the debug message stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will be implemented as an empty inline message
			 * which does nothing.
			 *
			 * @param msg Debug message.
			 */
			/*inline void addMessage(const std::string msg)
			{
	}*/

			/**
			 * Removes the newest debug message from the stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will be implemented as an empty inline message
			 * which does nothing.
		 	*/
			/*inline void removeMessage()
			{
	}*/

			/**
			 * Reads the newest debug message from the stack. If MYSDL_DEBUG_STACK is
			 * not set at compile time, this will return a simple string informing the
			 * user that the debug stack subsystem is disabled.
			 *
			 * @returns Newest debug message.
			 */
			/*inline const std::string getLatestMessage()
			{
				return(std::string("MySDL Debug Stack Subsystem disabled."));
	}*/

			/**
			 * Reads and removes the newest debug message from the stack. If
			 * MYSDL_DEBUG_STACK is not set at compile time, this will be implemented as
			 * an empty inline message which returns <code>false</code>.
			 *
			 * @param s Will contain the debug message.
			 *
			 * @returns <code>true</code> if the stack contained a
			 *          debug message. If <code>false</code> is
			 *          returned, s was not changed.
			 */
			/*inline bool popLatestMessage(std::string &s)
			{
				return(false);
	}*/
		#endif

		/** TODO: do we really need profiling stuff here? */
		#ifdef MYSDL_DEBUG_PROFILE
			void beginProfileSection(const std::string name);
			void endProfileSection();
			void printProfileStats();
			void beginProfileLoop();
			void endProfileLoop(bool incomplete=false);
			void clearProfileStats();
		#else
			inline void beginProfileSection(const std::string name)
			{
			}

			inline void endProfileSection()
			{
			}

			inline void printProfileStats()
			{
			}

			inline void beginProfileLoop()
			{
			}

			inline void endProfileLoop(bool incomplete=false)
			{
			}

			inline void clearProfileStats()
			{
			}
		#endif
	};
};

#endif
