/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_FILES_H
#define MYSDL_FILES_H

#include <string>

namespace mysdl
{
	/** Application categories. A program's category might have an effect on
	 *  the data directory paths. */
	enum SysDirCategory {sdApplication,sdGame};

	/**
	 * Available system directory types:
	 *
	 * @enum sdConfig    System-wide configuration directory, may be read-only.
	 * @enum sdUser      User's private configuration directory. Should be writeable.
	 * @enum sdConstData System-wide data directory for the application's resource files.
	 *                   May be read-only.
	 * @enum sdData      System-wide data directory for global application files (e.g.
	 *                   highscore lists). Should be writeable.
	 */
	enum SysDir {sdConfig,sdUser,sdConstData,sdVarData};

	/**
	 * Tries to create the specified directory. This also creates missing
	 * parent directories.
	 *
	 * @return <code>true</code> on success.
	 */
	bool mkdirs(std::string path);

	/**
	 * A class for retrieving system-specific paths for the application's data
	 * directories. Currently, implementations for Windows and Linux are provided.
	 */
	class SysDirManager
	{
		private:
			/** Name of the application/the application's subdirectories. */
			std::string applicationName;

			/** Category used for the application. */
			SysDirCategory category;

		public:
			/** If this is set, it will be returned as the dirConstData directory, instead
			 *  of the platform's correct system directory for constant application data. 
			 *  MUST include a trailing backslash. */
			std::string constDataDir;
			
			/**
			 * Creates a system directory manager.
			 *
			 * @param applicationName Name of the application. This will be used
			 *                        for the appication's sub directory inside the
			 *                        retrieved system directories.
			 * @param category        Category for the application.
			 */
			SysDirManager(std::string applicationName,SysDirCategory category);

			/**
			 * Retrieves the path to the application's directory in the specified system
			 * directory. The category and application sub directories will automatically
			 * be appended to this path.
			 *
			 * @param type System directory type.
			 *
			 * @return Path to the requested directory.
			 */
			std::string getDir(SysDir type);

			/**
			 * Retrieves the path to a sub directory inside the application's directory
			 * in a system directory.
			 *
			 * @param type   System directory type.
			 * @param subDir Sub directory inside the application's specified system
			 *               directory. Set this to "" to get the application's "root"
			 *               directory of the specified type.
			 * @param create If this is set to <code>true</code>, the manager will try to
			 *               create the specified directory if it does not exist.
			 *
			 * @return Path to the requested directory.
			 */
			std::string getSubDir(SysDir type,std::string subDir="",bool create=false);
	};
};

#endif
