/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_FRAMEWORK_H
#define MYSDL_FRAMEWORK_H

#include <string>
#include <list>
#include <map>

#include "controls.h"

namespace mysdl
{	
	/**
	 * An application's current functionality is defined by its state. An
	 * application may switch between several states (e.g. a menu state and
	 * a game state for a video game), where each state implements everything
	 * that happens as long as it is active.
	 *
	 * During each run through the application's main loop, the application first
	 * handles all pending events (see the documentation on ControlQueues for event
	 * handling details) and then calls the current state's doPerform() method,
	 * followed by its doDraw() method. These are the methods that implement everything
	 * that happens during the state.
	 *
	 * Some application may require common behaviour through different states and
	 * therefore implement a common base class for these states. For example, a space
	 * video game might want to display a starfield in the background both during the
	 * actual game and in its menu. In this case, both the menu and the game state need
	 * to update the starfield data during the doPerform() step and draw the starfield
	 * during doDraw().
	 *
	 * Since this may create more complex scenarios where a base state needs to
	 * perform two different actions while a derived state needs to perform one or
	 * more actions between the base state's actions, both doPerform() and doDraw()
	 * support a multi-stage concept. Each method will be executed as many times as
	 * the performStageCount and drawStageCount variables specify, receiving the
	 * current stage as an integer parameter.
	 */
	class ApplicationState
	{
		protected:
			/** If this is set to true, the state will be deleted during the next
			 *  call to perform(). */
			bool terminate;

			/** The number of perform stages to execute. */
			int performStageCount;

			/** The number of draw stages to execute. */
			int drawStageCount;

		public:
			/**
			 * Creates a new ApplicationState instance.
			 */
			ApplicationState();

			/**
			 * Deletes an application state.
			 */
			virtual ~ApplicationState();

			/**
			 * Called when this state is activated.
			 */
			virtual void activate();

			/**
			 * Called when this state is deactivated.
			 */
			virtual void deactivate();

			/**
			 * Handles an event received through the application's
			 * message queue.
			 *
			 * @param e Event.
			 */
			virtual void handleEvent(const Trigger *e)=0;

			/**
			 * Called to perform the state's functionality. This method
			 * is called once per run through the application's main
			 * loop.
			 *
			 * @param ticks The number of ticks since the previous call
			 *              to this method.
			 */
			virtual void perform(Uint32 ticks);

			/**
			 * Called to draw the state. This method is called once per
			 * run through the application's main loop (after perform()).
			 */
			virtual void draw();

		private:
			/**
			 * Called by perform() to perform the state's functionality.
			 * Each time perform() is executed, it calls this method as
			 * many times as <code>performStageCount</code> requests,
			 * passing the current stage number as a parameter.
			 *
			 * Override this method to implement your application state's
			 * logic.
       *
			 * @param stage Current stage.
			 * @param ticks The number of ticks since the previous call
			 *              to the perform() method.
			 */
			virtual void doPerform(int stage,Uint32 ticks)=0;

			/**
			 * Called by draw() to perform the state's functionality.
			 * Each time draw() is executed, it calls this method as
			 * many times as <code>drawStageCount</code> requests,
			 * passing the current stage number as a parameter.
			 *
			 * Override this method to implement your application state's
			 * drawing functionality.
       *
			 * @param stage Current stage.
			 */
			virtual void doDraw(int stage)=0;
	};
		
	/**
	 * A base class for applications. TODO: more?
	 */
	class Application
	{
		friend class ApplicationState;

		protected:
			/** Current state. */
			ApplicationState *state;

			/** Message queue for the application. */
			ControlQueue *queue;

			/**
			 * First reads all events into the message queue, then handles
			 * all pending even messages from the specified queue by
			 * calling the current state's handleEvent() method for each
			 * event.
			 *
			 * If a quit event is discovered, the current state is deleted.
			 *
			 * @param queue Control queue. TODO: why a parameter?
			 */
			virtual void handleEvents(ControlQueue *queue);

			/**
			 * Handles events, then performs and draws the current state.
			 *
			 * @param ticks Ticks since the previous call to this method.
			 */
			virtual void perform(Uint32 ticks);

		public:
			/**
			 * Creates a new application instance. No two Application
			 * instances should exist at the same time (for various
			 * reasons).
			 */
			Application();

			/**
			 * Deletes the application and terminates the program.
			 */
			virtual ~Application();

			/**
			 * Executes the application's main loop. The application
			 * will terminate when the current state becomes null.
			 */
			virtual void run();
	};

	/**
	 * An application class that limits the frame rate. Frame rates lower than a
	 * specified threshold value are treated like the treshold, while too fast
	 * framerates are compensated by application delays, so as to leave enough
	 * time to other programs and prevent the operating system from pausing this
	 * application at critical moments.
	 */
	class FrameRateLimitedApplication : public Application
	{
		protected:
			/**
		 	 * Handles events, then performs and draws the current state.
		 	 *
		 	 * @param ticks Ticks since the previous call to this method.
			 */
			virtual void perform(Uint32 ticks);
		
		public:
			/** Maximum ticks per frame. If this is not 0 and the current ticks exceed
			 *  this value, they are treated like the maximum. */
			Uint32 maxTicks;
			
			/** Desired ticks per frame. If this is not 0 and the current ticks are
			 *  lower, a delay is introduced. */
			Uint32 desiredTicks;
			
			/**
			 * Creates a new application instance. No two Application
			 * instances should exist at the same time (for various
			 * reasons).
			 */
			FrameRateLimitedApplication();
	};
		
	class FixedRateApplication : public Application
	{
		protected:
			Uint32 tickBuffer;
			
			virtual void perform(Uint32 ticks);
		
		public:
			Uint32 fixedTicks;
			
			FixedRateApplication();
	};
		
	extern Application *application; // TODO: change to getter method, to prevent changing?
	
	/**
	 * A macro to create a main() function which automatically
	 * creates and executes a custom application class.
	 */
	#define MYSDL_FRAMEWORK(ApplicationClass) \
	ApplicationClass *application; \
	int main(int argc,char **argv) \
	{ \
		try \
		{ \
			application=new ApplicationClass(); \
			application->run(); \
			delete application; \
			application=0; \
		} \
		catch(std::string ex) \
		{ \
			std::cerr << ex << std::endl; \
		} \
		assert(0); \
	}
};

#endif
