/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_ANIMATIONS_H
#define MYSDL_ANIMATIONS_H

#include <string>
#include <vector>

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include "images.h"

namespace mysdl
{
	namespace gfx
	{
		class Animation;
		class AnimationBank;

		/**
		 * Represents one frame in an animation.
		 */
		class AnimationFrame
		{
			public:
				/** The image represented by the frame. */
				Image *image;

				/** For how many ticks the image shall be displayed. */
				GLfloat delay;

				/**
				 * Creates a new AnimationFrame instance.
				 */
				AnimationFrame();

				/**
				 * Creates a new AnimFrame.
				 *
				 * @param image The image represented by the frame.
				 * @param delay For how many ticks the image shall be displayed.
				 */
				AnimationFrame(Image *image,GLfloat delay);
		};

		/**
		 * An animation reference is an instance of one animation. To play an
		 * animation, an AnimationPointer must be created from it by calling the
		 * animation's getReference() method. The animation referencecontains the
		 * current state of animation playback, making it possible to create several
		 * independent references to one animation and thus playing several individual
		 * instances of the animation (which do not have to by synched in time)
		 * simultaneously.
		 */
		class AnimationReference : public Image
		{
			protected:
				/** Current frame. */
				AnimationFrame *frame;

				/** Image referenced by the current frame. TODO: ? */
				Image *image;

			public:
				/** Animation instanced by this AnimationPointer. */
				Animation *animation;

				/** Playback speed factor. The original animation's delays are
				 *  multiplied by this factor. */
				GLfloat speed;

				/** Current frame index. */
				Uint32 index;

				/** TODO: ?? */
				GLfloat timeIndex;

				/** Index of the frame to which the animation will proceed over
				 *  time. Depending on its relative position to the current frame,
				 *  the animation will be played forward or backward. If the
				 *  index lies outside of the range of valid frame indexes, the
				 *  animation will wrap and loop when it reaches the end (or beginning,
				 *  in case it is proceeding backward). Otherwise, the animation will
				 *  stop when the target frame is reached. */
				Uint32 target;

				/**
				 * Creates a new animation pointer. TODO: Do we need this?
				 */
				AnimationReference();

				/**
				 * Creates a new animation pointer for an animation.
				 *
				 * @param animation Pointer to the animation that will be instanced
				 *                  by the animation pointer.
				 * @param speed     Speed factor (1=original speed).
				 */
				AnimationReference(Animation *animation,GLfloat speed=1);

				/**
				 * Sets the animation's current frame.
				 *
				 * @param frame New current frame.
				 */
				void setFrame(AnimationFrame *frame);

				/**
				 * Advances the animation by a specified amount of ticks.
				 *
				 * @param ticks Number of ticks.
				 */
				void perform(GLfloat ticks);


				/**
				 * Draws the animation at the specified coordinates.
				 *
				 * @param pos Destination coordinates.
				 */
				void draw(const Vector2D &pos,BlendMode blendMode=bmDefault) const;

				/**
				 * Draws the current frame in the specified rectangle. If the
				 * frame is larger than the rectangle, it will be clipped.
				 *
				 * @param pos Destination rectangle.
				 */
				void draw(const Rect &pos,BlendMode blendMode=bmDefault) const;

				/**
				 * Draws the current frame in the specified rectangle, stretching
				 * it to fit the rectangle exactly.
				 *
				 * @param pos Destination rectangle.
				 */
				void drawStretched(const Rect &pos,BlendMode blendMode=bmDefault) const;

				/**
				 * Determines whether the animation has finished, i.e. has
				 * reached the frame specified by <code>target</code>.
				 *
				 * @return <code>true</code> if the animation has finished.
				 */
				bool isFinished();
		};

		/**
		 * Stores a sequential list of animation frames, creating an animation.
		 */
		class Animation : public NamedResource<AnimationBank>
		{
			friend class AnimationReference;

			protected:
				/** Sequential list of frames. */
				std::vector<AnimationFrame> frames;

			public:
				/** Default (?) delay factor for frames in this animation. TODO: used? */
				GLfloat delay;

				/** Wheter to loop the animation when it has finished playing. If
				 *  this is set to <code>false</code>, the animation will stop at
				 *  the last frame and the last image will then be displayed
				 *  permanently. */
				bool loop;

				/**
				 * Creates a new animation.
				 */
				Animation();

				/**
				 * Creates a new animation.
				 *
				 * @param name The name for the animation, must be unique within
				 *             the animation bank.
				 */
				Animation(const std::string &name);

				/**
				 * Reads the contents for this animation from an XML node.
				 *
				 * @param root   XML node.
				 * @param images Image bank containing the images refernced by the animation
				 *               data stored in the XML node.
				 * @param loops  List of currently active loops. Used for recursive calls, pass an empty list
				 *               when calling manually.
				 */
				void getFromNode(const xml::Node *root,ImageBank *images,std::list<mysdl::xml::LoopData *> loops);

				/**
				 * Obtains a reference to this animation, which can then be used to perform and
				 * draw the animation.
				 *
				 * @param delay Delay factor which will be used then playing the animation reference.
				 *              TODO: is this used?
				 *
				 * @return Animation reference. This <b>must</b> be deleted when it is no longer
				 *         used.
				 */
				AnimationReference *getReference(GLfloat delay=1);
		};

		/**
		 * A named resource bank containing animations.
		 */
		class AnimationBank : public NamedResourceBank<Animation>
		{
			protected:
				/**
				 * Reads the contents of the animation bank from an XML node.
				 * Called by getFromNode().
				 *
				 * @param root   XML node.
				 * @param images Image bank containing the images referenced by
				 *               the animations stored in the XML node.
				 */
				void getContentsFromNode(const xml::Node *root,ImageBank *images);

			public:
				/**
				 * Deletes the animation bank and all its contents.
				 */
				~AnimationBank();

				/**
				 * Reads the contents of the animation bank from an XML node.
				 *
				 * @param root   XML node.
				 * @param images Image bank containing the images referenced by
				 *               the animations stored in the XML node. If the
				 *               XML data specifies the name of an image bank, and
				 *               this animation bank is managed by a resource manager,
				 *               the image bank specified here will be overriden.
				 */
				void getFromNode(const xml::Node *root,ImageBank *images);

				/**
				 * Reads the contents of the animation bank from an XML node. To
				 * determine the images referenced by the animations, the XML data
				 * must include the name of the image bank, and that image bank
				 * must be managed by the same resource manager as this animation
				 * bank.
				 *
				 * @param root    XML node.
				 * @param baseDir Ignored. TODO: necessary?
				 */
				void getFromNode(const xml::Node *root,const std::string &baseDir="");

				/**
				 * Returns the name of XML nodes containing animation data.
				 */
				std::string getNodeName();
		};
	};
};

#endif
