/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MSCHED_BITMASKS_H
#define MSCHED_BITMASKS_H

#include "SDL/SDL.h"

#include "../mysdl.h"
#include "../tools.h"

namespace mysdl
{
	namespace gfx
	{
		/**
		 * Releases the cache used for calculating bitmask bounding
		 * boxes. Automatically called by Bitmask's constructor if
		 * keepCache is <code>false</code>.
		 */
		void clearBitmaskCache();

		/** Type of atomic bitmask values. All the bits in an atom will be tested against
		*  those of another atom with a single statement. An atom should therefore have
		*  the type the CPU uses natively for bit operations. */
		typedef int atom;

		/** Number of bits within an atom. */
		const int BITWIDTH=sizeof(atom)*8;

		/** Exponent of 2 for the MSB of an atom. */
		const int MSB=BITWIDTH-1;

		/**
		* A bitmask stores a boolean pixel mask for an object. A 1 bit within the mask
		* marks a pixel that is occupied by the object, while a 0 bit marks a pixel that
		* is beneath the object. Bitmasks are used for exact collision tests of sprites.
		*/
		class Bitmask
		{
			public:
				/** Width of bitmask (pixels/bits per line) */
				Sint32 w;

				/** Height of bitmask (number of lines) */
				Sint32 h;

				/** Array of atomic bit chunks. */
				atom *bits;

				/** Size of the bits array. */
				Uint32 size;

				/** Specifies the bitmask's outer bounding box. Outside this box, no pixels are
				 *  "set". Two bitmasks can only collide if their outer bounding boxes overlap. */
				RectInt outerBoundingBox;

				/** Specifies the bitmask's inner bounding box. Within this box, all pixels are
				 *  considered to be "set". When two bitmasks' inner bounding boxes overlap, they
				 *  are colliding. */
				RectInt innerBoundingBox;

				/**
				 * Creates a new bitmask. All pixels in the new bitmask will be unset.
				 *
				 * @param w Bitmask width (pixels/bits per line).
				 * @param h Bitmask height (number of lines).
				 */
				Bitmask(Sint32 w,Sint32 h);

				/**
				 * Creates a new bitmask from the specified surface.
				 *
				 * @param surface   Surface for which a bitmask will be created. Currently, the surface must
				 *                  be in either 24 or 32 bit format and include an alpha channel.
				 * @param threshold Pixels with a normalised alpha value greater or equal to this threshold
				 *                  will be set in the bitmask, all other pixels will be cleared.
				 * @param keepCache If this is <code>true</code>, the cache used for calculating the bitmask's
				 *                  bounding boxes will be kept after the constructor finished. In this case,
				 *                  it must be released later, either by another call to Bitmask's constructor
				 *                  which does not keep the cache, or directly through clearBitmaskCache().
				 */
				Bitmask(SDL_Surface *surface,GLfloat threshold,bool keepCache);

				/**
				 * Releases a bitmask.
				 */
				~Bitmask();

				/**
				 * Clears the bitmask by setting all bits to 0.
				 */
				void clear();

				/**
				 * Prints the bitmask to stdout. For debugging purposes only.
				 */
				void print();

				/**
				 * Returns whether the pixel at a specified location in the bitmask is set.
				 *
				 * @param x X coordinate.
				 * @param y Y coordinate.
				 *
				 * @return <code>true</code> if the pixel is set.
				 */
				bool at(Sint32 x,Sint32 y);

				/**
				 * Returns the bitmask's outer bounding box.
				 *
				 * @return Outer bounding box.
				 */
				const RectInt *getOuterBoundingBox();

				/**
				 * Returns the bitmask's inner bounding box.
				 *
				 * @return Outer bounding box.
				 */
				const RectInt *getInnerBoundingBox();
		};

		/**
		 * Tests for collision between two bitmasks.
		 *
		 * @param a First bitmask.
		 * @param b Second bitmask.
		 * @param xOffset Specifies how many pixels the second bitmask is to the right of the first one.
		 * @param yOffset Specifies how many pixels the second bitmask is below the first one.
		 *
		 * @return <code>true</code> if a collision is detected.
		 */
		bool checkCollisionBitmask(Bitmask *a,Bitmask *b,Sint32 xOffset,Sint32 yOffset);

		/**
		 * Prints the overlapping bitmasks for a collision test. For debugging purposes only. For
		 * paramter description, see checkCollisionBitmask.
		 */
		void printCollisionBitmask(Bitmask *a,Bitmask *b,Sint32 xOffset,Sint32 yOffset);
	};
};

#endif
