/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_FONT_H
#define MYSDL_FONT_H

#include <string>

#include "../debug.h"
#include "../mysdl.h"
#include "../resources.h"
#include "../xml.h"

#include "images.h"

namespace mysdl
{
	/**
	 * Provides font classes and helpers for graphical text output.
	 */
	namespace fonts
	{
		/**
		 * Font style bits, can be combined to form a bitmask.
		 *
		 * @enum fsPlain      Plain text.
		 * @enum fsBold       Bold text.
		 * @enum fsItalic     Italic text.
		 * @enum fsUnderlined Underlined text.
		 */
		enum FontStyle {fsPlain=0,fsBold=1,fsItalic=2,fsUnderlined=4};

		/**
		 * Used to uniquely identify a specific font.
		 */
		class FontDescriptor
		{
			public:
				/** Name of the font's family. */
				std::string family;

				/** The font's size. No specific measurement unit is defined. */
				unsigned int size;

				/** The font's style. */
				FontStyle style;

				/**
				 * Creates a new, empty font descriptor.
				 */
				FontDescriptor();

				/**
				 * Creates a new font descriptor.
				 *
				 * @param family Font family.
				 * @param size   Font size.
				 * @param style  Font style.
				 */
				FontDescriptor(std::string family,unsigned int size=0,FontStyle style=fsPlain);

				/**
				 * Compares two font descriptors.
				 *
				 * @param d Second font descriptor.
				 *
				 * @return <code>true</code> if this font is "smaller" than the
				 *         specified font.
				 */
				bool operator<(const FontDescriptor &d) const;
		};

		class Font;

		/**
		 * A bank containing fonts.
		 */
		class FontBank;

		/**
		 * Abstract base class for fonts.
		 */
		class Font : public NamedResource<FontBank,FontDescriptor>
		{
			public:
				/** Vertical hotspot. */
				Uint8 yHotSpot;

				/**
				 * Creates a new font.
				 */
				Font();

				/**
				 * Releases the font.
				 */
				virtual ~Font();

				/**
				 * Gets the height of the font's characters.
				 *
				 * @return Character height.
				 */
				virtual Uint8 getHeight()=0;

				/**
				 * Calculates the width of a string when printed using this font.
				 *
				 * @return String width.
				 */
				virtual Uint16 getWidth(const char *s)=0;

				/**
				 * Prints a string using left justification.
				 *
				 * @param s Text string.
				 */
				virtual void print(const char *s)=0;

				/**
				 * Prints a string using the specified horizontal justification.
				 *
				 * @param jh Horizontal text justification.
				 * @param s  Text string.
				 */
				void print(JustificationHorizontal jh,const char *s);

				/**
				 * Prints a formatted string using left justification.
				 *
				 * @param fmt This and the following parameters specify the string to be
				 *            printed, using the same syntax as printf from stdlib.h
				 */
				void printf(const char *fmt,...);

				/**
				 * Prints a formatted string using the specified horizontal justification.
				 *
				 * @param jh  Horizontal text justification.
				 * @param fmt This and the following parameters specify the string to be
				 *            printed, using the same syntax as printf from stdlib.h
				 */
				void printf(JustificationHorizontal jh,const char *fmt,...);

				/**
				 * Prints a string in a rectangle, using the specified justification.
				 *
				 * @param rect Destination rectangle.
				 * @param jh   Horizontal text justification within the rectangle.
				 * @param jv   Vertical text justification within the rectangle.
				 * @param s    Text string.
				 */
				void print(const mysdl::Rect &rect,JustificationHorizontal jh,JustificationVertical jv,const char *s);

				/**
				 * Prints a formatted string in a rectangle, using the specified justification.
				 *
				 * @param rect Destination rectangle.
				 * @param jh   Horizontal text justification within the rectangle.
				 * @param jv   Vertical text justification within the rectangle.
				 * @param fmt  This and the following parameters specify the string to be
				 *             printed, using the same syntax as printf from stdlib.h
				 */
				void printf(const mysdl::Rect &rect,JustificationHorizontal jh,JustificationVertical jv,const char *fmt,...);

				/**
				 * Refreshes a font, by first releasing and then re-loading its data.
				 */
				virtual void refresh();

				/**
				 * Releases a font's data. This is an abstract method that must be overridden by
				 * derived classes.
				 */
				virtual void release()=0;

				/**
				 * Loads a font, e.g. by reading its font face images from a file system.
				 * This is an abstract method that must be overridden by derived classes.
				 */
				virtual void load()=0;
		};

		/**
		 * A font that generates its characters from one or more bitmap images.
		 * For displaying the font, OpenGL display lists are used; each characters
		 * has its own display list drawing a quad with the corresponding portion
		 * of the image file used as a texture.
		 */
		class BitmapFont : public Font
		{
			private:
				/** Directory containing the font's files. */
				std::string dir;

				/** ASCII codes of the first and last character in the font. */
				Uint8 firstChar;

				/** Number of characters in the font. */
				int charCount;

				/** Height of each character. */
				Uint8 charHeight;

				/** Width of the space between two characters. */
				Uint8 spacing;

				/** Index of the OpenGL list that stores the characters. */
				GLuint listBase;

				/** Character widths, beginning with the first character as specified by
				 *  <code>firstChar</code>. */
				Uint8 *widths;

				/** Points to the font image, if that image is the only image used by the font. */
				mysdl::gfx::SingleTextureImage *onlyImage;

			public:
				/** Stores images used for the font. */
				mysdl::gfx::ImageBank *images;

				/**
				 * Creates a new BitmapFont and loads it from the specified directory.
				 * The font descriptor file in this directory must be called
				 * <code>bitmapfont.xml</code>.
				 *
				 * @param dir Font directory.
				 */
				BitmapFont(const std::string dir);

				/**
				 * Frees the font.
				 */
				virtual ~BitmapFont();

				/**
				 * Loads the font from its directory.
				 */
				void load();

				/**
				 * Reads the font from the specified DOM node.
				 *
				 * @param root DOM node containing font.
				 * @param dir  Directiory containing font's files.
				 */
				void getFromNode(const xml::Node *root,const std::string dir);

				/**
				 * Gets the height of the font's characters.
				 *
				 * @return Character height.
				 */
				Uint8 getHeight();

				/**
				 * Calculates the width of a string when printed using this font.
				 *
				 * @return String width.
				 */
				Uint16 getWidth(const char *s);

				/**
				 * Prints a string using left justification.
				 *
				 * @param s Text string.
				 */
				void print(const char *s);

				/**
				 * Releases the font's data, including all its images.
				 */
				void release();
		};

		/**
		 * A mapped bank containing fonts.
		 */
		class FontBank : public NamedResourceBank<Font>
		{
			public:
				/**
				 * Release the bank along with all the fonts it contains.
				 */
				virtual ~FontBank();

				/**
				 * Returns the tag name for DOM nodes containing a font bank.
				 */
				std::string getNodeName();

				/**
				 * Reads the bank's fonts from the specified DOM node.
				 *
				 * @param root    DOM node containing font data.
				 * @param baseDir Base directory for fonts. Directories that may
				 *                be specified by the various fonts are relative
				 *                to this directory.
				 */
				void getFromNode(const xml::Node *root,const std::string &baseDir);

				/**
				 * Refreshes all fonts stored in the bank.
				 */
				void refresh();
		};

		/**
		 * A text message that can be displayed as part of a list of messages.
		 */
		class Message
		{
			public:
				/** Font to be used when displaying this message. */
				Font *font;

				/** The message text. */
				std::string text;

				/** Time to live. After this number of ticks, the message expires and will be
				 *  deleted. */
				GLfloat ttl;

				/**
				 * Creates a new, empty message.
				 */
				Message();

				/**
				 * Creates a new message.
				 *
				 * @param text Message text.
				 * @param ttl  Time to live. After this number of ticks, the message expires
				 *  					 and will be deleted.
				 */
				Message(std::string text,GLfloat ttl);

				/**
				 * Releases a message.
				 */
				virtual ~Message();

				/**
				 * Informs the message that the specified number of ticks have elapsed.
				 *
				 * @param ticks Number of ticks since the last call to perform().
				 */
				virtual void perform(GLfloat ticks);

				/**
				 * Draws the message in the specified rectangle, using the specified
				 * justification.
				 *
				 * @param rect Destination rectangle.
				 * @param jh   Horizontal justification within the rectangle.
				 * @param jv   Vertical justification within the rectangle.
				 */
				virtual void draw(Rect rect,JustificationHorizontal jh,JustificationVertical jv);
		};

		/**
		 * A blinking text message. Same as a simple Message, but flashes its text
		 * in regular intervals.
		 */
		class BlinkMessage : public Message
		{
			private:
				/** Current blink state counter. */
				GLfloat blinkCounter;

			public:
				/** Whether the message is currently visible (=blink state). */
				bool visible;

				/** Number of ticks within one blink phase during which the message
				 *  shall be displayed. */
				GLfloat blinkDurationOn;

				/** Number of ticks within one blink phase during which the message
				 *  shall be hidden. */
				GLfloat blinkDurationOff;

				/**
				 * Creates a new, empty blinking message.
				 */
				BlinkMessage();

				/**
				 * Creates a new blinking message.
				 *
				 * @param text             Message text.
				 * @param ttl              Time to live. After this number of ticks, the message
				 *                         expires and will be deleted.
				 * @param blinkDurationOn  Number of ticks within one blink phase during which the
				 *												 message shall be displayed.
				 * @param blinkDurationOff Number of ticks within one blink phase during which the
				 *												 message shall be hidden. Set this to a negative value if
				 *                         you want to use the same value as for blinkDurationOn.
				 */
				BlinkMessage(std::string text,GLfloat ttl,GLfloat blinkDurationOn,GLfloat blinkDurationOff=-1);

				/**
				 * Informs the message that the specified number of ticks have elapsed.
				 *
				 * @param ticks Number of ticks since the last call to perform().
				 */
				void perform(GLfloat ticks);

				/**
				 * Draws the message in the specified rectangle, using the specified
				 * justification.
				 *
				 * @param rect Destination rectangle.
				 * @param jh   Horizontal justification within the rectangle.
				 * @param jv   Vertical justification within the rectangle.
				 */
				void draw(Rect rect,JustificationHorizontal jh,JustificationVertical jv);
		};

		/**
		 * A list of messages which are displayed at a common location.
		 */
		class Messages
		{
			private:
				/** List of messages. */
				std::list<Message *> messages;

			public:
				/** Rectangle in which messages are displayed. */
				Rect rect;

				/** Default font for messages which don't have a font assigned. */
				Font *font;

				/** Whether to allow more than one message. If this is set to false, every
				 *  new message will remove all previous messages. */
				bool multiple;

				/**
				 * Creates a new list of messages.
				 */
				Messages();

				/**
				 * Frees the list of messages.
				 */
				~Messages();

				/**
				 * Adds a messages. The message list takes over ownership of the specified
				 * message and automatically frees the message when it expires.
				 */
				void operator+=(Message *message);

				/**
				 * Informs the messages that the specified number of ticks have elapsed.
				 *
				 * @param ticks Number of ticks since the last call to perform().
				 */
				void perform(GLfloat ticks);

				/**
				 * Draws the messages.
				 */
				void draw();

				/**
				 * Removes all messages from the list.
				 */
				void clear();

				/**
				 * Removes all expired messages from the list.
				 */
				void purge();
		};
	};
};

#endif
