/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_IMAGES_H
#define MYSDL_IMAGES_H

#include <string>
#include <map>

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include "../math.h"
#include "../mysdl.h"
#include "../resources.h"
#include "../xml.h"

#include "bitmasks.h"

namespace mysdl
{
	namespace gfx
	{
		class ImageResource;
		class ImageBank;

		/** Specifies an image filtering algorithm. */
		enum ImageFilter {ifNone=0,ifLinear,ifBilinear,ifTrilinear};

		/** String representations of the ImageFilter enum. */
		static const std::string STR_IMAGEFILTER[]={"none","linear","bilinear","trilinear"};

		/**
		 * Different blend modes for drawing sprites.
		 *
		 * @param bmDefault No specific blend mode, used to specify that a sprite shall
		 *                  use the default blend mode set in its engine.
		 *
		 * @param bmNormal  Pixels are drawn using their alpha value for the source
		 *                  and their complementary alpha value for the destination pixel.
		 *
		 * @param bmAdd     Pixels are drawn using their alpha value for the source
		 *                  and 1 for the destination pixel's alpha.
		 */
//		enum BlendMode {bmDefault,bmNormal,bmAdd};

		/**
		 * Sets OpenGL to use the specified blend mode.
		 *
		 * @param mode Desired blend mode.
		 */
// 		void setBlendMode(BlendMode mode);

		class ImageReference;

		class Image
		{
			protected:
				/** Image dimensions. \todo Make Vector2D? */
				Uint32 width,height;

				/** Hotspot position. */
				Vector2D hotspot;

				/** Bitmask (0 if no bitmask was generated). */
				Bitmask *bitmask;

				/** Specifies the image's outer bounding box. Outside this box, no pixels are
				 *  "set". Two images can only collide if their outer bounding boxes overlap. */
				RectInt outerBoundingBox;

				/** Specifies the image's inner bounding box. Within this box, all pixels are
				 *  considered to be "set". When two images' inner bounding boxes overlap, they
				 *  are colliding. */
				RectInt innerBoundingBox;

			public:
				Image();
				virtual ~Image();

				virtual void perform(GLfloat ticks);
				Uint32 getWidth() const;
				Uint32 getHeight() const;
				Bitmask *getBitmask() const;
				const RectInt *getOuterBoundingBox();
				const RectInt *getInnerBoundingBox();
				void setHotspot(Vector2D &hotspot);
				const Vector2D &getHotspot() const;

				/**
				 * Draws the image at the specified coordinates.
				 *
				 * @param pos Destination coordinates.
				 */
				virtual void draw(const Vector2D &pos,BlendMode blendMode=bmDefault) const=0;

				/**
				 * Draws the image in the specified rectangle. If the image
				 * is larger than the rectangle, it will be clipped. If the image
				 * is smaller than the rectangle, it will be repeated to completely
				 * fill the rectangle.
				 *
				 * @param pos Destination rectangle.
				 */
				virtual void draw(const Rect &pos,BlendMode blendMode=bmDefault) const=0;

				/**
				 * Draws the image in the specified rectangle, stretching
				 * it to fit the rectangle exactly.
				 *
				 * @param pos Destination rectangle.
				 */
				virtual void drawStretched(const Rect &pos,BlendMode blendMode=bmDefault) const=0;
		};

		/**
		 * Holds a graphical image and associated data.
		 */
		class ImageResource : public NamedResource<ImageBank>,public Image
		{
			friend class ImageBank;
			friend class ImageReference;

			protected:
				/** Image's file name. */
				std::string filename;

				/** Bitmask alpha threshold. Pixels with an alpha value not
				 *  greater than this value will result in 0 bits in the
				 *  bitmask, all other pixels will result in 1 bits. */
				GLfloat bitmaskThreshold;

				// TODO: ?
				bool clamp;

				/** Filtering algorithm used to draw the image. */
				ImageFilter filter;

			public:
				/**
				 * Creates a new ImageResource.
				 */
				ImageResource();

				/**
				 * Creates a new ImageResource... TODO: ??
				 */
				ImageResource(const std::string &name,const std::string &filename="");

				/**
				 * Frees the Image object and all associated data.
				 */
				virtual ~ImageResource();

				/**
				 * Creates a bitmask for the image, according to the
				 * current bitmaskType.
				 *
				 * @param surf ? \todo What is this?
				 * @param last ?
				 */
				void createBitmask(SDL_Surface *surf,bool last);

				/**
				 * Loads the image from the file, using the current
				 * filename.
				 */
				virtual void load(bool last,bool createBitmask)=0;

				/**
				 * Reloads the image from the file.
				 */
				void reload(bool last=false);

				/**
				 * Reloads the image from the file if necessary.
				 */
				virtual void refresh(bool last=false)=0;

				/**
				 * Releases the image (GL texture and bitmask).
				 */
				virtual void release(bool last=false);

				/**
				 * Sets the filtering algorithm that will be used to
				 * draw the image.
				 *
				 * @param filter Filter type.
				 */
				virtual void setFilter(ImageFilter filter)=0;

				/**
				 * Returns the filtering algorithm that will be used to
				 * draw the image.
				 *
				 * @return Filter type.
				 */
				ImageFilter getFilter() const;

				void setFilename(std::string filename);
				std::string getFilename();
				void setBitmaskThreshold(GLfloat bitmaskThreshold);
				GLfloat getBitmaskThreshold();
				void setClamp(bool clamp);
				bool getClamp() const;
		};

		class SingleTextureImage : public ImageResource
		{
			friend class ImageBank;

			protected:
				/** OpenGL texture index. */
				GLuint textureName;
				Rect texCoordRect;

				/** Texture dimensions. */
				Uint32 textureWidth,textureHeight;

			public:
				SingleTextureImage();
				SingleTextureImage(const std::string &name,const std::string &filename="");

				GLuint getTextureName() const;
				GLfloat getLeftTexCoord() const;
				GLfloat getRightTexCoord() const;
				GLfloat getUpperTexCoord() const;
				GLfloat getLowerTexCoord() const;
				const Rect &getTexCoordRect() const;

				/**
				 * Draws the image at the specified coordinates.
				 *
				 * @param pos Destination coordinates.
				 */
				void draw(const Vector2D &pos,BlendMode blendMode=bmDefault) const;

				/**
				 * Draws the image in the specified rectangle. If the image
				 * is larger than the rectangle, it will be clipped.
				 *
				 * @param rect Destination rectangle.
				 */
				void draw(const Rect &rect,BlendMode blendMode=bmDefault) const;

				/**
				 * Draws the image in the specified rectangle, stretching
				 * it to fit the rectangle exactly.
				 *
				 * @param rect Destination rectangle.
				 */
				void drawStretched(const Rect &rect,BlendMode blendMode=bmDefault) const;
		};

		class SimpleImage : public SingleTextureImage
		{
			friend class ImageBank;

			public:
				SimpleImage();
				SimpleImage(const std::string &name,const std::string &filename="");

				/**
				 * Loads the image from the file, using the current
				 * filename.
				 */
				void load(bool last,bool createBitmask);

				/**
				 * Reloads the image from the file.
				 */
				void reload(bool last=false);

				/**
				 * Reloads the image from the file if necessary.
				 */
				void refresh(bool last=false);

				/**
				 * Releases the image (GL texture and bitmask).
				 */
				void release(bool last=false);

				/**
				 * Sets the filtering algorithm that will be used to
				 * draw the image.
				 *
				 * @param filter Filter type.
				 */
				void setFilter(ImageFilter filter);
		};

		/**
		* Holds a list of images that can be loaded from an XML file.
		*/
		class ImageBank : public NamedResourceBank<ImageResource>,public RefreshingBank
		{
			public:
				ImageFilter filter;

				/**
				 * Creates a new, emtpy ImageBank.
				 */
				ImageBank();

				/**
				 * Frees the ImageBank.
				 */
				~ImageBank();

				/**
				 * Reads a list of images from a DOM node, adding them to
				 * the bank.
				 *
				 * @param root		DOM node containing image bank data.
				 * @param baseDir Base directory for image files.
				 */
				void getFromNode(const xml::Node *root,const std::string &baseDir="");

				/**
				 * TODO: ?
				 */
				std::string getNodeName();

				/**
				 * Reloads all images in the bank.
				 */
				void reload();

				/**
				 * Refreshes all images in the bank.
				 */
				void refresh(RefreshingBankHandler *handler);

				/**
				 * Releases all images in the bank.
				 */
				void release();

				/**
				 * Determines the number of images stored in the bank.
				 *
				 * @return Number of images.
				 */
				int size() const;
		};

		/**
		 * A class for loading images in the background.
		 */
		class ImageLoader : public BackgroundTask
		{
			private:
				/** The image bank whose images will be loaded by the loader task. */
				ImageBank *images;

				/** Current image loading progess. */
				ImageBank::iterator itr;

			public:
				/**
				 * Creates a new image loader.
				 *
				 * @param images   The image bank whose images will be loaded.
				 * @param callback Callback object for notifications; may be 0.
				 */
				ImageLoader(ImageBank *images,BackgroundTaskCallback *callback=0);

				/**
				 * Loads as many images as possible within the specified time
				 * limit. Call this method repeatedly until all images are
				 * loaded. Use the time limit to specify how long the current
				 * loading step may take, e.g. to update a menu display while
				 * the application is loading images.
				 * TODO: add return value telling whether loading was complete?
				 *
				 * @param timeLimit System time (in ticks) at which loading
				 *                  shall be interrupted. Before loading each
				 *                  next image, the method checks the current
				 *                  system time. If it is equal to or greater
				 *                  than the value specified here, the method
				 *                  will return.
				 */
				void perform(Uint32 timeLimit);
		};
	};
};

#endif
