/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_CORE_H
#define MYSDL_GUI_CORE_H

#include <list>
#include <string>

#include "../controls.h"
#include "../math.h"
#include "../mysdl.h"
#include "../resources.h"
#include "../xml.h"

#include "../gfx/fonts.h"
#include "../sprites/sprites.h"

namespace mysdl
{
	namespace gui
	{
		class Component;
		class StyleBank;
		class GUI;

		/**
		 * An ordered list of components.
		 */
		typedef std::list<Component *> Components;

		/**
		 * State of a button, can be used as a bit field.
		 *
		 * bsUp: Button is released.
		 * bsDown: Button is pressed.
		 * bsHot: Button is highlighted.
		 */
		enum ButtonState {bsUp=0,bsDown=1,bsHot=2};

		/**
		 * A direction specifier.
		 */
		enum Direction {dirNorth=0,dirEast=1,dirSouth=2,dirWest=3};

		/**
		 * Can be used to specify the orientation of certain components
		 * like slide bars.
		 */
		enum Orientation {orHorizontal,orVertical};

		/**
		 * An interface class for receiving and handling click events.
		 */
		class ClickEventHandler
		{
			public:
				virtual ~ClickEventHandler() {};

				/**
				 * Called when a click event occurs.

				 * @param source Component where the click event happened.
				 */
				virtual void handleClickEvent(Component *source)=0;
		};

		/**
		 * The base class for GUI styles. A style is anything that tells
		 * a component how to draw itself, e.g. by providing colour, font
		 * or texture information. Many components have their own custom
		 * style class.
		 */
		class Style : public NamedResource<StyleBank>
		{
			public:
				/**
				 * Reads a style from a DOM node. Override this when creating
				 * a derived style.
				 *
				 * @param node DOM node containing style data.
				 * @param gui  TODO: ?
				 */
				virtual void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A resource bank containing styles. In a GUI tree, the root
		 * component (the GUI instance) contains an instance of this
		 * class, providing the styles for its child components.
		 */
		class StyleBank : public NamedResourceBank<Style>
		{
		};

		/**
		 * Base class for all GUI components, including the GUI itself.
		 */
		class Component
		{
			protected:
				/** Parent of this component. 0 for the root component (which should be a GUI instance). */
				Component *parent;

				/** Ordered list of child components. */
				Components children;

				/** Whether this component is focused. TODO: what exactly is "focused"? */
				bool focused;

				/** TODO: ? */
				bool wantsFocus;

				/** Name of this component, can be used to find a component when its name is known. */
				std::string name;

				/** Coordinate of lower left corner of the client rectangle. */
				Vector2D pos;

				/** Dimensions of the client rectangle. */
				Vector2D dimensions;

				/** Coordinate of upper left (TODO:?) corner of the area which the component occupies
				 *  in the internal component grid, which is used for cursor navigation. */
				basic_Vector2D<int> gridPos;

				/** Dimensions of the component's area in the internal grid. */
				basic_Vector2D<int> gridDimensions;

				/** Whether this component (along with its children) is visibile. */
				bool visible;

				/** Whether this component (along with its children) is enabled. */
				bool enabled;

				/** Neighbour components, automatically calculated from the components' positions
				 *  in the internal component grid. */
				Component *neighbours[4];

				/** TODO: ? */
				Trigger *triggerMove[4];

			public:
				/** An arbitrary integer value. */
				int data;

				/** An arbitrary string value. */
				std::string dataString;

				/**
				 * Creates a new component instance.
				 */
				Component();

				/**
				 * Releases the component, removing it from its parent's list of children.
				 */
				virtual ~Component();

				/**
				 * Adds a new child component to the end of the children list.
				 *
				 * @param component Component which will be added.
				 */
				void operator+=(Component *component);

				/**
				 * Removes a child component.
				 *
				 * @param component Child which will be removed.
				 */
				void operator-=(Component *component);

				/**
				 * Finds (TODO: recursively?) the child component with the specified name.
				 *
				 * @param name Name of the requested component.
				 *
				 * @return Component with the specified name, or 0 if no such component was found.
				 */
				Component *operator[](const std::string name);

				/**
				 * Converts absolute coordinates into component-relative coordinates.
				 *
				 * @param pos Absolute coordinates.
				 *
				 * @return Relative coordinates.
				 */
				Vector2D getRelativeCoordinates(Vector2D pos);

				/**
				 * TODO: ?
				 */
				Trigger *getTriggerMove(Direction direction)
				{
					return(triggerMove[direction]);
				}

				/**
				 * TODO: ?
				 *
				 * @param direction
				 * @param trigger
				 */
				void setTriggerMove(Direction direction,Trigger *trigger)
				{
					triggerMove[direction]=trigger;
				}

				/**
				 * Sets the name of this component.

				 * @param name New name.
				 */
				void setName(const std::string &name)
				{
					this->name=name;
				}

				/**
				 * Sets the position (relative to the component's parent) and dimensions
				 * of the component's client rectangle.

				 * @param dimensions New position and dimensions.
				 */
				virtual void setDimensions(Rect dimensions)
				{
					this->pos=dimensions.pos;
					this->dimensions=dimensions.dimensions;
				}

				/**
				 * Sets the position (relative to the component's parent) and dimensions
				 * of the component's client rectangle.
				 *
				 * @param pos        New position.
				 * @param dimensions New dimensions.
				 */
				virtual void setDimensions(Vector2D pos,Vector2D dimensions)
				{
					setDimensions(Rect(pos,dimensions));
				}

				/**
				 * Sets the dimensions of the component's client rectangle.
				 *
				 * @param dimensions New dimensions.
				 */
				virtual void setDimensions(Vector2D dimensions)
				{
					setDimensions(Rect(pos,dimensions));
				}

				/**
				 * Shows or hides the component. If a component is invisible,
				 * all its children will effectively be invisible as well,
				 * independent of their own visibility setting.
				 *
				 * @param visible Whether the component shall be visible.
				 */
				virtual void setVisible(bool visible)
				{
					this->visible=visible;
				}

				/**
				 * Enables or disables the component. If a component is disabled,
				 * all its children will effectively be disabled as well,
				 * independent of their own setting. (TODO: true?)
				 *
				 * @param enabled Whether the component shall be enabled.
				 */
				virtual void setEnabled(bool enabled)
				{
					this->enabled=enabled;
				}

				/**
				 * Returns the coordinates of the lower left corner of the
				 * component's client rectangle, relative to the parent
				 * component.
				 *
				 * @return Position.
				 */
				virtual Vector2D getPos() const
				{
					return(pos);
				}

				/**
				 * Returns the dimensions of the component's client rectangle.
				 *
				 * @return Dimensions.
				 */
				virtual Vector2D getDimensions() const
				{
					return(dimensions);
				}

				/**
				 * Returns the component's name.

				 * @return Name.
				 */
				virtual const std::string &getName() const
				{
					return(name);
				}

				/**
				 * Returns the component's visibility setting.
				 *
				 * @return Visibility.
				 */
				virtual bool getVisible() const
				{
					return(visible);
				}

				/**
				 * Returns whether the component is enabled.
				 *
				 * @return <code>true</code> for enabled components.
				 */
				virtual bool getEnabled() const
				{
					return(enabled);
				}

				/**
				 * Returns the upper left (TODO:?) corner of the component's
				 * area in the internal component grid.
				 *
				 * @return Grid position.
				 */
				basic_Vector2D<int> getGridPos() const
				{
					return(gridPos);
				}

				/**
				 * Sets the upper left (TODO:?) corner of the component's
				 * area in the internal component grid.
				 *
				 * @param gridPos Grid position.
				 */
				void setGridPos(basic_Vector2D<int> gridPos)
				{
					this->gridPos=gridPos;
				}

				/**
				 * Returns the dimensions of the component's
				 * area in the internal component grid.
				 *
				 * @return Grid dimensions.
				 */
				basic_Vector2D<int> getGridDimensions() const
				{
					return(gridDimensions);
				}

				/**
				 * Sets the dimensions of the component's
				 * area in the internal component grid.
				 *
				 * @param gridDimensions Grid dimensions.
				 */
				void setGridDimensions(basic_Vector2D<int> gridDimensions)
				{
					this->gridDimensions=gridDimensions;
				}

				/**
				 * Updates the neighbour settings for all child components, based
				 * on the areas they occupy in the internal component grid.
				 */
				void recalcNeighbours();

				/**
				 * Brings a component to the front of all its "siblings" (i.e. the
				 * other children of the component's parent).
				 */
				void bringToFront();

				/**
				 * Moves the component to the centre of the area with the specified
				 * dimensions, assuming 0/0 as the lower left coordinate of that
				 * area.
				 *
				 * @param dimensions Dimension within which the component will be
				 *                   centred.
				 */
				void center(const Vector2D dimensions);

				/**
				 * Reads the component and all its children from the specified DOM
				 * node.
				 *
				 * @param node DOM node containing component data.
				 * @param gui  TODO: ?
				 */
				void getComponentFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Reads the component's children from the specified DOM
				 * node.
				 *
				 * @param node DOM node containing component data.
				 * @param gui  TODO: ?
				 */
				void getChildrenFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Reads the component's own properties from the specified DOM
				 * node. Derived classes should override this method and add
				 * their own properties.
				 *
				 * @param node DOM node containing component data.
				 * @param gui  TODO: ?
				 */
				virtual void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Returns the number of child components.
				 *
				 * @return Number of children.
				 */
				int size()
				{
					return(children.size());
				}

				/**
				 * Returns the component's parent.
				 *
				 * @return Parent component (0 if this component is the root).
				 */
				Component *getParent();

				/**
				 * Return's the root component, which must be a GUI instance.
				 *
				 * @return The GUI/root component.
				 */
				GUI *getRoot();

				/**
				 * Draws the component and all its children.
				 */
				virtual void draw();

				/**
				 * Draws the component at the current graphical cursor location.
				 * This method is called before the component's children are drawn.
				 * Override this in derived classes to draw widgets.
				 */
				virtual void drawSelf() {};

				/**
				 * Draws the component at the current graphical cursor location.
				 * This method is called after the component's children are drawn.
				 * Override this in derived classes to draw widgets.
				 */
				virtual void drawSelfPost() {};

				/**
				 * TODO: ?
				 */
				void captureMouse(bool capture);

				/**
				 * Removes the focus from the currently focused component and
				 * instead focuses this component, or removes the focus from
				 * this component without focusing another one.
				 *
				 * @param focused <code>true</code> to focus, <code>false</code> to unfocus
				 *                the component.
				 */
				void setFocused(bool focused);

				/**
				 * Determines whether the component is focused.
				 *
				 * @return <code>true</code> for focused components.
				 */
				bool isFocused();

				/**
				 * Determines whether the component's client area includes the specified
				 * coordinate.
				 *
				 * @param pos Coordinate (relative to the component).
				 *
				 * @return <code>true</code> if the component includes the coordinate.
				 */
				bool contains(Vector2D pos);

				/**
				 * Returns the component at the specified position.
				 *
				 * @param pos Coordinate (relative to the component).
				 *
				 * @return A child of the component if it was found at the specified coordinate,
				 *         this component itself if it included the coordinate but now child was
				 *         found there, or 0 if the coordinate lies outside the component's client
				 *         rectangle.
				 */
				Component *find(const Vector2D pos);

				/**
				 * Determines whether the component is enabled <b>and</b> visible.
				 *
				 * @return <code>true</code> if the component is enabled.
				 */
				bool isEnabled();

				/**
				 * Moves the focus to the next component.
				 */
				void focusNext();

				/**
				 * Moves the focus to the previous component.
				 */
				void focusPrevious();

				/**
				 * Updates the component and all its children.
				 *
				 * @param queue Current message queue. Some components might
				 *              write to this queue to fire events.
				 * @param ticks Number of ticks since the last perform call.
				 */
				void perform(ControlQueue *queue,GLfloat ticks);

				/**
				 * Performs a move action on the component. If a move trigger is set for this
				 * direction, it is written to the queue, otherwise the focus is moved in the
				 * specified direction, if the component has a neighbour there.
				 *
				 * @param dir   Direction.
				 * @param queue Current message queue.
				 */
				virtual void move(Direction dir,ControlQueue *queue);

				/**
				 * Updates the component. Override this for derived classes which require
				 * custom behaviour.
				 *
				 * @param queue Current message queue. Some components might
				 *              write to this queue to fire events.
				 * @param ticks Number of ticks since the last perform call.
				 */
				virtual void performSelf(ControlQueue *queue,GLfloat ticks) {};

				/**
				 * Called when the component receives focus.
				 *
				 * @param queue Current message queue.
				 * @param pos   TODO: ?
				 */
				virtual void enter(ControlQueue *queue,Vector2D pos) {};

				/**
				 * Called when the component loses focus.
				 *
				 * @param queue Current message queue.
				 * @param pos   TODO: ?
				 */
				virtual void leave(ControlQueue *queue,Vector2D pos) {};

				/**
				 * Called when the component receives a mouse down event.
				 *
				 * @param queue  Current message queue.
				 * @param pos    Coordinate (relative to the component) at which the mouse
				 *               event occured.
				 * @param button Number of the button that was pressed.
				 */
				virtual void mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button) {};

				/**
				 * Called when the component receives a mouse up event.
				 *
				 * @param queue  Current message queue.
				 * @param pos    Coordinate (relative to the component) at which the mouse
				 *               event occured.
				 * @param button Number of the button that was released.
				 */
				virtual void mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button) {};

				/**
				 * Called when the component receives a mouse move event.
				 *
				 * @param queue  Current message queue.
				 * @param pos    Coordinate (relative to the component) to which the mouse
				 *               was moved.
				 */
				virtual void mouseMove(ControlQueue *queue,Vector2D pos) {};

				/**
				 * Called when the component receives a key down event.
				 *
				 * @param queue    Current message queue.
				 * @param key      Key that was pressed.
				 * @param modifier Currently active modifier keys.
				 * @param unicode  Unicode of the character that was pressed.
				 *
				 * @return <code>true</code> if the event was handled, <code>false</code> otherwise
				 *         (in which case the parent component gets a chance to handle the event).
				 */
				virtual bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Called when the component receives a key up event.
				 *
				 * @param queue    Current message queue.
				 * @param key      Key that was released.
				 * @param modifier Currently active modifier keys.
				 * @param unicode  Unicode of the character that was released. TODO: ?
				 *
				 * @return <code>true</code> if the event was handled, <code>false</code> otherwise
				 *         (in which case the parent component gets a chance to handle the event).
				 */
				virtual bool keyUp(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode) { return(false); };

				/**
				 * Called when the component receives a joy button down event.
				 *
				 * @param queue    Current message queue.
				 * @param joy      Index of the joystick.
				 * @param button   Index of the joystick button.
				 *
				 * @return <code>true</code> if the event was handled, <code>false</code> otherwise
				 *         (in which case the parent component gets a chance to handle the event).
				 */
				virtual bool joyButtonDown(ControlQueue *queue,Uint8 joy,Uint8 button) { return(false); };

				/**
				 * Called when the component receives a joy button up event.
				 *
				 * @param queue    Current message queue.
				 * @param joy      Index of the joystick.
				 * @param button   Index of the joystick button.
				 *
				 * @return <code>true</code> if the event was handled, <code>false</code> otherwise
				 *         (in which case the parent component gets a chance to handle the event).
				 */
				virtual bool joyButtonUp(ControlQueue *queue,Uint8 joy,Uint8 button) { return(false); };

				/**
				 * Called when the component receives a joy axis event.
				 *
				 * @param queue        Current message queue.
				 * @param joy          Index of the joystick.
				 * @param axis         Index of the joystick axis.
				 * @param digitalValue Digital value of the axis' current position.
				 *
				 * @return <code>true</code> if the event was handled, <code>false</code> otherwise
				 *         (in which case the parent component gets a chance to handle the event).
				 */
				virtual bool joyAxisMotion(ControlQueue *queue,Uint8 joy,Uint8 axis,short int digitalValue);
		};

		/**
		 * A callback handler for general GUI events.
		 */
		class GUICallback
		{
			public:
				/**
				 * Empty destructor.
				 */
				virtual ~GUICallback() {};

				/**
				 * Called when the user moves the focus from one control to another.
				 */
				virtual void onMove()=0;

				/**
				 * Called when the user clicks or otherwise confirms a component.
				 */
				virtual void onConfirm()=0;
		};

		/**
		 * A GUI is the root class for a tree of components.
		 */
		class GUI : public Component,public Evaluator,public BasicResourceBank
		{
			friend class Component;

			protected:
				/** Fonts used by the GUI. TODO: ? */
				fonts::FontBank *fonts;

				/** Images used by the GUI. TODO: ? */
				mysdl::gfx::ImageBank *images;

				/** TODO: ? */
				std::map<std::string,Trigger *> *triggers;

				/** Styles for the components in this GUI. */
				StyleBank *styles;

				/** The GUI's mouse cursor. */
				mysdl::sprites::Sprite *mouseCursor;

				/** The component that currently has captured the mouse (0 if none). */
				Component *mouseCaptured;

				/** The component that currently has the focus. */
				Component *focused;

				/** TODO: ? */
				bool active;

				/** TODO: ? */
				std::string dir;

				/** Delay (in ticks) after which a pressed key is repeated. */
				GLfloat keydelay;

				/** Delay (in ticks) between two repetitions of a pressed key. */
				GLfloat keyrepeat;

				/** Callback interface for general GUI events. */
				GUICallback *callback;

			public:
				/**
				 * Creates a new GUI instance.
				 */
				GUI();

				/**
				 * Releases the GUI along with all its children.
				 */
				~GUI();

				/**
				 * Reads the GUI's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing GUI data.
				 * @param gui  Must be 0. TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * TODO: ?
				 */
				virtual bool evaluate(const SDL_Event &event,ControlQueue *queue);

				/**
				 * Reads the GUI, including all its children, from the specified DOM node.
				 *
				 * @param root    DOM node containing GUI data.
				 * @param baseDir TODO: ?
				 */
				void getFromNode(const xml::Node *root,const std::string &baseDir="")
				{
					getComponentFromNode(root,0);
				}

				/**
				 * Draws the GUI along with all its children.
				 */
				void draw();

				/**
				 * Draws the mouse cursor after drawing the GUI's children.
				 */
				void drawSelfPost();


				/**
				 * TODO: ?
				 */
				void setFonts(fonts::FontBank *fonts);

				/**
				 * Returns the font bank used by the GUI.
				 *
				 * @return Font bank.
				 */
				virtual fonts::FontBank *getFonts();

				/**
				 * TODO: ?
				 */
				void setImages(mysdl::gfx::ImageBank *images);

				/**
				 * Returns the image bank used by the GUI.
				 *
				 * @return Font bank.
				 */
				virtual mysdl::gfx::ImageBank *getImages();

				/**
				 * Finds the trigger with the specified name.
				 *
				 * @param name Trigger name.
				 *
				 * @return Trigger with the specified name, or 0 if no such trigger
				 *         was found.
				 */
				Trigger *findTrigger(const std::string &name) const;

				/**
				 * TODO: ?
				 */
				virtual std::map<std::string,Trigger *> *getTriggers();

				/**
				 * Returns the style bank used by the GUI.
				 *
				 * @return Style bank.
				 */
				virtual StyleBank *getStyles();

				/**
				 * Returns the sprite used as the GUI's mouse cursor.

				 * @return Mouse cursor.
				 */
				virtual mysdl::sprites::Sprite *getMouseCursor();

				/**
				 * Returns the component that currently captured the mouse.
				 *
				 * @return Component, or 0 if the mouse is not captured.
				 */
				virtual Component *getMouseCaptured();

				/**
				 * Returns the component that currently has the focus.
				 *
				 * @return Focused component, or 0 if no component currently
				 *         has the focus.
				 */
				virtual Component *getFocused();

				/**
				 * Returns whether the GUI is currently active.
				 *
				 * @return <code>true</code> if the GUI is active.
				 */
				virtual bool getActive();

				/**
				 * Returns the delay (in ticks) after which a pressed key is repeated.
				 *
				 * @return Key delay.
				 */
				virtual GLfloat getKeyDelay();

				/**
				 * Returns the delay (in ticks) between two repetitions of a pressed key.
				 *
				 * @return Key repeat rate.
				 */
				virtual GLfloat getKeyRepeat();

				/**
				 * Sets the sprite that will be used as the mouse cursor. The GUI will
				 * automatically delete this sprite when the cursor is changed or the
				 * GUI is released.
				 *
				 * @param mouseCursor Mouse cursor sprite.
				 */
				virtual void setMouseCursor(mysdl::sprites::Sprite *mouseCursor);

				/**
				 * Activates or deactivates the GUI. If the GUI is inactive, it will not
				 * react to user input.
				 *
				 * @param active <code>true</code> to activate, <code>false</code> to
				 *               deactivate the GUI.
				 */
				virtual void setActive(bool active);

				/**
				 * Sets the GUI's key repetition delay properties.
				 *
				 * @param keydelay  Delay (in ticks) after which a pressed key is repeated.
				 * @param keyrepeat Delay (in ticks) between two repetitions of a pressed key.
				 */
				virtual void setKeyRepeat(GLfloat keydelay,GLfloat keyrepeat);

				/**
				 * Sets the callback handler for general GUI events.
				 *
				 * @param callback Callback handler. Use <code>0</code> to ignore events.
				 */
				void setCallback(GUICallback *callback);

				/**
				 * Returns the callback handler for general GUI events.
				 *
				 * @return Callback handler.
				 */
				GUICallback *getCallback();

				/**
				 * Called when the user moves the focus from one control to another.
				 * Calls the corresponding callback handler method, if a handler is
				 * available.
				 */
				void doMove();

				/**
				 * Called when the user clicks or otherwise confirms a component.
				 * Calls the corresponding callback handler method, if a handler is
				 * available.
				 */
				void doConfirm();
		};

		/**
		 * A base class for object (usually components) which have a caption
		 * text and possibly an image.
		 */
		class CaptionedObject
		{
			protected:
				/** Caption text. */
				std::string caption;

				/** Caption image (may be 0). */
				mysdl::gfx::Image *image;

			public:
				/**
				 * Creates a new captioned object with an empty caption text
				 * and no image.
				 */
				CaptionedObject();

				/**
				 * Releases the captioned object.
				 */
				virtual ~CaptionedObject();

				/**
				 * Reads the caption properties from the specified DOM node.
				 *
				 * @param node DOM node containing caption data.
				 * @param gui  TODO: ?
				 */
				virtual void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Sets the caption text.
				 *
				 * @param caption Caption text.
				 */
				virtual void setCaption(const std::string &caption);

				/**
				 * Sets the caption iamge.
				 *
				 * @param image Caption image.
				 */
				virtual void setImage(mysdl::gfx::Image *image);

				/**
				 * Returns the caption text.
				 *
				 * @return Caption text.
				 */
				virtual std::string getCaption();

				/**
				 * Returns the caption image.
				 *
				 * @return Caption image.
				 */
				virtual mysdl::gfx::Image *getImage();
		};
	};

	/**
	 * Translates a string into an Orientation.
	 *
	 * @param str String.
	 *
	 * @return Orientation.
	 *
	 * @throws std::string if str does not represent an orientation.
	 */
	template<> mysdl::gui::Orientation strTo<mysdl::gui::Orientation>(std::string str);
};

#endif
