/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_LISTS_H
#define MYSDL_GUI_LISTS_H

#include <list>
#include <string>

#include "core.h"
#include "styles.h"

namespace mysdl
{
	namespace gui
	{
		/**
		 * An item in a list, which can be displayed in list components.
		 */
		class ListItem
		{
			public:
				/** Caption. */
				std::string caption;

				/** ID for internal use. */
				int id;

				/** Data for internal use. */
				void *data;

				/**
				 * Compares two list items.
				 *
				 * @param item Second list items.
				 *
				 * @return <code>true</code> if item is identical to this item.
				 */
				bool operator==(const ListItem &item);

				/**
				 * Reads the item's properties from a DOM node.
				 *
				 * @param node DOM node containing item properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(xml::Node *node);
		};

		/**
		 * An ordered list of list items which can be displayed in
		 * list components.
		 */
		class List
		{
			private:
				/** The number of times this list is currently used. */
				int useCount;

			public:
				/** Ordered list of list items. */
				std::list<ListItem *> list;

				/**
				 * Creates a new list.
				 */
				List();

				/**
				 * Releases the list and all items it contains.
				 */
				~List();

				/**
				 * Increments the use counter by one.
				 */
				void use();

				/**
				 * Decrements the use counter by one. If it reaches 0,
				 * the list is automatically deleted.
				 */
				void unuse();

				/**
				 * Clears the list, deleting all items it contains.
				 */
				void clear();

				/**
				 * Adds a list item to the end of the list.
				 *
				 * @param item New item.
				 */
				void operator+=(ListItem *item);

				/**
				 * Removes a list item from the list.
				 *
				 * @param item List item.
				 */
				void operator-=(ListItem *item);

				/**
				 * Finds a list item by its referenced data.
				 *
				 * @param data Data the requested list item references.
				 *
				 * @return List item, or 0 if no matching item was found.
				 */
				ListItem *findByData(void *data);
		};

		/**
		 * A component which contains a list and displays only the currently
		 * selected entry when inactive. Activating it (e.g. by clicking on
		 * it) drops down the list of items and lets the user select a new
		 * item.
		 *
		 * <b>Note:</b> The current implementation does not yet drop down a
		 *              list of item. Instead, activating the component simply
		 *              cycles through the available items directly.
		 */
		class DropdownList : public Component,public CaptionedObject
		{
			protected:
				/** Whether the user is currently "holding" the list, e.g. by
				 *  having it selected with a not yet released mouse button. */
				bool holding;

				/** Style used for drawing the list. */
				DropdownListStyle *style;

				/** Trigger to be used when the user changes the selected item. */
				Trigger *trigger;

				/** List contents.*/
				List *items;

				/** The currently selected item. */
				ListItem *selected;

			public:
				/**
				 * Creates a new drop down list.
				 */
				DropdownList();

				/**
				 * Releases the list.
				 */
				~DropdownList();

				/**
				 * Sets the list displayed by the drop down list and
				 * selects the first item. TODO: ?
				 *
				 * @param items New list.
				 */
				virtual void setItems(List *items);

				/**
				 * Selects the specified item.
				 *
				 * @param item New selected item.
				 */
				virtual void selectItem(ListItem *item);

				/**
				 * Gets the list's current state.
				 *
				 * @return Button state.
				 */
				ButtonState getState();

				/**
				 * Draws the list.
				 */
				void drawSelf();

				/**
				 * Marks the list as "held" when the left mouse button is clicked on it.
				 */
				void mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * "Releases" the list if it was "held".
				 */
				void mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Tests whether the list is currently "held".
				 *
				 * @return <code>true</code> if the list is "held" and focused.
				 */
				bool isDown();

				/**
				 * Reads the list's properties from a DOM node.
				 *
				 * @param node DOM node containing drop down list properties.
				 * @param gui  TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Performs a click on the list, selecting the next item and
				 * triggering the list's trigger.
				 *
				 * @param queue Current control queue.
				 */
				void doClick(ControlQueue *queue);

				/**
				 * Cycles through the list when the return or space key is pressed.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Returns the list of items used by the drop down list.
				 *
				 * @return List.
				 */
				virtual List *getItems();

				/**
				 * Returns the currently selected item.
				 *
				 * @return Selected item (may be 0).
				 */
				virtual ListItem *getSelected();

				/**
				 * Sets the style to be used for drawing the list.
				 *
				 * @param style New style.
				 */
				virtual void setStyle(DropdownListStyle *style);

				/**
				 * Returns the style used for drawing the list.
				 *
				 * @return Style.
				 */
				virtual DropdownListStyle *getStyle();

				/**
				 * Sets the trigger to be used when the selected
				 * item changes.
				 *
				 * @param trigger New trigger.
				 */
				virtual void setTrigger(Trigger *trigger);

				/**
				 * Returns the trigger used when the selected
				 * item changes.
				 *
				 * @return Trigger.
				 */
				virtual Trigger *getTrigger();
		};
	};
};

#endif
