/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_SLIDER_H
#define MYSDL_GUI_SLIDER_H

#include <list>
#include <string>

#include "core.h"
#include "standard.h"
#include "styles.h"
#include "xml.h"

namespace mysdl
{
	namespace gui
	{
		/**
		 * Bitfield specifiers for slider buttons.
		 */
		enum SliderButtons {sbNone=0,sbDec=1,sbInc=2,sbBoth=3};
	}

	/**
	 * Translates a string to a slider button specifier.
	 *
	 * @param str String.
	 *
	 * @return Slider button specifier.
	 *
	 * @throws std::string If str was invalid.
	 */
	template<> mysdl::gui::SliderButtons strTo<mysdl::gui::SliderButtons>(std::string str);

	namespace gui
	{
		/**
		 * A slider component which lets the user drag a bar between a minimum and a maximum
		 * value. Optionally, a slider can have buttons to increment and/or decrement the
		 * current value.
		 */
		template<typename ValueType=int> class Slider : public Component,public ClickEventHandler
		{
			protected:
				/** Current rectangle of the slider's entire bar (from minimum to maximum value). */
				Rect barRect;

				/** Current rectangle of the slider's handle. */
				Rect handleRect;

				/** Whether the user is currently holding down the handle. */
				bool holding;

				/** Offset between the handle coordinate and the coordinate where the user
				 *  clicked to drag the handle. */
				Vector2D grabOffset;

				/** Decrement button placed before the bar. */
				Button *bDecBefore;

				/** Increment button placed before the bar. */
				Button *bIncBefore;

				/** Decrement button placed after the bar. */
				Button *bDecAfter;

				/** Increment button placed after the bar. */
				Button *bIncAfter;

				/**
				 * Calculates the button and rectangle sizes for the
				 * current slider properties.
				 */
				void calculateSizes();

				/** Trigger to be used when the slider position changes. */
				Trigger *trigger;

			public:
				/** Specifies which buttons should appear before the bar. */
				SliderButtons buttonsBefore;

				/** Specifies which buttons should appear after the bar. */
				SliderButtons buttonsAfter;

				/** Slider orientation. */
				Orientation orientation;

				/** TODO: ? */
				GLfloat size;

				/** Minimum slider value. */
				ValueType min;

				/** Maximum slider value. */
				ValueType max;

				/** Small increment/decrement step, used when the slider buttons
				 *  are clicked. */
				ValueType stepSmall;

				/** Large increment/decrement step, used when the user clicks
				 *  inside the slider bar beneath the handle. */
				ValueType stepLarge;

				/** The slider's current value. */
				ValueType value;

				/** Style used to draw the slider's bar. */
				RectStyle *barStyle;

				/** Style used to draw the slider's handle. */
				ButtonStyle *handleStyle;

				/** Style used to draw the slider's buttons. */
				ButtonStyle *buttonStyle;

				/**
				 * Creates a new slider.
				 */
				Slider();

				/**
				 * Draws the slider.
				 */
				void drawSelf();

				/**
				 * TODO: ?
				 * @return
				 */
				GLfloat getBarSize();

				/**
				 * Reads the component's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing component data.
				 * @param gui  Must be 0. TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Returns the handle's current state.
				 *
				 * @return Handle's button state.
				 */
				ButtonState getState();

				/**
				 * Enters grab mode if the left mouse button is clicked on the handle,
				 * changes the slider value if the button is clicked beneath the handle.
				 */
				void mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Leaves grab mode.
				 */
				void mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Moves the handle when in grab mode.
				 */
				void mouseMove(ControlQueue *queue,Vector2D pos);

				/**
				 * Returns whether the slider is currently in grab mode (i.e. whether
				 * the handle is down).

				 * @return <code>true</code> if grab mode is active.
				 */
				bool isDown();

				/**
				 * Changes the slider's value when certain keys are pressed.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);
				
				/**
				 * Changes the slider's value when a certain joy axis changes.
				 */
				bool joyAxisMotion(ControlQueue *queue,Uint8 joy,Uint8 axis,short int digitalValue);

				/**
				 * Increments the slider's value.
				 *
				 * @param large <code>true</code> uses stepLarge,
				 *              <code>false</code> uses stepSmall.
				 */
				void increase(bool large=false);

				/**
				 * Decrements the slider's value.
				 *
				 * @param large <code>true</code> uses stepLarge,
				 *              <code>false</code> uses stepSmall.
				 */
				void decrease(bool large=false);

				/**
				 * Specifies what buttons to display before the slider bar.
				 *
				 * @param b Button specifiers.
				 */
				void setButtonsBefore(SliderButtons b);

				/**
				 * Specifies what buttons to display after the slider bar.
				 *
				 * @param b Button specifiers.
				 */
				void setButtonsAfter(SliderButtons b);

				/**
				 * Determines the size used for buttons belonging to this slider.
				 *
				 * @return Button dimensions.
				 */
				Vector2D getButtonSize();

				/**
				 * Recalculates the slider parts' sizes when the
				 * slider dimensions change.
				 */
				void setDimensions(Rect dimensions);

				/**
				 * Updates the value if one of the slider's buttons is clicked.
				 */
				void handleClickEvent(Component *source);
		};

		// ================================================================
		//  Slider
		// ================================================================

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> Slider<ValueType>::Slider() :
			Component(),
			barRect(),handleRect(),holding(false),grabOffset(),
			bDecBefore(0),bIncBefore(0),bDecAfter(0),bIncAfter(0),
			trigger(0),
			buttonsBefore(sbNone),buttonsAfter(sbNone),orientation(orHorizontal),size(0.25),
			min(),max(),stepSmall(1),stepLarge(1),
			value(),
			barStyle(0),handleStyle(0),buttonStyle(0)
		{
			wantsFocus=true;
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> Vector2D Slider<ValueType>::getButtonSize()
		{
			return(orientation==orHorizontal ? Vector2D(dimensions.y,dimensions.y) : Vector2D(dimensions.x,dimensions.x));
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::setDimensions(Rect dimensions)
		{
			Component::setDimensions(dimensions);
			calculateSizes();
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> GLfloat Slider<ValueType>::getBarSize()
		{
			return(orientation==orHorizontal ? barRect.dimensions.x : barRect.dimensions.y);
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::calculateSizes()
		{
			Vector2D buttonDimensions(getButtonSize());
			GLfloat buttonSize=(orientation==orHorizontal ? buttonDimensions.x : buttonDimensions.y);

			barRect=Rect(Vector2D(0,0),dimensions);

			if(bDecBefore)
			{
				bDecBefore->setDimensions(Rect(barRect.pos,buttonDimensions));

				if(orientation==orHorizontal)
				{
					barRect.reduceLeft(buttonSize);
				}
				else
				{
					barRect.reduceBottom(buttonSize);
				}
			}

			if(bIncBefore)
			{
				bIncBefore->setDimensions(Rect(barRect.pos,buttonDimensions));

				if(orientation==orHorizontal)
				{
					barRect.reduceLeft(buttonSize);
				}
				else
				{
					barRect.reduceBottom(buttonSize);
				}
			}

			Vector2D offset(buttonSize,buttonSize);//=(orientation==orHorizontal ? Vector2D(buttonSize,0) : Vector2D(0,buttonSize));

			if(bIncAfter)
			{
				bIncAfter->setDimensions(Rect(barRect.pos+barRect.dimensions-offset,buttonDimensions));

				if(orientation==orHorizontal)
				{
					barRect.reduceRight(buttonSize);
				}
				else
				{
					barRect.reduceTop(buttonSize);
				}
			}

			if(bDecAfter)
			{
				bDecAfter->setDimensions(Rect(barRect.pos+barRect.dimensions-offset,buttonDimensions));

				if(orientation==orHorizontal)
				{
					barRect.reduceRight(buttonSize);
				}
				else
				{
					barRect.reduceTop(buttonSize);
				}
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::setButtonsBefore(SliderButtons b)
		{
			if(b!=buttonsBefore)
			{
				if(b & sbDec)
				{
					if(!bDecBefore)
					{
						bDecBefore=new Button();
						bDecBefore->setName("bDecBefore");
						(*this)+=bDecBefore;

						bDecBefore->setCaption("<");
						bDecBefore->setStyle(buttonStyle);
						bDecBefore->setVisible(true);
						bDecBefore->setEnabled(true);
						bDecBefore->setOnClick(this);
						bDecBefore->setRepeat(true);
					}
				}
				else
				{
					if(bDecBefore)
					{
						delete bDecBefore;
						bDecBefore=0;
					}
				}

				if(b & sbInc)
				{
					if(!bIncBefore)
					{
						bIncBefore=new Button();
						bIncBefore->setName("bIncBefore");
						bIncBefore->setCaption(">");
						bIncBefore->setStyle(buttonStyle);
						bIncBefore->setVisible(true);
						bIncBefore->setEnabled(true);
						bIncBefore->setOnClick(this);
						bIncBefore->setRepeat(true);

						(*this)+=bIncBefore;
					}
				}
				else
				{
					if(bIncBefore)
					{
						delete bIncBefore;
						bIncBefore=0;
					}
				}
			}

			calculateSizes();
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::setButtonsAfter(SliderButtons b)
		{
			if(b!=buttonsAfter)
			{
				if(b & sbDec)
				{
					if(!bDecAfter)
					{
						bDecAfter=new Button();
						bDecAfter->setName("bDecAfter");
						bDecAfter->setCaption("<");
						bDecAfter->setStyle(buttonStyle);
						bDecAfter->setVisible(true);
						bDecAfter->setEnabled(true);
						bDecAfter->setOnClick(this);
						bDecAfter->setRepeat(true);

						(*this)+=bDecAfter;
					}
				}
				else
				{
					if(bDecAfter)
					{
						delete bDecAfter;
						bDecAfter=0;
					}
				}

				if(b & sbInc)
				{
					if(!bIncAfter)
					{
						bIncAfter=new Button();
						bIncAfter->setName("bIncAfter");
						bIncAfter->setCaption(">");
						bIncAfter->setStyle(buttonStyle);
						bIncAfter->setVisible(true);
						bIncAfter->setEnabled(true);
						bIncAfter->setOnClick(this);
						bIncAfter->setRepeat(true);

						(*this)+=bIncAfter;
					}
				}
				else
				{
					if(bIncAfter)
					{
						delete bIncAfter;
						bIncAfter=0;
					}
				}
			}

			calculateSizes();
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::increase(bool large)
		{
			value=clamp(value+(large ? stepLarge : stepSmall),min,max);
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::decrease(bool large)
		{
			value=clamp(value-(large ? stepLarge : stepSmall),min,max);
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::handleClickEvent(Component *source)
		{
			if((source==bDecBefore)||(source==bDecAfter))
			{
				decrease();
			}
			else if((source==bIncBefore)||(source==bIncAfter))
			{
				increase();
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> bool Slider<ValueType>::keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode)
		{
			if(key==SDLK_HOME)
			{
				value=min;

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if(key==SDLK_END)
			{
				value=max;

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if((orientation==orHorizontal)&&(key==SDLK_LEFT))
			{
				decrease();//value=clamp(value-stepSmall,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if((orientation==orHorizontal)&&(key==SDLK_RIGHT))
			{
				increase();//value=clamp(value+stepSmall,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if((orientation==orVertical)&&(key==SDLK_DOWN))
			{
				decrease();//value=clamp(value-stepSmall,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if((orientation==orVertical)&&(key==SDLK_UP))
			{
				increase();//value=clamp(value+stepSmall,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if(key==SDLK_PAGEUP)
			{
				decrease(true);//value=clamp(value-stepLarge,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else if(key==SDLK_PAGEDOWN)
			{
				increase(true);//value=clamp(value+stepLarge,min,max);

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return(true);
			}
			else
			{
				return(Component::keyDown(queue,key,modifier,unicode));
			}
		}
		
		/*
		* See the template declaration for documentation.
		*/
		template<typename ValueType> bool Slider<ValueType>::joyAxisMotion(ControlQueue *queue,Uint8 joy,Uint8 axis,short int digitalValue)
		{
			if((orientation==orHorizontal)&&(axis==0)&&(digitalValue<0))
			{
				decrease();

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return true;
			}
			else if((orientation==orHorizontal)&&(axis==0)&&(digitalValue>0))
			{
				increase();

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return true;
			}
			else if((orientation==orVertical)&&(axis==1)&&(digitalValue<0))
			{
				decrease();

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return true;
			}
			else if((orientation==orVertical)&&(axis==1)&&(digitalValue>0))
			{
				increase();

				if(trigger)
				{
					(*queue)<<trigger;
				}

				return true;
			}
			else
			{
				return Component::joyAxisMotion(queue,joy,axis,digitalValue);
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button)
		{
			if(button==4)
			{
				increase();
			}
			else if(button==5)
			{
				decrease();
			}
			else if(button==1)
			{
				if(orientation==orHorizontal)
				{
					if(pos.x<handleRect.pos.x)
					{
						value=clamp(value-stepLarge,min,max);
					}
					else if(pos.x>=handleRect.pos.x+handleRect.dimensions.x)
					{
						value=clamp(value+stepLarge,min,max);
					}
					else
					{
						holding=true;
					}
				}
				else
				{
					if(pos.y<handleRect.pos.y)
					{
						value=clamp(value-stepLarge,min,max);
					}
					else if(pos.y>=handleRect.pos.y+handleRect.dimensions.y)
					{
						value=clamp(value+stepLarge,min,max);
					}
					else
					{
						holding=true;
					}
				}

				if(holding)
				{
					captureMouse(true);
					grabOffset=pos-handleRect.pos;
				}
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::mouseMove(ControlQueue *queue,Vector2D pos)
		{
			if(holding)
			{
				pos-=grabOffset;

				GLfloat barSize=getBarSize();

				GLfloat newValue;

				if(size>0.0)
				{
					if(orientation==orHorizontal)
					{
						newValue=((pos.x/(barSize*(1-size)))*(GLfloat)(max-min)+(GLfloat)min);
					}
					else
					{
						newValue=((pos.y/(barSize*(1-size)))*(GLfloat)(max-min)+(GLfloat)min);
					}
				}
				else
				{
					if(orientation==orHorizontal)
					{
						newValue=((pos.x/(barSize+size))*(GLfloat)(max-min)+(GLfloat)min);
					}
					else
					{
						newValue=((pos.y/(barSize+size))*(GLfloat)(max-min)+(GLfloat)min);
					}
				}

				value=(ValueType)clamp(newValue,(GLfloat)min,(GLfloat)max);
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button)
		{
			if(button==1)
			{
				holding=false;
				captureMouse(false);
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> ButtonState Slider<ValueType>::getState()
		{
			ButtonState state=ButtonState(bsUp);

			if(isDown())
			{
				state=ButtonState(state | bsDown);
			}

			if(focused)
			{
				state=ButtonState(state | bsHot);
			}

			return(state);
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> bool Slider<ValueType>::isDown()
		{
			return((holding)&&(focused));
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::drawSelf()
		{
			GLfloat barSize=getBarSize();
			GLfloat handleSize=size>0.0 ? barSize*size : -size;

			if(barStyle)
			{
				barStyle->draw(barRect);
			}

			if(handleStyle)
			{
				ButtonState state=getState();

				if(size>0.0)
				{
					handleRect.pos.x=(orientation==orHorizontal ? (barSize*(1-size))*((GLfloat)(value-min)/(GLfloat)(max-min)) : 0);
					handleRect.pos.y=(orientation==orVertical   ? (barSize*(1-size))*((GLfloat)(value-min)/(GLfloat)(max-min)) : 0);
				}
				else
				{
					handleRect.pos.x=(orientation==orHorizontal ? (barSize+size)*((GLfloat)(value-min)/(GLfloat)(max-min)) : 0);
					handleRect.pos.y=(orientation==orVertical   ? (barSize+size)*((GLfloat)(value-min)/(GLfloat)(max-min)) : 0);
				}

				handleRect.dimensions.x=(orientation==orHorizontal ? handleSize : dimensions.x);
				handleRect.dimensions.y=(orientation==orVertical   ? handleSize : dimensions.y);

				handleRect.pos+=barRect.pos;

				handleStyle->draw(handleRect,state);

				//if(handleStyle->fontStyle)
				{
					glPushMatrix();
					handleStyle->print(handleRect,state,toStr(value).c_str(),0);//caption.c_str(),image);
					glPopMatrix();
				}
			}
		}

		/*
		 * See the template declaration for documentation.
		 */
		template<typename ValueType> void Slider<ValueType>::getSelfFromNode(const xml::Node *node,GUI *gui)
		{
			Component::getSelfFromNode(node,gui);

			if(const std::string *style=node->getAttribute(XML_GUI_BAR_STYLE))
			{
				this->barStyle=dynamic_cast<RectStyle *>((*gui->getStyles())[*style]);
			}

			if(const std::string *style=node->getAttribute(XML_GUI_HANDLE_STYLE))
			{
				this->handleStyle=dynamic_cast<ButtonStyle *>((*gui->getStyles())[*style]);
			}

			if(const std::string *style=node->getAttribute(XML_GUI_BUTTON_STYLE))
			{
				this->buttonStyle=dynamic_cast<ButtonStyle *>((*gui->getStyles())[*style]);
			}

			if(const std::string *value=node->getAttribute(XML_GUI_ORIENTATION))
			{
				try
				{
					orientation=strTo<Orientation>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.orientation: \"%s\".",value->c_str());
				}
			}

			size=node->getAttributeFloat(XML_GUI_SIZE,size);

			if(const std::string *value=node->getAttribute(XML_GUI_MIN))
			{
				try
				{
					min=strTo<ValueType>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.min: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_MAX))
			{
				try
				{
					max=strTo<ValueType>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.max: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_STEP_SMALL))
			{
				try
				{
					stepSmall=strTo<ValueType>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.stepSmall: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_STEP_LARGE))
			{
				try
				{
					stepLarge=strTo<ValueType>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.stepLarge: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_BUTTONS_BEFORE))
			{
				try
				{
					setButtonsBefore(strTo<SliderButtons>(*value));
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.buttonsBefore: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_BUTTONS_AFTER))
			{
				try
				{
					setButtonsAfter(strTo<SliderButtons>(*value));
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.buttonsAfter: \"%s\".",value->c_str());
				}
			}

			if(const std::string *value=node->getAttribute(XML_GUI_VALUE))
			{
				try
				{
					this->value=strTo<ValueType>(*value);
				}
				catch(...)
				{
					debug::warning(dbgGui,"Could not decode value for Slider.value: \"%s\".",value->c_str());
				}
			}

			if(const std::string *trigger=node->getAttribute(XML_GUI_TRIGGER))
			{
				this->trigger=gui->findTrigger(*trigger);
			}
		}
	};
};

#endif
