/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_STANDARD_H
#define MYSDL_GUI_STANDARD_H

#include <list>
#include <string>

#include "core.h"
#include "styles.h"
#include "xml.h"

namespace mysdl
{
	namespace gui
	{
		/**
		 * A component used to capture keys and joystick axes and button,
		 * useful for letting the user configure input settings.
		 */
		class InputCaptor : public Component
		{
			protected:
				/** Trigger to be used when a key is pressed. */
				Trigger *triggerKey;

				/** Trigger to be used when a joystick button is pressed. */
				Trigger *triggerJoyButton;

				/** Trigger to be used when a joystick axis is moved. */
				Trigger *triggerJoyAxis;

				/** Stores the last key that was captured by the component. */
				SDLKey key;

				/** Stores the last joystick that was captured by the component. */
				Uint8 joy;

				/** Stores the last joystick button that was captured by the component. */
				Uint8 joyButton;

				/** Stores the last joystick axis was captured by the component. */
				Uint8 joyAxis;

				/** Stores the direction of the last axis event. */
				short int joyAxisDirection;

			public:
				/**
				 * Creates a new input captor.
				 */
				InputCaptor();

				/**
				 * Reads the component's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing component data.
				 * @param gui  Must be 0. TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Stores the pressed key and triggers the key trigger.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Stores the pressed joystick button and triggers the joystick button trigger.
				 */
				bool joyButtonDown(ControlQueue *queue,Uint8 joy,Uint8 button);

				/**
				 * Stores the moved joystick axis and triggers the joystick axis trigger.
				 */
				bool joyAxisMotion(ControlQueue *queue,Uint8 joy,Uint8 axis,short int digitalValue);

				/**
				 * Returns the last stored key.
				 *
				 * @return Key.
				 */
				SDLKey getKey();

				/**
				 * Returns the joystick for the last stored joystick event.
				 *
				 * @return Joystick.
				 */
				Uint8 getJoy();

				/**
				 * Returns the last stored joystick button.
				 *
				 * @return Joystick button.
				 */
				Uint8 getJoyButton();

				/**
				 * Returns the last stored joystick axis.
				 *
				 * @return Joystick axis.
				 */
				Uint8 getJoyAxis();

				/**
				 * Returns the direction of the last stored joystick axis.
				 *
				 * @return Direction (digital value of the last axis event).
				 */
				short int getJoyAxisDirection();
		};

		/**
		 * A simple captioned button which can be clicked.
		 */
		class Button : public Component,public CaptionedObject
		{
			protected:
				/** Whether the user is currently holding down the button. */
				bool holding;

				/** Style for drawing the button. */
				ButtonStyle *style;

				/** Trigger to be used when the button is clicked. */
				Trigger *trigger;

				/** Event handler to be notified when the button is clicked. */
				ClickEventHandler *onClick;

				/** Whether holding the button shall repeatedly trigger clicks. */
				bool repeat;

				/** Timer for repeating clicks. */
				GLfloat repeatTimer;

			public:
				/**
				 * Creates a new button.
				 */
				Button();

				/**
				 * Returns the button's current state.
				 *
				 * @return Button state.
				 */
				ButtonState getState();

				/**
				 * Draws the button.
				 */
				void drawSelf();

				/**
				 * Holds down the button when the left mouse button is clicked.
				 */
				void mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Releases the button when the left mouse button is released and
				 * triggers a click if necessary.
				 */
				void mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Returns whether the button is currently held down.
				 *
				 * @return <code>true</code> if the button is down.
				 */
				bool isDown();

				/**
				 * Reads the component's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing component data.
				 * @param gui  Must be 0. TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Triggers a button click, writing the trigger to the queue and
				 * notifying the event handler.
				 *
				 * @param queue Current control queue.
				 */
				void doClick(ControlQueue *queue);

				/**
				 * Triggers a click event if the enter or space key is pressed on the
				 * button.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Triggers a click event if the left joystick button is pressed on the
				 * button. TODO: handle like mouse buttons?
				 */
				bool joyButtonDown(ControlQueue *queue,Uint8 joy,Uint8 button);

				/**
				 * Performs repeated clicks if the repeat property is set.
				 */
				void performSelf(ControlQueue *queue,GLfloat ticks);

				/**
				 * Sets the style to be used for drawing the button.
				 *
				 * @param style New style.
				 */
				virtual void setStyle(ButtonStyle *style);

				/**
				 * Returns the style used for drawing the button.
				 *
				 * @return Style.
				 */
				virtual ButtonStyle *getStyle();

				/**
				 * Sets the trigger to be used when the button is clicked.
				 *
				 * @param trigger New trigger.
				 */
				virtual void setTrigger(Trigger *trigger);

				/**
				 * Returns the trigger used when the button is clicked.
				 *
				 * @return Trigger.
				 */
				virtual Trigger *getTrigger();

				/**
				 * Sets the handler to be notified when the button is clicked.
				 *
				 * @param onClick Handler.
				 */
				void setOnClick(ClickEventHandler *onClick);

				/**
				 * Returns the handler used when the button is clicked.
				 *
				 * @return Handler.
				 */
				ClickEventHandler *getOnClick();

				/**
				 * Enables/disables the repeat property. If this property is
				 * set, holding down the button will repeatedly trigger
				 * clicks.
				 *
				 * @param repeat <code>true</code> to enable, <code>false</code>
				 *               to disable click repetition.
				 */
				void setRepeat(bool repeat);

				/**
				 * Returns the current repeat setting.
				 *
				 * @return Repeat setting.
				 */
				bool getRepeat();
		};

		/**
		 * A button-like component which can change between two primary
		 * states (checked and unchecked).
		 */
		class Checkbox : public Component,public CaptionedObject
		{
			protected:
				/** Whether the user is currently holding down the button. */
				bool holding;

				/** Whether the check box is currently checked. */
				bool checked;

				/** Style to be used for drawing the check box. */
				CheckboxStyle *style;

				/** Trigger to be used when checking the box. */
				Trigger *triggerOn;

				/** Trigger to be used when unchecking the box. */
				Trigger *triggerOff;

			public:
				/**
				 * Creates a new check box.
				 */
				Checkbox();

				/**
				 * Returns the check box's current state.
				 *
				 * @return Button state.
				 */
				ButtonState getState();

				/**
				 * Draws the checkbox.
				 */
				void drawSelf();

				/**
				 * Holds down the button when the left mouse button is pressed.
				 */
				void mouseDown(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Releases the button if the left mouse button is pressed and
				 * triggers a click if necessary.
				 */
				void mouseUp(ControlQueue *queue,Vector2D pos,Uint8 button);

				/**
				 * Returns whether the button is currently held down.
				 *
				 * @return <code>true</code> if the button is down.
				 */
				bool isDown();

				/**
				 * Reads the component's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing component data.
				 * @param gui  Must be 0. TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Triggers a button click, changing the checked state, writing the
				 * trigger to the queue and notifying the event handler.
				 *
				 * @param queue Current control queue.
				 */
				void doClick(ControlQueue *queue);

				/**
				 * Triggers a click event if the enter or space key is pressed on the
				 * button.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Triggers a click event if the left joystick button is pressed on the
				 * button. TODO: handle like mouse buttons?
				 */
				bool joyButtonDown(ControlQueue *queue,Uint8 joy,Uint8 button);

				/**
				 * Sets the checked state.
				 *
				 * @param checked <code>true</code> to check, <code>false</code> to
				 *                uncheck the check box.
				 */
				virtual void setChecked(bool checked);

				/**
				 * Returns whether the check box is currently checked.
				 *
				 * @return <code>true</code> if the box is checked.
				 */
				virtual bool getChecked();

				/**
				 * Sets the style to be used for drawing the check box.
				 *
				 * @param style New style.
				 */
				virtual void setStyle(CheckboxStyle *style);

				/**
				 * Returns the style used for drawing the check box.
				 *
				 * @return Style.
				 */
				virtual CheckboxStyle *getStyle();

				/**
				 * Sets the trigger to be used when the box is checked.
				 *
				 * @param triggerOn New trigger.
				 */
				virtual void setTriggerOn(Trigger *triggerOn);

				/**
				 * Returns the trigger used when the box is checked.
				 *
				 * @return Trigger.
				 */
				virtual Trigger *getTriggerOn();

				/**
				 * Sets the trigger to be used when the box is unchecked.
				 *
				 * @param triggerOn New trigger.
				 */
				virtual void setTriggerOff(Trigger *triggerOff);

				/**
				 * Returns the trigger used when the box is unchecked.
				 *
				 * @return Trigger.
				 */
				virtual Trigger *getTriggerOff();
		};
	};
};

#endif
