/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_STYLES_H
#define MYSDL_GUI_STYLES_H

#include "core.h"

namespace mysdl
{
	namespace gui
	{
		enum StretchMode {smSingle,smRepeat,smStretch};
	}

	template<> mysdl::gui::StretchMode strTo<mysdl::gui::StretchMode>(std::string str);

	namespace gui
	{
		/**
		 * An abstract base class for styles containing font information.
		 */
		class FontStyle : public Style
		{
			public:
				/** Horizontal text justification. */
				JustificationHorizontal justificationHorizontal;

				/** Vertical text justification. */
				JustificationVertical justificationVertical;

				/**
				 * Creates a new font style instance.
				 */
				FontStyle();

				/**
				 * Prints a text using the style. This is an abstract method which
				 * must be implemented by derived concrete font styles.
				 *
				 * @param rect   Rectangle in which the text will be printed,
				 *               taking the style's justification settings
				 *               into account.
				 * @param string String which will be printed.
				 */
				virtual void print(const Rect &rect,const char *string)=0;

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A simple font style class.
		 */
		class SimpleFontStyle : public FontStyle
		{
			public:
				/** Font used by the style. */
				fonts::Font *font;

				/** Colour used by the style. */
				mysdl::Color color;

				/**
				 * Creates a new simple font style.
				 */
				SimpleFontStyle();

				/**
				 * Prints a text using the style.
				 *
				 * @param rect   Rectangle in which the text will be printed,
				 *               taking the style's justification settings
				 *               into account.
				 * @param string String which will be printed.
				 */
				void print(const Rect &rect,const char *string);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * Base class for rectangle styles, that is styles used to draw
		 * rectangles.
		 */
		class RectStyle : public Style
		{
			public:
				/**
				 * Creates a new rectangle style instance.
				 */
				RectStyle();

				/**
				 * Draws a rectangle using this style. This is an abstract
				 * method that must be overriden by derived concrete rectangle
				 * style classes.
				 *
				 * @param rect Coordinates and dimensions of the rectangle that
				 *             shall be drawn.
				 */
				virtual void draw(const Rect &rect)=0;
		};

		/**
		 * A style for drawing simple rectangles. It provides a background
		 * colour as well as two colours and a thickness for the border.
		 */
		class PlainRectStyle : public RectStyle
		{
			public:
				/** Background colour. */
				Color color;

				/** Border colours (0=upper & left, 1=lower & right). */
				Color colorBorder[2];

				/** Border thickness (0 means no border). */
				GLfloat border;

				/**
				 * Creates a new plain rectangle style instance.
				 */
				PlainRectStyle();

				/**
				 * Draws a rectangle using this style.
				 *
				 * @param rect Coordinates and dimensions of the rectangle that
				 *             shall be drawn.
				 */
				void draw(const Rect &rect);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A rectangle style for drawing textured rectangles.
		 */
		class TexturedRectStyle : public RectStyle
		{
			public:
				/** The colour to be used for drawing the background.
				 *  Set this to white to draw the background texture
				 *  without modifications. */
				Color color;

				/** The colour to be used for drawing the border.
				 *  Set this to white to draw the border textures
				 *  without modifications. */
				Color colorBorder;

				/** Textures used by the style. */
				mysdl::gfx::Image *imgUpperLeft,*imgUpper,*imgUpperRight,
													*imgLeft,*imgCenter,*imgRight,
													*imgLowerLeft,*imgLower,*imgLowerRight;

				/** Whether to stretch the textures for larger rectangles
				 *  or to repeat them if they are too small. */
				StretchMode stretchUpper;
				StretchMode stretchLower;
				StretchMode stretchLeft;
				StretchMode stretchRight;
				StretchMode stretchCenter;

				/**
				 * Creates a new textured rectangle style.
				 */
				TexturedRectStyle();

				/**
				 * Draws a rectangle using this style.
				 *
				 * @param rect Coordinates and dimensions of the rectangle that
				 *             shall be drawn.
				 */
				void draw(const Rect &rect);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A style for drawing labels, consisting of a rectangle style
		 * for the label's client rectangle, a font style for label's
		 * text, and additional probperties for the label's image.
		 */
		class LabelStyle : public Style
		{
			public:
				/** Style for the client rectangle. */
				RectStyle *rect;

				/** Style for the caption text. */
				FontStyle *font;

				/** Horizontal justification for the caption image. */
				JustificationHorizontal imageHorizontal;

				/** Vertical justification for the caption image. */
				JustificationVertical imageVertical;

				/** Colour for the caption image. */
				Color imageColor;

				/**
				 * Creates a new label style.
				 */
				LabelStyle();

				/**
				 * Draws a rectangle using this style.
				 *
				 * @param rect Rectangle coordinates and dimensions.
				 */
				void draw(const Rect &rect);

				/**
				 * Prints a caption using this style.
				 *
				 * @param rect   Rectangle into which to draw the caption.
				 * @param string Caption text.
				 * @param image  Caption image.
				 */
				void print(Rect rect,const char *string,const mysdl::gfx::Image *image);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * Style for drawing buttons.
		 */
		class ButtonStyle : public Style
		{
			public:
				/** Style for drawing the button when it is released. */
				LabelStyle *up;

				/** Style for drawing the button when it is pressed. */
				LabelStyle *down;

				/** Style for drawing the button when it is released and highlighted. */
				LabelStyle *upHot;

				/** Style for drawing the button when it is pressed and highlighted. */
				LabelStyle *downHot;

				/**
				 * Creates a new button style.
				 */
				ButtonStyle();

				/**
				 * Returns the style for the specified button state.
				 *
				 * @param state Button state.
				 *
				 * @return Style.
				 */
				LabelStyle *getStyle(ButtonState state);

				/**
				 * Draws a button's rectangle using this style.
				 *
				 * @param rect  Button rectangle.
				 * @param state Button state.
				 */
				void draw(const Rect &rect,ButtonState state);

				/**
				 * Prints a button's caption using this style.
				 *
				 * @param rect   Button rectangle.
				 * @param state  Button state.
				 * @param string Caption string.
				 * @param image  Caption image.
				 */
				void print(const Rect &rect,ButtonState state,const char *string,const mysdl::gfx::Image *image);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A style used for drawing checkboxes. It consists of two button
		 * styles, one for each of the checkbox's two states: checked and
		 * unchecked.
		 */
		class CheckboxStyle : public Style
		{
			public:
				/** Style to use for drawing the box when checked. */
				ButtonStyle *checked;

				/** Style to use for drawing the box when unchecked. */
				ButtonStyle *unchecked;

				/**
				 * Creates a new checkbox style.
				 */
				CheckboxStyle();

				/**
				 * Returns the button style to used for the specified state.
				 *
				 * @param checked <code>true</code> for checked state,
				 *                <code>false</code> for unchecked state.
				 */
				ButtonStyle *getStyle(bool checked);

				/**
				 * Returns the label style to used for the specified state.
				 *
				 * @param state   Button state of the checkbox.
				 * @param checked <code>true</code> for checked state,
				 *                <code>false</code> for unchecked state.
				 */
				LabelStyle *getStyle(ButtonState state,bool checked);

				/**
				 * Draws a checkbox's client rectangle using this style.
				 *
				 * @param rect    Client rectangle of the checkbox.
				 * @param state   Button state of the checkbox.
				 * @param checked Whether the checkbox is currently checked.
				 */
				void draw(const Rect &rect,ButtonState state,bool checked);

				/**
				 * Draws a checkbox's caption using this style.
				 *
				 * @param rect    Client rectangle of the checkbox.
				 * @param state   Button state of the checkbox.
				 * @param checked Whether the checkbox is currently checked.
				 * @param string  Caption string.
				 * @param image   Caption image.
				 */
				void print(const Rect &rect,ButtonState state,bool checked,const char *string,const mysdl::gfx::Image *image);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};

		/**
		 * A style used to draw drop down lists. Currently, this is simply a
		 * button style with an added image, as the implementation does not
		 * yet actually drop down a list, but simply cycles through the available
		 * list items when the list box is clicked.
		 */
		class DropdownListStyle : public ButtonStyle
		{
			public:
				/** Image to be used for the list's caption. */
				mysdl::gfx::Image *image;

				/**
				 * Creates a new drop down list style.
				 */
				DropdownListStyle();

				/**
				 * Prints a list's caption, using the style's image as the
				 * caption image.
				 *
				 * @param rect   Client rectangle for the list box.
				 * @param state  Current state of the list box button.
				 * @param string Caption string, typically the currently selected
				 *               item.
				 */
				void print(const Rect &rect,ButtonState state,const char *string);

				/**
				 * Reads the style's properties from a DOM node.
				 *
				 * @param node DOM node containing style properties.
				 * @param gui  TODO: ?
				 */
				void getFromNode(const xml::Node *node,GUI *gui);
		};
	};
};

#endif
