/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_GUI_TEXT_H
#define MYSDL_GUI_TEXT_H

#include <list>
#include <string>

#include "core.h"
#include "styles.h"

namespace mysdl
{
	namespace gui
	{
		/**
		 * A simple captioned component.
		 */
		class Label : public Component,public CaptionedObject
		{
			protected:
				/** Style for the label. */
				LabelStyle *style;

			public:
				/**
				 * Creates a new label.
				 */
				Label();

				/**
				 * Draws the label.
				 */
				void drawSelf();

				/**
				 * Reads the label's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing label data.
				 * @param gui  TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Sets the style to be used for drawing the label.
				 *
				 * @param style Style.
				 */
				virtual void setStyle(LabelStyle *style);

				/**
				 * Returns the style used for drawing the label.
				 *
				 * @return Style.
				 */
				virtual LabelStyle *getStyle();
		};

		/**
		 * A captioned component with an editable caption.
		 */
		class Textfield : public Component,public CaptionedObject
		{
			private:
				/** The unicode of the currently pressed key. */
				Uint16 keydown;

				/** Timer (in ticks) for key repetition. */
				GLfloat repeatTimer;

				/** The style used to draw the text field. */
				LabelStyle *style;

				/** Trigger which will be used when the used confirms the input
				 *  into the text field. */
				Trigger *triggerConfirm;

			public:
				/**
				 * Creates a new text field.
				 */
				Textfield();

				/**
				 * Draws the text field.
				 */
				void drawSelf();

				/**
				 * Reads the label's properties from the specified DOM node.
				 * TODO: make private?
				 *
				 * @param node DOM node containing label data.
				 * @param gui  TODO: ?
				 */
				void getSelfFromNode(const xml::Node *node,GUI *gui);

				/**
				 * Called when a key is pressed. Adds the key to the caption string and
				 * stores it for key repetition.
				 */
				bool keyDown(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Called when a key is released. Disables key repetition.
				 */
				bool keyUp(ControlQueue *queue,SDLKey key,SDLMod modifier,Uint16 unicode);

				/**
				 * Switches the text field into edit mode when it receives focus.
				 */
				void enter(ControlQueue *queue,Vector2D pos);

				/**
				 * Disable edit mode when the text field loses focus.
				 */
				void leave(ControlQueue *queue,Vector2D pos);

				/**
				 * Performs key repetition.
				 */
				void performSelf(ControlQueue *queue,GLfloat ticks);

				/**
				 * Puts the confirmation trigger into the queue.
				 */
				void doConfirm(ControlQueue *queue);

				/**
				 * Sets the style to be used for drawing the label.
				 *
				 * @param style Style.
				 */
				virtual void setStyle(LabelStyle *style);

				/**
				 * Returns the style used for drawing the label.
				 *
				 * @return Style.
				 */
				virtual LabelStyle *getStyle();

				/**
				 * Sets the trigger to be used when the user confirms
				 * the input into the text field.
				 *
				 * @param trigger Confirmation trigger.
				 */
				virtual void setTriggerConfirm(Trigger *trigger);

				/**
				 * Returns the trigger used when the user confirms
				 * the input into the text field.
				 *
				 * @return Confirmation trigger.
				 */
				virtual Trigger *getTriggerConfirm();
		};
	};
};

#endif
