/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_HIGHSCORES_H
#define MYSDL_HIGHSCORES_H

#include "xml.h"

namespace mysdl
{
	static const std::string XML_HIGHSCORES_HIGHSCORES="highscores";
	static const std::string XML_HIGHSCORES_ENTRY     ="entry";

	/**
	 * Template for implementing a highscore table.
	 *
	 * The table maintains a list of a fixed number of entries, which must
	 * be instances of a class providing the following methods:
	 *
	 * void getFromNode(mysdl::xml::Node *node) - Reads the entry from a node.
	 * void putInNode(mysdl::xml::Node *node) - Writes the entry to a node.
	 *
	 * The class must also contain a paramterless constructor and be
	 * comparable using the ">" operator, so new entries can be placed
	 * correctly in the list.
	 *
	 * @param EntryType Type of highscore entries stored in the list.
	 * @param size      Number of entries in the list.
	 */
	template <typename EntryType,int size=10> class HighscoreTable
	{
		public:
			/** Array of size instances of the entry type class, i.e.
			 *  the actual high score table. */
			EntryType entries[size];

			/**
			 * Clears the highscore table by replacing all entries with
			 * new instances of the entry type.
			 */
			void clear()
			{
				for(unsigned int i=0; i<size; i++)
				{
					entries[i]=EntryType();
				}
			}

			/**
			 * Reads the highscore table from an XML node.
			 *
			 * @param root XML node containing a highscore table.
			 */
			void getFromNode(const xml::Node *root)
			{
				unsigned int i=0;

				for(xml::Nodes::const_iterator itr=root->children.begin(); itr!=root->children.end(); itr++)
				{
					xml::Node *node=*itr;

					if(node->name==XML_HIGHSCORES_ENTRY)
					{
						entries[i].getFromNode(node);
						i++;

						if(i>=size)
						{
							break;
						}
					}
				}
			}

			/**
			 * Builds a DOM tree containing the highscore table's data.
			 *
			 * @return The root node of the created DOM tree.
			 */
			xml::Node *buildDomTree()
			{
				xml::Node *root=new xml::Node(XML_HIGHSCORES_HIGHSCORES.c_str());

				for(unsigned int i=0; i<size; i++)
				{
					xml::Node *node=new xml::Node(XML_HIGHSCORES_ENTRY.c_str());
					entries[i].putInNode(node);
					(*root)+=node;
				}

				return(root);
			}

			/**
			 * Loads the highscore table from a file.
			 *
			 * @param filename Name of the file containing the highscores.
			 */
			void load(std::string filename)
			{
				xml::Node *root=xml::loadXML(filename.c_str());

				if(root)
				{
					if(root->name==XML_HIGHSCORES_HIGHSCORES)
					{
						getFromNode(root);
					}

					delete root;
				}
			}

			/**
			 * Writes the highscore table to a file.
			 *
			 * @param filename Name of the file to be written.
			 */
			void save(std::string filename)
			{
				xml::Node *root=buildDomTree();
				root->saveXML(filename.c_str());
				delete root;
			}

			/**
			 * Determines the position the specified highscore entry
			 * would be placed at if it were added to the highscore table.
			 *
			 * @param entry The entry whose potential position shall be determined.
			 *
			 * @returns Index at which the entry would be placed in the list (0 being
			 *          the top score), or -1 if all entries stored in the list are
			 *          higher.
			 */
			int getPos(const EntryType &entry)
			{
				for(unsigned int i=0; i<size; i++)
				{
					if(entry>entries[i])
					{
						return(i);
					}
				}

				return(-1);
			}

			/**
			 * Adds the specified entry to the table.
			 *
			 * @param entry Entry which will be added to the table.
			 *
			 * @return Index at which the entry was placed, or -1 if it
			 *         was too low to be added to the list.
			 */
			int add(const EntryType &entry)
			{
				int pos=getPos(entry);

				if(pos>=0)
				{
					EntryType newEntry=entry;

					for(unsigned int i=(int)pos; i<size; i++)
					{
						EntryType help=entries[i];
						entries[i]=newEntry;
						newEntry=help;
					}
				}

				return(pos);
			}
	};
};

#endif
