/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_MATH_H
#define MYSDL_MATH_H

#include <math.h>
#include <stdlib.h>

#include "SDL/SDL_opengl.h"

#include "tools.h"

#ifndef M_PI
	#define M_PI 3.141592654
#endif

namespace mysdl
{
	template<typename ValueType> ValueType min(ValueType a,ValueType b)
	{
		return(a<b ? a : b);
	}

	template<typename ValueType> ValueType max(ValueType a,ValueType b)
	{
		return(a>b ? a : b);
	}

	template<typename ValueType> ValueType clamp(ValueType x,ValueType min,ValueType max)
	{
		if(x<min)
		{
			return(min);
		}
		else if(x>max)
		{
			return(max);
		}
		else
		{
			return(x);
		}
	}

  /**
   * Rounds a number to the nearest Uint32 value.
	 *
	 * @param x Value to be rounded.
	 *
	 * @return Rounded value.
   */
  template<typename ResultType,typename ValueType> ResultType round(ValueType x)
	{
		return((ResultType)(x+0.5));
	}

	void randomize();

	template<typename ValueType> ValueType random(ValueType max)
	{
		return(((ValueType)(rand()))/((ValueType)(RAND_MAX))*max);
	}

	template<typename ValueType> ValueType random(ValueType min,ValueType max)
	{
		return(min+random(max-min));
	}

	/*template<int> */int random(int min,int max);/*
	{
    return(min+(int)(((double)(max-min+1))*rand()/(RAND_MAX+1.0)));
	}*/

	/*template<int> */int random(int max);/*
	{
		return(random<int>(0,max));
	}*/

	int random_int(int min,int max);
	int random_int(int max);

	GLfloat random(GLfloat max);
	GLfloat random(GLfloat min,GLfloat max);

	template<typename ValueType> ValueType randomProb(ValueType max,GLfloat *probabilities)
	{
		GLfloat x=random(1.0);
		GLfloat f=0;

		for(ValueType i=(ValueType)(0); i<max; i++)
		{
			f+=probabilities[i];

			if(x<=f)
			{
				return(i);
			}
		}

		return(max);
	}

	const GLfloat PI=3.141592654;
  const GLfloat EPS = 0.0000001;

  template<typename ValueType=GLfloat> class basic_Matrix2D;
  template<typename ValueType=GLfloat> class basic_Matrix3D;

	/**
	 * A two-dimensional basic_Vector.
	 */
	template<typename ValueType=GLfloat> class basic_Vector2D
	{
		public:
			/** basic_Vector coordinates. */
			ValueType x,y;

			/**
			 * Creates a new basic_Vector2D (0/0).
			 */
			basic_Vector2D() : x(0),y(0)
			{
			}

			/**
			 * Creates a new basic_Vector2D using the specified values.
			 *
			 * @param x X coordinate.
			 * @param y Y coordinate.
			 */
			basic_Vector2D(ValueType x,ValueType y) : x(x),y(y)
			{
			}

			/**
			 * Creates a new basic_Vector2D using the values of the specified source vector.
			 *
			 * @param src Source vector
			 */
			basic_Vector2D(const basic_Vector2D<ValueType> &src) : x(src.x),y(src.y)
			{
			}

			void clear()
			{
				x=y=0;
			}

			/**
			 * Returns the length of the basic_Vector.
			 *
			 * @return Length.
			 */
      ValueType length() const
			{
				return(sqrt(x*x+y*y));
			}

			/**
			 * Normalizes the basic_Vector.
			 */
			void normalize()
			{
				ValueType length=sqrt(x*x+y*y);

				if(length<EPS)
				{
					x=0;
					y=0;
				}
				else
				{
					x/=length;
					y/=length;
				}
			}

			void scale(ValueType length)
			{
				normalize();
				x*=length;
				y*=length;
			}
			
			bool operator<(const basic_Vector2D<ValueType> v) const
			{
				if(this->x!=v.x)
				{
					return(this->x<v.x);
				}
				else
				{
					return(this->y<v.y);
				}
			}

			/**
			 * Adds a basic_Vector.
			 *
			 * @param v2d basic_Vector which will be added.
			 */
			basic_Vector2D<ValueType> &operator+=(const basic_Vector2D<ValueType> &v2d)
			{
				x+=v2d.x;
				y+=v2d.y;

				return(*this);
			}

			/**
			 * Subtracts a basic_Vector.
			 *
			 * @param v2d basic_Vector which will be subtracted.
			 */
			basic_Vector2D<ValueType> &operator-=(const basic_Vector2D<ValueType> &v2d)
			{
				x-=v2d.x;
				y-=v2d.y;

				return(*this);
			}

			/**
			 * Adds a basic_Vector.
			 *
			 * @param v2d basic_Vector which will be added.
			 */
			basic_Vector2D<ValueType> operator+(const basic_Vector2D<ValueType> &v2d) const
			{
				static basic_Vector2D<ValueType> result;

				result.x=x+v2d.x;
				result.y=y+v2d.y;

				return(result);
			}

			/**
			 * Subtracts a basic_Vector.
			 *
			 * @param v2d basic_Vector which will be subtracted.
			 */
			basic_Vector2D<ValueType> operator-(const basic_Vector2D<ValueType> &v2d) const
			{
				static basic_Vector2D<ValueType> result;

				result.x=x-v2d.x;
				result.y=y-v2d.y;

				return(result);
			}

			basic_Vector2D<ValueType> operator*(const ValueType a) const
			{
				return(basic_Vector2D<ValueType>(x*a,y*a));
			}

			/**
			 * Divides a basic_Vector by a scalar.
			 *
			 * @param a Scalar divisor.
			 */
			basic_Vector2D<ValueType> operator/(const ValueType a) const
			{
				return(basic_Vector2D<ValueType>(x/a,y/a));
			}

			/**
			 * Transforms a basic_Vector using a basic_Matrix.
			 *
			 * @param m2d Transformation basic_Matrix.
			 */
			void transform(const basic_Matrix2D<ValueType> &m2d);
	};

	/**
	 * A three-dimensional basic_Vector.
	 */
	template<typename ValueType> class basic_Vector3D
	{
		public:
			/** basic_Vector coordinates. */
			ValueType x,y,z;

			/**
			 * Creates a new basic_Vector3D (0/0).
			 */
			basic_Vector3D() : x(),y(),z()
			{
			}

			/**
			 * Copy constructor.
			 *
			 * @param v3d This basic_Vector will be copied.
			 */
			basic_Vector3D(const basic_Vector3D<ValueType> &v3d) : x(v3d.x),y(v3d.y),z(v3d.z)
			{
			}

			/**
			 * Creates a new basic_Vector3D using the specified values.
			 *
			 * @param x X coordinate.
			 * @param y Y coordinate.
			 * @param z Z coordinate.
			 */
			basic_Vector3D(ValueType x,ValueType y,ValueType z) : x(x),y(y),z(z)
			{
			}

			void clear()
			{
				x=y=z=0;
			}

			bool operator<(const basic_Vector3D<ValueType> &v) const
			{
				if(this->x!=v.x)
				{
					return(this->x<v.x);
				}
				else if(this->y!=v.y)
				{
					return(this->y<v.y);
				}
				else
				{
					return(this->z<v.z);
				}
			}

			void setArray(ValueType *array)
			{
				*(array+0)=x;
				*(array+1)=y;
				*(array+2)=z;
			}

			bool checkDeviation(const basic_Vector3D<ValueType> &v,ValueType dev)
			{
				ValueType dx=x-v.x,dy=y-v.y,dz=z-v.z;

				return((dx<-dev)||(dx>dev)||(dy<-dev)||(dy>dev)||(dz<-dev)||(dz>dev));
			}

			/**
			 * Adds a basic_Vector.
			 *
			 * @param v3d basic_Vector which will be added.
			 */
			basic_Vector3D<ValueType> &operator+=(const basic_Vector3D<ValueType> &v3d)
			{
				x+=v3d.x;
				y+=v3d.y;
				z+=v3d.z;

				return(*this);
			}

			/**
			 * Subtracts a basic_Vector.
			 *
			 * @param v3d basic_Vector which will be subtracted.
			 */
			basic_Vector3D<ValueType> &operator-=(const basic_Vector3D<ValueType> &v3d)
			{
				x-=v3d.x;
				y-=v3d.y;
				z-=v3d.z;

				return(*this);
			}

			/**
			 * Multiplies the basic_Vector.
			 *
			 * @param v2d basic_Vector with which this basic_Vector will be multiplied.
			 */
			basic_Vector3D<ValueType> &operator*=(ValueType s)
			{
				x*=s;
				y*=s;
				z*=s;

				return(*this);
			}

			/**
			 * Divides the basic_Vector.
			 *
			 * @param v2d basic_Vector by which this basic_Vector will be divided.
			 */
			basic_Vector3D<ValueType> &operator/=(ValueType s)
			{
				if(s>=EPS)
				{
					x/=s;
					y/=s;
					z/=s;
				}

				return(*this);
			}

			/**
			 * Adds a basic_Vector.
			 *
			 * @param v3d basic_Vector which will be added.
			 */
			basic_Vector3D<ValueType> operator+(const basic_Vector3D<ValueType> &v3d) const
			{
				static basic_Vector3D<ValueType> result;

				result.x=x+v3d.x;
				result.y=y+v3d.y;
				result.z=z+v3d.z;

				return(result);
			}

			/**
			 * Subtracts a basic_Vector.
			 *
			 * @param v3d basic_Vector which will be subtracted.
			 */
			basic_Vector3D<ValueType> operator-(const basic_Vector3D<ValueType> &v3d) const
			{
				static basic_Vector3D<ValueType> result;

				result.x=x-v3d.x;
				result.y=y-v3d.y;
				result.z=z-v3d.z;

				return(result);
			}

			/**
			 * Multiplies the basic_Vector.
			 *
			 * @param v2d basic_Vector with which this basic_Vector will be multiplied.
			 */
			basic_Vector3D<ValueType> operator*(ValueType s) const
			{
				static basic_Vector3D<ValueType> result;

				result.x=x*s;
				result.y=y*s;
				result.z=z*s;

				return(result);
			}

			/**
			 * Divides the basic_Vector.
			 *
			 * @param v2d basic_Vector by which this basic_Vector will be divided.
			 */
			basic_Vector3D<ValueType> operator/(ValueType s) const
			{
				static basic_Vector3D<ValueType> result;

				if(s==0)
				{
					return(basic_Vector3D<ValueType>(0,0,0));
				}
				else
				{
					result.x=x/s;
					result.y=y/s;
					result.z=z/s;
				}

				return(result);
			}

			/**
			 * Compares two basic_Vectors.
			 *
			 * @param v3d Comparison basic_Vector.
			 */
      bool operator==(const basic_Vector3D<ValueType> &v3d) const
			{
				return((x==v3d.x)&&(y==v3d.y)&&(z==v3d.z));
			}

			bool operator!=(const basic_Vector3D<ValueType> &v3d) const
			{
				return((x!=v3d.x)||(y!=v3d.y)||(z!=v3d.z));
			}

			/**
			 * Calculates the cross product between this basic_Vector and
			 * another basic_Vector and stores it in this basic_Vector.
			 *
			 * @param v3d Second basic_Vector.
			 */
			void crossProduct(const basic_Vector3D<ValueType> &v3d)
			{
				static ValueType tmpx,tmpy;

				tmpx=(y*v3d.z)-(z*v3d.y);
				tmpy=(z*v3d.x)-(x*v3d.z);

				z=(x*v3d.y)-(y*v3d.x);

				x=tmpx;
				y=tmpy;
			}

			/**
			 * Calculates the dot product between this basic_Vector and
			 * another basic_Vector and stores it in this basic_Vector.
			 *
			 * @param v3d Second basic_Vector.
			 */
			ValueType dotProduct(const basic_Vector3D<ValueType> &v3d)
			{
				return((x*v3d.x)+(y*v3d.y)+(z*v3d.z));
			}

			/**
			 * Transforms a basic_Vector using a basic_Matrix.
			 *
			 * @param m3d Transformation basic_Matrix.
			 */
			void transform(const basic_Matrix3D<ValueType> &m3d);

			/**
			 * Returns the point on line (start,end) closest to this point
			 *
			 * @param start Starting point of the line.
			 * @param end   Ending point of the line.
			 */
      basic_Vector3D<ValueType> closestPointOnLine(const basic_Vector3D<ValueType> &start,const basic_Vector3D<ValueType> &end) const;

			/**
			 * Returns the distance of this point to another one
			 *
			 * @param to The distance to this basic_Vector will be mesured.
			 */
      ValueType distance(const basic_Vector3D<ValueType> &to) const
			{
				basic_Vector3D<ValueType> fromto=(*this)-to;
				return(sqrt(fromto.x*fromto.x+fromto.y*fromto.y+fromto.z*fromto.z));
			}

			/**
			 * Returns the length of the basic_Vector.
			 *
			 * @return Length.
			 */
      ValueType length() const
			{
				return(sqrt(x*x+y*y+z*z));
			}

			/**
			 * Normalizes the basic_Vector.
			 */
			void normalize()
			{
				//debug::addMessage("basic_Vector3D::normalize()");

				ValueType length=sqrt(x*x+y*y+z*z);

				if(length<EPS)
				{
					x=0;
					y=0;
					z=0;
				}
				else
				{
					x/=length;
					y/=length;
					z/=length;
				}

				//debug::removeMessage();
			}
	};

	/**
		* Calculates the cross product between two basic_Vectors.
		*
		* @param v1 First basic_Vector.
		* @param v2 Second basic_Vector.
		*
		* @return Cross product.
		*/
	template<typename ValueType> inline basic_Vector3D<ValueType> crossProduct(const basic_Vector3D<ValueType> &v1,const basic_Vector3D<ValueType> &v2)
	{
		static basic_Vector3D<ValueType> result;

		result.x=(v1.y*v2.z)-(v1.z*v2.y);
		result.y=(v1.z*v2.x)-(v1.x*v2.z);
		result.z=(v1.x*v2.y)-(v1.y*v2.x);

		return(result);
	}

	/**
		* Calculates the dot product between two basic_Vectors.
		*
		* @param v1 First basic_Vector.
		* @param v2 Second basic_Vector.
		*
		* @return Dot product.
		*/
	template<typename ValueType> inline ValueType dotProduct(const basic_Vector3D<ValueType> &v1,const basic_Vector3D<ValueType> &v2)
	{
  	return((v1.x*v2.x)+(v1.y*v2.y)+(v1.z*v2.z));
	}

  template<typename ValueType> inline basic_Vector3D<ValueType> closestPointOnLine(const basic_Vector3D<ValueType> &start,const basic_Vector3D<ValueType> &end,
                              const basic_Vector3D<ValueType> &point)
  {
    basic_Vector3D<ValueType> line=end-start;
    ValueType length=line.length();

    if(length<EPS)
		{
			return(start);
		}

    line/=length;

    basic_Vector3D<ValueType> v1=point-start;
    ValueType t=dotProduct(line,v1);

    if(t<=0)
		{
			return(start);
		}
		else if(t>=length)
		{
			return(end);
		}
		else
		{
    	return(start+(line*t));
		}
  }

	template<typename ValueType> class basic_Matrix2D
	{
		public:
			ValueType m[3][3];

			basic_Matrix2D()
			{
				//debug::addMessage("basic_Matrix2D::basic_Matrix2D()");
				setIdentity();
				//debug::removeMessage();
			}

			basic_Matrix2D &operator*=(const basic_Matrix2D<ValueType> &m2d)
			{
				multRight(m2d);
				return(*this);
			}

			void setIdentity()
			{
				for(int i=0; i<3; i++)
				{
					for(int j=0; j<3; j++)
					{
						m[i][j]=(i==j) ? 1 : 0;
					}
				}
			}

			void setRotate(ValueType angle,bool inRadians=false) /** \todo remove inRadians (for speed) */
			{
				//debug::addMessage("basic_Matrix2D::setRotate(ValueType,bool=false)");

				double radians=inRadians ? angle : (angle/180.0)*M_PI;

				double cosr=cos(radians);
				double sinr=sin(radians);

				m[0][0]=cosr;  m[0][1]=-sinr;  m[0][2]=0.0;
				m[1][0]=sinr;	 m[1][1]= cosr;  m[1][2]=0.0;
				m[2][0]=0.0;   m[2][1]= 0.0;   m[2][2]=1.0;

				//debug::removeMessage();
			}

			void setTranslate(const ValueType x,const ValueType y)
			{
				m[0][0]=1.0;  m[0][1]=0.0;  m[0][2]=x;
				m[1][0]=0.0;  m[1][1]=1.0;  m[1][2]=y;
				m[2][0]=0.0;  m[2][1]=0.0;  m[2][2]=1.0;
			}

			void setTranslate(const basic_Vector2D<ValueType> &v2d)
			{
				setTranslate(v2d.x,v2d.y);
			}

			void setScale(const ValueType x,const ValueType y)
			{
				m[0][0]=x;    m[0][1]=0.0;  m[0][2]=0.0;
				m[1][0]=0.0;  m[1][1]=y;    m[1][2]=0.0;
				m[2][0]=0.0;  m[2][1]=0.0;  m[2][2]=1.0;
			}

			ValueType subDeterminant(int i,int k) const
			{
				if((i>2)||(i<0)||(k>2)||(k<0))
				{
					return(0.0);
				}
				else
				{
					return(m[0<i?0:1][0<k?0:1]*m[1<i?1:2][1<k?1:2]-
							   m[0<i?0:1][1<k?1:2]*m[1<i?1:2][0<k?0:1]);
				}
			}

			ValueType adjoint(int i,int k) const
			{
				if((i>2)||(i<0)||(k>2)||(k<0))
				{
					return(0.0);
				}
				else
				{
					return(pow(-1.0,i+k)*subDeterminant(i,k));
				}
  		}

			ValueType determinant() const
			{
				return(m[0][0]*m[1][1]*m[2][2]+m[0][1]*m[1][2]*m[2][0]+
							 m[0][2]*m[1][0]*m[2][1]-m[0][2]*m[1][1]*m[2][0]-
							 m[0][0]*m[1][2]*m[2][1]-m[0][1]*m[1][0]*m[2][2]);
			}

      basic_Matrix2D<ValueType> inverse() const
			{
				return(inverse(determinant()));
			}

      basic_Matrix2D<ValueType> inverse(ValueType det) const
			{
				if(det==0.0)
				{
					return(*this); // basic_Matrix is singular
				}

				static basic_Matrix2D<ValueType> inverse;//=*this;

				for(int i=0; i<3; i++)
				{
					for (int k=0; k<3; k++)
					{
						inverse.m[i][k]=adjoint(k,i)*(1.0/det);
					}
				}

				return(inverse);
			}

      basic_Matrix2D<ValueType> &invert()
			{
				return(invert(determinant()));
			}

      basic_Matrix2D<ValueType> &invert(ValueType det)
			{
				if(det==0.0)
				{
					return(*this); // basic_Matrix is singular
				}

				basic_Matrix2D<ValueType> temp=*this;

				for(int i=0; i<3; i++)
				{
					for (int k=0; k<3; k++)
					{
						m[i][k]=temp.adjoint(k,i)*(1.0/det);
					}
				}

				return(*this);
			}

			basic_Matrix2D<ValueType> &multRight(const basic_Matrix2D<ValueType> &right)
			{
				//debug::addMessage("basic_Matrix2D::multRight()");

				basic_Matrix2D<ValueType> temp=*this;

				for(int r=0; r<3; r++)
				{
					for(int c=0; c<3; c++)
					{
						m[r][c]=0;

						for(int i=0; i<3; i++)
						{
							m[r][c]+=temp.m[r][i]*right.m[i][c];
						}
					}
				}

				//debug::removeMessage();

				return(*this);
			}

			basic_Matrix2D<ValueType> &multLeft(const basic_Matrix2D<ValueType> &left)
			{
				//debug::addMessage("basic_Matrix2D::multLeft()");

				basic_Matrix2D<ValueType> temp=*this;

				for(int r=0; r<3; r++)
				{
					for(int c=0; c<3; c++)
					{
						m[r][c]=0;

						for(int i=0; i<3; i++)
						{
							m[r][c]+=left.m[r][i]*temp.m[i][c];
						}
					}
				}

				//debug::removeMessage();

				return(*this);
			}

			basic_Vector2D<ValueType> transformVector(const basic_Vector2D<ValueType> &v2d) const
			{
				static basic_Vector2D<ValueType> result;

				result.x=m[0][0]*v2d.x+m[0][1]*v2d.y+m[0][2];
				result.y=m[1][0]*v2d.x+m[1][1]*v2d.y+m[1][2];

				return(result);
			}
	};

	template<typename ValueType> class basic_Matrix3D
	{
		public:
			ValueType m[4][4];

			basic_Matrix3D()
			{
				//debug::addMessage("basic_Matrix3D::basic_Matrix3D()");
				setIdentity();
				//debug::removeMessage();
			}

			basic_Matrix3D(const basic_Matrix3D<ValueType> &clone)
			{
				//debug::addMessage("basic_Matrix3D::basic_Matrix3D(const basic_Matrix3D&)");

				for (int i=0; i<4; i++)
				{
					for (int j=0; j<4; j++)
					{
						m[i][j]=clone.m[i][j];
					}
				}

				//debug::removeMessage();
			}

			basic_Matrix3D<ValueType> getTransposed() const
			{
				basic_Matrix3D<ValueType> basic_Matrix;

				for(int i=0; i<4; i++)
				{
					for(int j=0; j<4; j++)
					{
						basic_Matrix.m[i][j]=m[j][i];
					}
				}

				return(basic_Matrix);
			}

			void transpose()
			{
				basic_Matrix3D<ValueType> basic_Matrix=*this;
				*this=basic_Matrix.getTransposed();
			}

			std::string toString()
			{
				//debug::addMessage("basic_Matrix3D::toString()");

				static char buffer[1000];

				std::string s;

				for(int i=0; i<4; i++)
				{
					for(int j=0; j<4; j++)
					{
						sprintf(buffer,"[%.2f]",m[i][j]);
						s+=buffer;
					}
				}

				//debug::removeMessage();

				return(s);
			}

			basic_Matrix3D<ValueType> &operator*=(const basic_Matrix3D& m3d)
			{
				multRight(m3d);
				return(*this);
			}

			void setIdentity()
			{
				for (int i=0; i<4; i++)
				{
					for (int j=0; j<4; j++)
					{
						m[i][j]=(i==j) ? 1 : 0;
					}
				}
			}

			void setRotate(ValueType angle,ValueType x,ValueType y,ValueType z,bool inRadians=false)
			{
				//debug::addMessage("basic_Matrix3D::setRotate(ValueType,ValueType,ValueType,ValueType,bool=false)");

				setRotate(angle,basic_Vector3D<ValueType>(x,y,z),inRadians);

				//debug::removeMessage();
			}

			void setRotate(ValueType angle,const basic_Vector3D<ValueType> &axis,bool inRadians=false)
			{
				//debug::addMessage("basic_Matrix3D::setRotate(ValueType,basic_Vector3D&,bool=false)");

				double radians=inRadians ? angle : (angle/180.0)*M_PI;

				double cosr=cos(radians);
				double sinr=sin(radians);

				basic_Vector3D<ValueType> u=axis;
				u.normalize();

				m[0][0]=u.x*u.x*(1-cosr)+cosr;
				m[1][0]=u.y*u.x*(1-cosr)+u.z*sinr;
				m[2][0]=u.z*u.x*(1-cosr)-u.y*sinr;
				m[3][0]=0.0;

				m[0][1]=u.x*u.y*(1-cosr)-u.z*sinr;
				m[1][1]=u.y*u.y*(1-cosr)+cosr;
				m[2][1]=u.z*u.y*(1-cosr)+u.x*sinr;
				m[3][1]=0.0;

				m[0][2]=u.x*u.z*(1-cosr)+u.y*sinr;
				m[1][2]=u.y*u.z*(1-cosr)-u.x*sinr;
				m[2][2]=u.z*u.z*(1-cosr)+cosr;
				m[3][2]=0.0;

				m[0][3]=0.0;
				m[1][3]=0.0;
				m[2][3]=0.0;
				m[3][3]=1.0;

				//debug::removeMessage();
			}

			void setTranslate(ValueType x,ValueType y,ValueType z)
			{
				m[0][0]=1.0;  m[0][1]=0.0;  m[0][2]=0.0;  m[0][3]=x;
				m[1][0]=0.0;  m[1][1]=1.0;  m[1][2]=0.0;  m[1][3]=y;
				m[2][0]=0.0;  m[2][1]=0.0;  m[2][2]=1.0;  m[2][3]=z;
				m[3][0]=0.0;  m[3][1]=0.0;  m[3][2]=0.0;  m[3][3]=1.0;
			}

			void setTranslate(const basic_Vector3D<ValueType> &v3d)
			{
				setTranslate(v3d.x,v3d.y,v3d.z);
			}

			void setScale(ValueType x,ValueType y,ValueType z)
			{
				m[0][0]=x;    m[0][1]=0.0;  m[0][2]=0.0;  m[0][3]=0.0;
				m[1][0]=0.0;  m[1][1]=y;    m[1][2]=0.0;  m[1][3]=0.0;
				m[2][0]=0.0;  m[2][1]=0.0;  m[2][2]=z;    m[2][3]=0.0;
				m[3][0]=0.0;  m[3][1]=0.0;  m[3][2]=0.0;  m[3][3]=1.0;
			}

			ValueType subDeterminant(int i,int k) const
			{
				if((i>3)||(i<0)||(k>3)||(k<0))
				{
					return(0.0);
				}

				static basic_Matrix2D<ValueType> subbasic_Matrix;

				for(int h=0; h<3; h++)
				{
					for(int j=0; j<3; j++)
					{
						subbasic_Matrix.m[h][j]=m[h<i?h:h+1][j<k?j:j+1];
					}
				}

				return(subbasic_Matrix.determinant());
			}

			ValueType adjoint(int i,int k) const
			{
				if((i>3)||(i<0)||(k>3)||(k<0))
				{
					return(0.0);
				}
				else
				{
					return(pow(-1.0,i+k)*subDeterminant(i,k));
				}
			}

			ValueType determinant(int i=3,bool row=true) const
			{
				if((i>3)||(i<0))
				{
					return(0.0);
				}

				if(row)
				{
					return(m[i][0]*adjoint(i,0)+
								 m[i][1]*adjoint(i,1)+
								 m[i][2]*adjoint(i,2)+
								 m[i][3]*adjoint(i,3));
				}
				else
				{
					return(m[0][i]*adjoint(0,i)+
								 m[1][i]*adjoint(1,i)+
								 m[2][i]*adjoint(2,i)+
								 m[3][i]*adjoint(3,i));
				}
			}

      basic_Matrix3D<ValueType> inverse() const
			{
				return(inverse(determinant()));
			}

			basic_Matrix3D<ValueType> inverse(ValueType det) const
			{
				if(det==0.0)
				{
					return(*this); // basic_Matrix is singular
				}

				static basic_Matrix3D<ValueType> inverse;

				for(int i=0; i<4; i++)
				{
					for(int k=0; k<4; k++)
					{
						inverse.m[i][k]=adjoint(k,i)*(1.0/det);
					}
				}

				return(inverse);
			}

      basic_Matrix3D<ValueType> &invert()
			{
				return(invert(determinant()));
			}

      basic_Matrix3D<ValueType> &invert(ValueType det)
			{
				if(det==0.0)
				{
					return(*this); // basic_Matrix is singular
				}

				basic_Matrix3D<ValueType> temp=*this;

				for(int i=0; i<4; i++)
				{
					for(int k=0; k<4; k++)
					{
						m[i][k]=temp.adjoint(k,i)*(1.0/det);
					}
				}

				return(*this);
			}

			basic_Matrix3D<ValueType> &multRight(const basic_Matrix3D<ValueType> &right)
			{
				//debug::addMessage("basic_Matrix3D::multRight()");

				basic_Matrix3D<ValueType> temp=*this;

				for(int r=0; r<4; r++)
				{
					for(int c=0; c<4; c++)
					{
						m[r][c]=0;

						for(int i=0; i<4; i++)
						{
							m[r][c]+=temp.m[r][i]*right.m[i][c];
						}
					}
				}

				//debug::removeMessage();

				return(*this);
			}

			basic_Matrix3D<ValueType> &multLeft(const basic_Matrix3D<ValueType> &left)
			{
				basic_Matrix3D<ValueType> temp=*this;

				//debug::addMessage("basic_Matrix3D::multLeft()");

				for(int r=0; r<4; r++)
				{
					for(int c=0; c<4; c++)
					{
						m[r][c]=0;

						for(int i=0; i<4; i++)
						{
							m[r][c]+=left.m[r][i]*temp.m[i][c];
						}
					}
				}

				//debug::removeMessage();

				return(*this);
			}

			basic_Vector3D<ValueType> transformVector(const basic_Vector3D<ValueType> &v3d) const
			{
				basic_Vector3D<ValueType> result;

				result.x=m[0][0]*v3d.x+m[0][1]*v3d.y+m[0][2]*v3d.z+m[0][3];
				result.y=m[1][0]*v3d.x+m[1][1]*v3d.y+m[1][2]*v3d.z+m[1][3];
				result.z=m[2][0]*v3d.x+m[2][1]*v3d.y+m[2][2]*v3d.z+m[2][3];

				return(result);
			}
	};

	template<typename ValueType> inline void basic_Vector2D<ValueType>::transform(const basic_Matrix2D<ValueType> &m2d)
	{
		//debug::addMessage("basic_Vector2D::transform()");

		static ValueType tmpx;

		tmpx=x*m2d.m[0][0]+y*m2d.m[0][1]+m2d.m[0][2];
		y   =x*m2d.m[1][0]+y*m2d.m[1][1]+m2d.m[1][2];

		x=tmpx;

		//debug::removeMessage();
	}

	template<typename ValueType> inline void basic_Vector3D<ValueType>::transform(const basic_Matrix3D<ValueType> &m3d)
	{
		static ValueType tmpx,tmpy;

		tmpx=x*m3d.m[0][0]+y*m3d.m[0][1]+z*m3d.m[0][2]+m3d.m[0][3];
		tmpy=x*m3d.m[1][0]+y*m3d.m[1][1]+z*m3d.m[1][2]+m3d.m[1][3];

		z   =x*m3d.m[2][0]+y*m3d.m[2][1]+z*m3d.m[2][2]+m3d.m[2][3];

		x=tmpx;
		y=tmpy;
	}

	template<typename ValueType> inline basic_Vector3D<ValueType> basic_Vector3D<ValueType>::closestPointOnLine(const basic_Vector3D<ValueType> &start,const basic_Vector3D<ValueType> &end) const
	{
		basic_Vector3D<ValueType> line=end-start;
		ValueType length=line.length();

		if(length<EPS)
		{
			return(start);
		}

		line/=length;

		basic_Vector3D<ValueType> v1=(*this)-start;
		ValueType t=mysdl::dotProduct(line,v1);

		if(t<=0)
		{
			return(start);
		}
		else if(t>=length)
		{
			return(end);
		}
		else
		{
			return(start+(line*t));
		}
	}

	typedef basic_Vector2D<GLfloat> Vector2D;
	typedef basic_Vector3D<GLfloat> Vector3D;
	typedef basic_Matrix2D<GLfloat> Matrix2D;
	typedef basic_Matrix3D<GLfloat> Matrix3D;

	template<typename ValueType> class Rectangle
	{
		public:
			basic_Vector2D<ValueType> position;
			basic_Vector2D<ValueType> dimension;
			
			Rectangle() :
					position(),dimension()
					{
					};
			
			Rectangle(basic_Vector2D<ValueType> position,basic_Vector2D<ValueType> dimension) :
					position(position),dimension(dimension)
					{
					};
			
			Rectangle(ValueType x,ValueType y,ValueType width,ValueType height) :
					position(x,y),dimension(width,height)
					{
					};
	};	
};

#endif
