/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_MYSDL_H
#define MYSDL_MYSDL_H

#include <iomanip>
#include <sstream>
#include <string>

#include "SDL/SDL.h"
#include "SDL/SDL_opengl.h"

#include "math.h"
#include "xml.h"

namespace mysdl
{
	// --- Define which platforms have lossy surfaces, i.e. surfaces  ---
	// --- which may get lost during video mode changes and therefore ---
	// --- have to be reloaded every time an application switches     ---
	// --- resolutions or full screen mode.                           ---
	#ifdef win32
		#define LOSSY_SURFACES
	#endif

	/** Basic data types for the MySDL system. */
	typedef GLfloat myfloat;
	typedef int myint;
	typedef unsigned int myuint;

	/** System debug group constants. */
	static const int dbgCore=-1;
	static const int dbgControls=-2;
	static const int dbgGui=-3;
	static const int dbgDdd=-4;
	static const int dbgMath=-5;
	static const int dbgSprites=-6;
	static const int dbgImages=-7;
	static const int dbgFonts=-8;
	static const int dbgXml=-9;
	static const int dbgAudio=-10;
	static const int dbgCollision=-11;
	static const int dbgAnimations=-12;
	static const int dbgResources=-13;

	/** Horizontal text/image justification. */
	enum JustificationHorizontal {jhLeft,jhCenter,jhRight};

	/** Vertical text/image justification. */
	enum JustificationVertical {jvTop,jvCenter,jvBottom};

	/** String representations of the JustificationHorizontal values. */
	extern const char *STR_JUSTIFICATION_HORIZONTAL[];

	/** String representations of the JustificationVertical values. */
	extern const char *STR_JUSTIFICATION_VERTICAL[];

	/**
	 * Converts a string into a JustificationHorizontal value.
	 *
	 * @param str String representing a horizontal justification value.
	 *
	 * @return Justification value.
	 */
	extern JustificationHorizontal justificationHorizontal(const char *str); // \todo toStr

	/**
	 * Converts a string into a JustificationVertical value.
	 *
	 * @param str String representing a vertical justification value.
	 *
	 * @return Justification value.
	 */
	extern JustificationVertical justificationVertical(const char *str); // \todo toStr

	/**
	 * Different blend modes for drawing sprites.
	 *
	 * @param bmDefault No specific blend mode, used to specify that a sprite shall
	 *                  use the default blend mode set in its engine.
	 *
	 * @param bmNormal  Pixels are drawn using their alpha value for the source
	 *                  and their complementary alpha value for the destination pixel.
	 *
	 * @param bmAdd     Pixels are drawn using their alpha value for the source
	 *                  and 1 for the destination pixel's alpha.
	 */
	enum BlendMode {bmDefault,bmNormal,bmAdd};

	/**
	 * Sets OpenGL to use the specified blend mode.
	 *
	 * @param mode Desired blend mode.
	 */
	void setBlendMode(BlendMode mode);

	/**
	 * A rectangle.
	 */
	template<typename ValueType> class basic_Rect
	{
		public:
			/** Position (lower left corned). */
			basic_Vector2D<ValueType> pos;

			/** Dimensions (vector from lower left to upper right corner). */
			basic_Vector2D<ValueType> dimensions;

			/**
			 * Creates a "zero rectangle" at position 0/0 and with dimensions 0/0.
			 */
			basic_Rect() : pos(),dimensions()
			{
			}

			/**
			 * Creates a copy of the specified rectangle.
			 *
			 * @param rect Rectangle to be copied.
			 */
			basic_Rect(const basic_Rect<ValueType> &rect) : pos(rect.pos),dimensions(rect.dimensions)
			{
			}

			/**
			 * Creates a new rectangle.
			 *
			 * @param pos        Position (lower left corned).
			 * @param dimensions Dimensions (vector from lower left to upper right corner).
			 *                   Can be omitted, in which case the dimensions will be set
			 *                   to 0/0.
			 */
			basic_Rect(basic_Vector2D<ValueType> pos,basic_Vector2D<ValueType> dimensions=Vector2D()) :
				pos(pos),dimensions(dimensions)
			{
			}

			/**
			 * Creates a new rectangle.
			 *
			 * @param x Left border.
			 * @param y Lower border.
			 * @param w Width.
			 * @param h Height.
			 */
			basic_Rect(ValueType x,ValueType y,ValueType w,ValueType h) :
				pos(basic_Vector2D<ValueType>(x,y)),dimensions(basic_Vector2D<ValueType>(w,h))
			{
			}

			/**
			 * Clears the rectangle by settings its pos and dimensions vectors
			 * to default values.
			 */
			void clear()
			{
				pos.clear();
				dimensions.clear();
			}

			/**
			 * Copies data from another rectangle.
			 *
			 * @param rect Source rectangle.
			 */
			void operator=(basic_Rect<ValueType> rect)
			{
				pos=rect.pos;
				dimensions=rect.dimensions;
			}

			/**
			 * Moves the rectangle by the specified offset. This is done by adding the
			 * offset vector to the rectangle's pos vector.
			 *
			 * @param v Move offset.
			 */
			void offset(basic_Vector2D<ValueType> v)
			{
				pos.x+=v.x;
				pos.y+=v.y;
			}

			/**
			 * Reduces each side of the rectangle by the specified amount.
			 *
			 * @param border The rectangle will be reduced by this value.
			 */
			void reduceAll(ValueType border)
			{
				pos.x+=border;
				pos.y+=border;
				dimensions.x-=border*2;
				dimensions.y-=border*2;
			}

			/**
			 * Reduces the left side of the rectangle by the specified amount.
			 *
			 * @param border The rectangle will be reduced by this value.
			 */
			void reduceLeft(ValueType border)
			{
				pos.x+=border;
				dimensions.x-=border;
			}

			/**
			 * Reduces the right side of the rectangle by the specified amount.
			 *
			 * @param border The rectangle will be reduced by this value.
			 */
			void reduceRight(ValueType border)
			{
				dimensions.x-=border;
			}

			/**
			 * Reduces the top side of the rectangle by the specified amount.
			 *
			 * @param border The rectangle will be reduced by this value.
			 */
			void reduceTop(ValueType border)
			{
				dimensions.y-=border;
			}

			/**
			 * Reduces the bottom side of the rectangle by the specified amount.
			 *
			 * @param border The rectangle will be reduced by this value.
			 */
			void reduceBottom(ValueType border)
			{
				pos.y+=border;
				dimensions.y-=border;
			}

			/**
			 * Tests if the rectangle's area is bigger than that of another rectangle.
			 *
			 * @param rect Other rectangle for comparison.
			 *
			 * @return <code>true</code>, if this rectangle's area is bigger than the
			 *         other rectangle's.
			 */
			bool operator>(const basic_Rect<ValueType> &rect)
			{
				return(dimensions.x*dimensions.y>rect.dimensions.x*rect.dimensions.y);
			}

			/**
			 * Tests if this rectangle overlaps with a second rectangle.
			 *
			 * @param rect Second rectangle.
			 *
			 * @return <code>true</code>, if the two rectangles overlap.
			 */
			bool overlap(const basic_Rect<ValueType> &rect)
			{
				if(rect.pos.x+rect.dimensions.x<pos.x)
				{
					return(false);
				}

				if(rect.pos.x>=pos.x+dimensions.x)
				{
					return(false);
				}

				if(rect.pos.y+rect.dimensions.y<pos.y)
				{
					return(false);
				}

				if(rect.pos.y>=pos.y+dimensions.y)
				{
					return(false);
				}

				return(true);
			}
	};

	/** A rectangle using GLfloat vectors. \todo is there a way to call the template the same as this type? */
	typedef basic_Rect<GLfloat> Rect;

	/** A rectangle using int vectors. */
	typedef basic_Rect<int> RectInt;

	/**
	 * Represents a 24 bit RGB colour with 8 bit alpha channel.
	 */
	class Color
	{
		public:
			/** Normalized RGBA values. */
			GLfloat r,g,b,a;

			/**
			 * Creates a new default colour (black).
			 */
			Color() : r(0),g(0),b(0),a(0)
			{
			}

			/**
			 * Creates a new colour from the specified values.
			 *
			 * @param r Normalized red value.
			 * @param g Normalized green value.
			 * @param b Normalized blue value.
			 * @param a Normalized alpha value.
			 */
			Color(const GLfloat r,const GLfloat g,const GLfloat b,const GLfloat a) :
				r(r),g(g),b(b),a(a)
			{
			}

			/**
			 * Creates a new colour, reading its data from the specified
			 * DOM node.
			 *
			 * @param node DOM node containing colour data.
			 */
			Color(const xml::Node *node);

			/**
			 * Creates a new colour from the specified hex string.
			 *
			 * @param str 32 bit RGBA hex string.
			 */
			Color(const char *str) : r(0),g(0),b(0),a(0)
			{
				set(str);
			}

			/**
			 * Sets the colour to the colour represented by the specified hex string.
			 *
			 * @param str 32 bit RGBA hex string.
			 */
			void set(const char *str)
			{
				unsigned long l=strtoul(str,0,16);

				r=((l>>24)&0xFF)/255.0;
				g=((l>>16)&0xFF)/255.0;
				b=((l>>8)&0xFF)/255.0;
				a=(l&0xFF)/255.0;
			}

			/**
			 * Sets the colour as the current colour.
			 */
			void apply() const;

			/**
			 * Adds a second color. This simply adds the second color's
			 * red, green, blue and alpha values to this colour's values.
			 *
			 * The colour's components are truncated to 1 if they exceed this value.
			 * \todo do we really want this??
			 *
			 * @param c Second color.
			 */
			void operator+=(const Color &c)
			{
				r+=c.r;
				g+=c.g;
				b+=c.b;
				a+=c.a;

				if(r>1)
				{
					r=1;
				}

				if(g>1)
				{
					r=1;
				}

				if(b>1)
				{
					r=1;
				}

				if(a>1)
				{
					r=1;
				}
			}

			/**
			 * Multiplies this colour with a second colour. This is done by
			 * multiplying this colour's red, green and alpha values with those
			 * of the other colour.
			 *
			 * The colour's components are truncated to 1 if they exceed this value.
			 * \todo do we really want this??
			 *
			 * @param c Second colour.
			 */
			void operator*=(const Color &c)
			{
				r*=c.r;
				g*=c.g;
				b*=c.b;
				a*=c.a;

				if(r>1)
				{
					r=1;
				}

				if(g>1)
				{
					r=1;
				}

				if(b>1)
				{
					r=1;
				}

				if(a>1)
				{
					r=1;
				}
			}

			/**
			 * Creates a new colour by multiplying this colour with a constant
			 * factor. This is done by multiplying the colour's red, green, blue
			 * and alpha values with the specified factor.
			 *
			 * The colour's components are truncated to 1 if they exceed this value.
			 * \todo do we really want this??
			 *
			 * @param x Factor.
			 *
			 * @return Multiplied colour.
			 */
			Color operator*(const GLfloat x)
			{
				Color cr;

				cr.r=r*x;
				cr.g=g*x;
				cr.b=b*x;
				cr.a=a*x;

				if(cr.r>1)
				{
					cr.r=1;
				}

				if(cr.g>1)
				{
					cr.r=1;
				}

				if(cr.b>1)
				{
					cr.r=1;
				}

				if(cr.a>1)
				{
					cr.r=1;
				}

				return(cr);
			}

			/**
			 * Subtracts a second color. This simply subtractss the second color's
			 * red, green, blue and alpha values from this colour's values.
			 *
			 * The colour's components are truncated to 0 if they become negative.
			 * \todo do we really want this??
			 *
			 * @param c Second color.
			 */
			void operator-=(const Color &c)
			{
				r=(r<c.r ? 0 : r-c.r);
				g=(g<c.g ? 0 : g-c.g);
				b=(b<c.b ? 0 : b-c.b);
				a=(a<c.a ? 0 : a-c.a);
			}
	};

	class Viewport;
	class RenderContext;

	/** Pointer to the current viewport. */
	extern Viewport *currentViewport;

	/** Viewport representing the full screen (or application window). */
	extern Viewport vpScreen;

	/** The currently active render context. Should be considered read-only. */
	extern RenderContext *currentRenderContext;

	/**
	 * A viewport represents a drawing area on the screen. The viewport
	 * rectangle is specified by its bottom left coordinate and its
	 * dimensions.
	 */
	class Viewport
	{
		public:
			/** Bottom left coordinate, in pixels. */
			Vector2D pos;

			/** Width and height, in pixels. */
			Vector2D dimensions;

			/**
			 * Creates a new viewport.
			 */
			Viewport() : pos(),dimensions()
			{
			}

			/**
			 * Releases the viewport.
			 */
			~Viewport()
			{
				if(currentViewport==this)
				{
					currentViewport=(this==&vpScreen ? 0 : &vpScreen);
				}
			}

			/**
			 * Creates a new viewport.
			 *
			 * @param x      Left coordinate, in pixels.
			 * @param y      Bottom coordinate, in pixels.
			 * @param width  Width, in pixels.
			 * @param height Height, in pixels.
			 */
			Viewport(GLfloat x,GLfloat y,GLfloat width,GLfloat height) :
				pos(x,y),dimensions(width,height)
			{
			}

			/**
			 * Creates a new viewport.
			 *
			 * @param pos 			 Position (bottom left corner).
			 * @param dimensions Dimensions (width and height).
			 */
			Viewport(const Vector2D &pos,const Vector2D &dimensions) :
				pos(pos),dimensions(dimensions)
			{
			}

			/**
			 * Applies a viewport, meaning that from this point on,
			 * drawing operations will refer to the specified viewport
			 * rectangle.
			 */
			void apply()
			{
				currentViewport=this;
				glViewport((GLint)pos.x,(GLint)pos.y,(GLsizei)dimensions.x,(GLsizei)dimensions.y);
			}

			/**
			 * Determines the aspect ratio (width/height) of the
			 * viweport.
			 *
			 * @return Aspect ratio.
			 */
			GLfloat getAspectRatio()
			{
				return(dimensions.x/dimensions.y);
			}
	};

	/** Joystick handle type. */
	typedef SDL_Joystick Joystick;

	/** \todo doc or change */
	enum SubSystem {ssNone=0,
									ssVideo=SDL_INIT_VIDEO,
									//ssAudio=SDL_INIT_AUDIO, // \todo deprecated? probably conflicts with openal?
									ssJoystick=SDL_INIT_JOYSTICK/*,
									ssAll=SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_JOYSTICK*/};

	/**
	 * Initializes MySDL and the specified subsystems.
	 *
	 * @param systems Sub systems which shall be initialized.
	 *
	 * @param debug   Whether to enable MySDL debugging features, like
	 *                setting the default signal handler (recommended).
	 *
	 * @throws std::string When an error occurs.
	 */
	void init(SubSystem systems,bool debug=true);

	/**
	 * Frees all MySDL resources.
	 */
	void cleanUp();

	/**
	 * Cleanly terminates the program, freeing all MySDL resources.
	 */
	void quit();

	/**
	 * Initializes the screen, setting the clear colour to black.
	 *
	 * @param width      Screen width in pixels.
	 * @param height     Screen height in pixels.
	 * @param bpp        Number of bitplanes.
	 * @param fullscreen If this is <code>true</code>, the program will switch to fullscreen mode.
	 * @param depth      Desired depth buffer depth (in bits). By default, no depth buffer is allocated.
	 * @param stencil    Desired stencil buffer depth (in bits). By default, no stencil buffer is allocated.
	 *
	 * @throws std::string on error.
	 */
	void setResolution(int width,int height,int bpp,bool fullscreen,int depth=0,int stencil=0);

	/**
	 * Sets the program window's caption.
	 *
	 * @param caption New window caption.
	 */
	void setWindowCaption(char *const caption);
	
	/**
	 * Sets the program window's icon.
	 * 
	 * @param imageFileName Name of the image file to be used as the icon.
	 */
	void setWindowIcon(const char *imageFileName);

	/**
	 * Shows or hides the cursor within the program's window.
	 *
	 * @param show <code>true</code> to show,
	 *             <code>false</code> to hide the cursor.
	 */
	void showCursor(bool show);

	/**
	 * Enables or disables double buffering.
	 *
	 * @param enable <code>true</code> to enable,
	 *               <code>false</code> to disable double buffering.
	 */
	void enableDoubleBuffer(bool enable);

	/**
	 * Specifies the colour used for clearing the screen.
	 *
	 * @param color New clear color.
	 */
	void setClearColor(const Color &color);

	/**
	 * Moves the drawing cursor to the origin.
	 */
	void resetCursor();

	/**
	 * Moves the drawing cursor by the specified 2D vector.
	 *
	 * @param v Vector.
	 */
	void moveCursor(const Vector2D &v);

	/**
	 * Moves the drawing cursor by the specified 3D vector.
	 *
	 * @param v Vector.
	 */
	void moveCursor(const Vector3D &v);

	/**
	 * Rotates the drawing cursor.
	 *
	 * @param angle Rotation angle in degrees.
	 * @param v     Rotation axis.
	 */
	void rotateCursor(GLfloat angle,const Vector3D &v);

	/**
	 * Shading model.
	 */
	enum ShadeModel {smFlat,smSmooth};

	/**
	 * Fog model.
	 */
	enum FogModel {fmNone,fmLinear,fmExponential,fmExponential2};

	/**
	 * A render context holds parameters and settings for a render task.
	 * When rendering a frame, at least one RenderContext is typically used. More
	 * than one context can be used, also in combination with viewports, to
	 * mix 2D and 3D drawing, or to create splitscreen effects.
	 */
	class RenderContext
	{
		public:
			/** Whether polygons shall be rendered as wireframes instead of filled
			 *  polygons. The default is false. */
			bool wireframe;

			/** Shade model. */
			ShadeModel shadeModel;

			/** Fog rendering quality. 0 is normal. */
			short qualityFog;

			/** Perspective quality. 0 is normal. */
			short qualityPerspective;

			/** Point rendering quality. 0 is normal. */
			short qualityPoints;

			/** Polygon rendering quality. 0 is normal. */
			short qualityPolygons;

			/** Light quality. 0 is normal. */
			short qualityLight;

			/** Whether lighting should be used. */
			bool lighting;

			/** Ambient colour. */
			Color ambient;

			/**
			 * Creates a new render context, using default values.
			 */
			RenderContext();

			/**
			 * Releases the render context.
			 */
			virtual ~RenderContext() {};

			/**
			 * Activates the render context.
			 */
			virtual void apply();
	};

	/**
	 * A 2D render context allows for depth buffer-less 2D drawing
	 * using orthogonal projection. Unit dimensions are set to match
	 * the current viewport, meaning that each unit represents one
	 * pixel.
	 */
	class RenderContext2D : public RenderContext
	{
		public:
			/** Virtual dimensions for the viewport. By default, the current
			 *  viewport dimensions are used, resulting in 1:1 pixel mapping. */
			Vector2D virtualDimensions;

			/** Whether blending should be used. */
			bool blending;

			/** Blend mode. */
			BlendMode blendMode;

			/**
			 * Creates a new 2D render context, using default values.
			 */
			RenderContext2D();

			/**
			 * Creates a new 2D render context, using the specified virtual
			 * dimensions. 0/0 will be mapped to the lower left corner of
			 * the viewport, while the specified vector corresponds to the
			 * upper right corner.
			 *
			 * \todo check: really corresponding to *viewport*?
			 *
			 * @param virtualDimensions Virtual dimensions for this viewport.
			 */
			RenderContext2D(Vector2D virtualDimensions);

			/**
			 * Activates the render context. This sets the unit dimensions
			 * according to the current viewport.
			 */
			void apply();
	};

	/**
	 * A 3D render context can be used to render 3-dimensional scenes.
	 */
	class RenderContext3D : public RenderContext
	{
		public:
			/** Whether depth testing should be enabled. */
			bool depthTest;

			/** Original value for depth buffer pixels. */
			GLfloat clearDepth;

			/** Fog model. \todo Add bool switch */
			FogModel fogModel;

			/** Starting distance for fog. */
			GLfloat fogStart;

			/** Ending distance for fog. */
			GLfloat fogEnd;

			/** Fog density. */
			GLfloat fogDensity;

			/** Fog quality. 0 is normal. */
			short qualityFog;

			/** Fog colour. */
			Color fogColor;

			/**
			 * Creates a new 3D render context, using default values.
			 */
			RenderContext3D();

			/**
			 * Activates the render context.
			 */
			void apply();
	};

	/**
	 * Clears the screen using the colour set by setClearColor().
	 */
	void clearScreen();

	/**
	 * Swaps front and back buffers in double buffered mode.
	 */
	void flipScreen();

	/**
	 * Enables/disables fullscreen mode. Does not work in Windows.
	 *
	 * @param fullscreen <code>true</code> To switch to fullscreen mode,
	 *                   <code>false</code> for windowed mode.
	 */
	void setFullscreen(bool fullscreen);

	/**
	 * Enables/disableds fullscreen mode. Does not work in Windows.
	 * \todo *crashes* in Windows??
	 */
	void toggleFullscreen();

	/**
	 * Initializes a joystick.
	 * \todo make class?
	 *
	 * @param id Joystick ID (typically 0 for the first joystick, 1 for the second etc.).
	 *
	 * @return Joystick object for the opened joystick, or <code>0</code> on error.
	 */
	Joystick *initJoystick(int id=0);

	/**
	 * Closes a joystick.
	 *
	 * @param joy Open joystick.
	 */
	void cleanUpJoystick(Joystick *joy);

	/**
	 * Saves a screenshots as a BMP file.
	 *
	 * Based on code by Ray Kelm:
	 * http://www.libsdl.org/pipermail/sdl/2000-September/030012.html
	 *
	 * @param filename Bitmap file name.
	 *
	 * @throws std::string If an error occurs.
	 */
	void saveScreenshot(const char *filename);

	/**
	 * MySDL's default signal handler. Prints the signal type and
	 * current debug stack trace to stderr and then terminates
	 * the application.
	 *
	 * @param signal Signal type.
	 */
	void handleSignal(int signal);

	/**
	 * Converts a value to a string.
	 *
	 * @param var Some value.
	 *
	 * @returns String representation of the value.
	 */
	template<typename T> std::string toStr(T var)
	{
		std::ostringstream tmp;
		tmp << var;

		return(tmp.str());
	}

	/**
	 * Converts a (preferrably numerical) value to a string.
	 *
	 * @param var   Some value.
	 * @param width Desired width for the resulting string.
	 * @param fill  Character which will be used to fill the remaining
	 *              part of the string if the number is shorter than the
	 *              desired width.
	 *
	 * @returns String representation of the value.
	 */
	template<typename T> std::string toStr(T var,int width,char fill=' ')
	{
		if(width==0)
		{
			return(toStr<T>(var));
		}
		else
		{
			std::ostringstream tmp;
			tmp.fill(fill);
			tmp << std::fixed << std::setw(width) << var;

			return(tmp.str());
		}
	}

	/**
	 * Converts a string to a value.
	 *
	 * @param var String representation of a value.
	 *
	 * @return Value read from the string.
	 *
	 * @throws std::string If the specified string does not contain
	 *                     a value of the requested type.
	 */
	template<typename T> T strTo(std::string var)
	{
		T res;
		std::istringstream tmp(var);

		if((tmp >> res).fail())
		{
			throw(var);
		}
		else
		{
			return(res);
		}
	}

	/**
	 * Saves an SDL surface to an RGBA PNG file. Intended for internal
	 * use and helper applications, since it requires explicit dependency
	 * on the SDL library.
	 *
	 * @param surf     Surface that will be saved.
	 * @param filename Image filename.
	 *
	 * @throws std::string on error.
	 */
	void savePng(SDL_Surface *surf,const char *filename);

	class BackgroundTask;

	/**
	 * Defines callbacks for the BackgroundTask class. Override this class's
	 * methods to implement your desired callback functionality.
	 */
	class BackgroundTaskCallback
	{
		public:
			virtual ~BackgroundTaskCallback() {};

			/**
			 * Called when a background task starts.
			 *
			 * @param task The background task which starts.
			 */
			virtual void onStart(BackgroundTask *task)=0;

			/**
			 * Called when a background task finished.
			 *
			 * @param task The background task which finished.
			 */
			virtual void onFinish(BackgroundTask *task)=0;
	};

	/**
	 * A BackgroundTask is a task that can be run "in the background". For this purpose,
	 * mysdl::perform() executes as many background task steps as possible when called
	 * (i.e. once every frame in a standard application).
	 *
	 * In order to register a background task, it must be added to the mysdl::backgroundTasks
	 * list. Tasks from this list are automatically deleted when they have finished.
	 */
	class BackgroundTask
	{
		friend void perform(Uint32 timeLimit);

		protected:
			/** Callback. Can be <code>0</code>. */
			BackgroundTaskCallback *callback;

			/** Whether this task has finished. Finished tasks will be automatically deleted
			 *  (provided they are registered in the backgroundTasks list). */
			bool hasFinished;

			/** Number of finished steps for this task. */
			Uint32 finishedCount;

			/** Total number of steps for this task. */
			Uint32 totalCount;

			/**
			 * Called when the task has finished.
			 */
			virtual void finish();

		public:
			/**
			 * Creates a new background task. To run automatically, and for the task to be
			 * automatically deleted when it finishes, it must be added to the mysdl::backgroundTasks
			 * list.
			 *
			 * @param callback Callback object. Can be <code>0</code>.
			 */
			BackgroundTask(BackgroundTaskCallback *callback=0);

			virtual ~BackgroundTask() {};

			/**
			 * Queries task progress.
			 *
			 * @param finished Will contain the number of finished steps for this task.
			 * @param total    Will contain the total number of steps for this task.
			 */
			void getProgress(Uint32 *finished,Uint32 *total) const;

			/**
			 * Returns the relative task progress. The returned value is between 0 (no progress)
			 * and 1 (task finished).
			 *
			 * @return Relative progress.
			 */
			GLfloat getProgressRelative() const;

			/**
			 * Performs background task steps.
			 *
			 * @param timeLimit When this system time is reached, the background task will
			 *                  be suspended and mysdl::perform() will return. Background tasks
			 *                  will continue to run at the next call to mysdl::perform().
			 */
			virtual void perform(Uint32 timeLimit)=0;
	};

	/** A list of all currently active backgroundTasks. These will be automatically executed
	 *  by mysdl::perform(), and automatically deleted upon completion. */
	extern std::list<BackgroundTask *> backgroundTasks;

	/**
	 * Performs background tasks.
	 *
	 * @param timeLimit When this system time is reached, the current background task will
	 *                  be suspended and mysdl::perform() will return. Background tasks
	 *                  will continue to run at the next call to mysdl::perform().
	 */
	void perform(Uint32 timeLimit);

	std::list<basic_Vector2D<myint> > getVideoModes();
};

#endif
