/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_RESOURCES_H
#define MYSDL_RESOURCES_H

#include "debug.h"
#include "xml.h"

namespace mysdl
{
	template<class Type> void secureDelete(Type &resource)
	{
		if(resource!=0)
		{
			delete resource;
			resource=0;
		}
	}

	class Reference
	{
		private:
			unsigned int referenceCount;

		public:
			Reference() : referenceCount(0)
			{
			}

			void use();
			void unuse();
	};

	inline void Reference::use()
	{
		referenceCount++;
	}

	inline void Reference::unuse()
	{
		if(referenceCount<=1)
		{
			referenceCount=0;
			delete this;
		}
		else
		{
			referenceCount--;
		}
	}

	class BasicResourceBank;

	class ResourceBankGenerator
	{
		public:
			ResourceBankGenerator();
			virtual ~ResourceBankGenerator();

			virtual BasicResourceBank *generateBank(const std::string &name) const =0;
	};

	//extern std::list<const ResourceBankGenerator *> resourceBankGenerators;

	template<class BankType> class Resource
	{
		private:

		public:
			typedef BankType ResourceBankType;

			ResourceBankType *owner;

			Resource();
			virtual ~Resource();
	};

	template<class BankType,class NameType=std::string> class NamedResource : public Resource<BankType>
	{
		private:

		public:
			typedef NameType ResourceNameType;

			NameType name;

			NamedResource();
			NamedResource(std::string name);
	};

	template<class BankType> Resource<BankType>::Resource() :
		owner(0)
	{
	}

	template<class BankType,class NameType> NamedResource<BankType,NameType>::NamedResource() :
		Resource<BankType>(),name()
	{
	}

	template<class BankType,class NameType> NamedResource<BankType,NameType>::NamedResource(std::string name) :
		Resource<BankType>(),name(name)
	{
	}

	template<class BankType> Resource<BankType>::~Resource()
	{
		if(owner)
		{
			(*owner)-=this;
			owner=0;
		}
	}

	class ResourceManager;

	class BasicResourceBank
	{
		public:
			ResourceManager *manager;

			virtual ~BasicResourceBank()
			{
			}

			virtual std::string getNodeName()
			{
				return("");
			}

			virtual void getFromNode(const xml::Node *root,const std::string &baseDir="")
			{
			}

      virtual void load(const std::string &filename,const std::string &baseDir="")
			{
				std::string dir=baseDir;

				if(dir.length()==0)
				{
					// --- Determine the path of the file (image filenames are ---
					// --- considered to be relative to the path of the file   ---
					// --- defining the bank).                                 ---
					std::string::size_type i=filename.find_first_of('/',0);

					if(i!=std::string::npos)
					{
						//unsigned int j;
						std::string::size_type j;

						while((j=filename.find_first_of('/',i+1))!=std::string::npos)
						{

							i=j;
						}

						dir=filename.substr(0,i+1);
					}
				}

				if((dir.length()>0)&&(dir[dir.length()-1]!='/'))
				{
					dir+='/';
				}

				xml::Node *root=xml::loadXML(filename.c_str());

				if(root)
				{
					if(root->name==getNodeName())
					{
						dir+=root->getAttribute("dir",""); // \todo make constant

						getFromNode(root,dir);
					}

					delete root;
				}
			}
	};

	template<class ResourceType> class NamedResourceBank : public BasicResourceBank
	{
		public:
			typedef typename ResourceType::ResourceNameType NameType;
			typedef typename ResourceType::ResourceBankType SelfType;
			typedef typename std::map<NameType,ResourceType *>::iterator MapIterator;
			typedef typename std::map<NameType,ResourceType *>::const_iterator ConstMapIterator;

			class iterator_impl
			{
				private:
					MapIterator itr;

				public:
					iterator_impl() : itr()
					{
					};

					iterator_impl(MapIterator itr) : itr(itr)
					{
					};

					const NameType &name() const
					{
						return(itr->first);
					}

					bool operator==(const iterator_impl &itr) const
					{
						return(this->itr==itr.itr);
					};

					bool operator!=(const iterator_impl &itr) const
					{
						return(!(operator==(itr)));
					};

					ResourceType *&operator*() const
					{
						return(itr->second);
					};

					ResourceType *operator->() const
					{
						return(itr->second);
					}

					iterator_impl &operator++()
					{
						++itr;
						return(*this);
					}

					iterator_impl operator++(int dummy)
					{
						iterator i=itr;
						++itr;

						return(i);
					}
			};

			class const_iterator_impl
			{
				private:
					ConstMapIterator itr;

				public:
					const_iterator_impl(ConstMapIterator itr) : itr(itr)
					{
					};

					const NameType &name() const
					{
						return(itr->first);
					}

					bool operator==(const const_iterator_impl &itr) const
					{
						return(this->itr==itr.itr);
					};

					bool operator!=(const const_iterator_impl &itr) const
					{
						return(!(operator==(itr)));
					};

					ResourceType const *&operator*() const
					{
						return(itr->second);
					}

					ResourceType const *operator->() const
					{
						return(itr->second);
					}

					const_iterator_impl &operator++()
					{
						++itr;
						return(*this);
					}

					const_iterator_impl operator++(int dummy)
					{
						const_iterator i=itr;
						++itr;

						return(i);
					}
			};

			typedef iterator_impl iterator;
			typedef const_iterator_impl const_iterator;

		private:
			std::map<NameType,ResourceType *> map;

		public:
			~NamedResourceBank();

			ResourceType *const operator[](const NameType &name);
			ResourceType *const at(const NameType &name) const;
			void operator+=(ResourceType *value);
			void operator-=(ResourceType *value);

			void clear();
			bool empty() const;
			virtual int size() const;

			void operator-=(void *resource) { (*this)-=(ResourceType *)(resource); };

			iterator find(const NameType &name)
			{
				return(iterator(map.find(name)));
			}

			iterator begin()
			{
				return(iterator(map.begin()));
			}

			iterator end()
			{
				return(iterator(map.end()));
			}

			const_iterator begin() const
			{
				return(const_iterator(map.begin()));
			}

			const_iterator end() const
			{
				return(const_iterator(map.end()));
			}
	};

	template<class ResourceType> NamedResourceBank<ResourceType>::~NamedResourceBank()
	{
		clear();
	}

	template<class ResourceType> inline ResourceType *const NamedResourceBank<ResourceType>::operator[](const NameType &name)
	{
		ResourceType *resource=at(name);
		return(resource);
	}

	template<class ResourceType> inline ResourceType *const NamedResourceBank<ResourceType>::at(const NameType &name) const
	{
		ConstMapIterator itr=map.find(name);

		if(itr==map.end())
		{
			return(0);
		}
		else
		{
			return(itr->second);
		}
	}

	template<class ResourceType> inline void NamedResourceBank<ResourceType>::operator+=(ResourceType *resource)
	{
		typename std::map<NameType,ResourceType *>::iterator itr=map.find(resource->name);

		if(itr!=map.end())
		{
			delete itr->second;
		}

		map[resource->name]=resource;
		resource->owner=(SelfType *)(this);
	}

	template<class ResourceType> inline void NamedResourceBank<ResourceType>::operator-=(ResourceType *resource)
	{
		if(map.erase(resource->name))
		{
			resource->owner=0;
		}
		else
		{
			debug::warning(0,"Could not remove resource %p from bank %p.",resource,this); // TODO: return value?
		}
	}

	template<class ResourceType> inline void NamedResourceBank<ResourceType>::clear()
	{
		while(!empty())
		{
			delete map.rbegin()->second;
		}
	}

	template<class ResourceType> inline bool NamedResourceBank<ResourceType>::empty() const
	{
		return(map.empty());
	}

	template<class ResourceType> inline int NamedResourceBank<ResourceType>::size() const
	{
		return(map.size());
	}

	class ResourceManager : public BasicResourceBank
	{
		protected:
			std::map<std::string,BasicResourceBank *> banks;

		public:
			virtual ~ResourceManager()
			{
				clear();
			}

			std::string getNodeName()
			{
				return("resources");
			}

			int size()
			{
				return(banks.size());
			}

			void clear()
			{
				banks.clear();
			}

			void rename(const std::string &oldName,const std::string &newName)
			{
				std::map<std::string,BasicResourceBank *>::iterator itr=banks.find(oldName);

				if(itr==banks.end())
				{
					throw std::string("Resource \""+oldName+"\" not found.");
				}
				else
				{
					BasicResourceBank *bank=itr->second;

					banks.erase(itr);
					banks[newName]=bank;
				}
			}

			void add(const std::string &name,BasicResourceBank *bank)
			{
				std::map<std::string,BasicResourceBank *>::iterator itr=banks.find(name);

				if(itr!=banks.end())
				{
					delete itr->second;
				}

				banks[name]=bank;
				bank->manager=this;
			}

			void erase(const std::string &name)
			{
				std::map<std::string,BasicResourceBank *>::iterator itr=banks.find(name);

				if(itr!=banks.end())
				{
					delete itr->second;
					banks.erase(itr);
				}
			}

			void erase(BasicResourceBank *bank)
			{
				std::map<std::string,BasicResourceBank *>::iterator itr;

				for(itr=banks.begin(); itr!=banks.end(); itr++)
				{
					if(itr->second==bank)
					{
						break;
					}
				}

				if(itr!=banks.end())
				{
					delete itr->second;
				}
			}

			BasicResourceBank *operator[](const std::string &name)
			{
				std::map<std::string,BasicResourceBank *>::iterator itr=banks.find(name);

				if(itr==banks.end())
				{
					return(0);
				}
				else
				{
					return(itr->second);
				}
			}

			void getFromNode(const xml::Node *root,const std::string &baseDir="");
			virtual BasicResourceBank* createResourceBank(const std::string &name);
	};

	class RefreshingBankHandler
	{
		public:
			virtual ~RefreshingBankHandler() {};
			virtual void elementRefreshed()=0;
	};

	class RefreshingBank
	{
		public:
			virtual ~RefreshingBank() {};
			virtual int size() const=0;
			virtual void refresh(RefreshingBankHandler *handler)=0;
	};

	class ResourceLoaderHandler
	{
		public:
			virtual ~ResourceLoaderHandler() {};
			virtual void loaderFinished()=0;
			virtual void loaderUpdate(RefreshingBank *current,int progress,int total)=0;
	};

	class ResourceLoader : public RefreshingBankHandler
	{
		private:
			std::list<RefreshingBank *> list;
			ResourceLoaderHandler *handler;

			RefreshingBank *current;
			int progress;
			int total;

		public:
			ResourceLoader(ResourceLoaderHandler *handler);
			virtual ~ResourceLoader();

			void operator+=(RefreshingBank *add);
			void start();
			void thread();
			void elementRefreshed();
	};
};

#endif
