/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_SPRITES_H
#define MYSDL_SPRITES_H

#include <map>

#include "SDL/SDL_opengl.h"

#include "../tools.h"

#include "../gfx/animations.h"
#include "../gfx/bitmasks.h"
#include "../gfx/images.h"

namespace mysdl
{
	/**
	 * Contains a sprite engine class as well as abstract base classes for sprites and a
	 * concrete class for 2D sprites displaying images or animations.
	 */
	namespace sprites
	{
		/**
		 * Different blend modes for drawing sprites.
		 *
		 * @param bmDefault No specific blend mode, used to specify that a sprite shall
		 *                  use the default blend mode set in its engine.
		 *
		 * @param bmNormal  Pixels are drawn using their alpha value for the source
		 *                  and their complementary alpha value for the destination pixel.
		 *
		 * @param bmAdd     Pixels are drawn using their alpha value for the source
		 *                  and 1 for the destination pixel's alpha.
		 */
// 		enum BlendMode {bmDefault,bmNormal,bmAdd};

		/**
		 * Sets OpenGL to use the specified blend mode.
		 *
		 * @param mode Desired blend mode.
		 */
// 		void setBlendMode(BlendMode mode);

		class Sprite;
		class SpriteEngine;

		/**
		* Abstract 2D Sprite base class.
		*
		* Every Sprite has a 2D position, width and hotspot. The width can be used for
		* bounding box collision tests. Sprites can (and should) be attached to a
		* sprite engine which takes care of managing sprite resources, drawing and
		* collision testing.
		*
		* Sprite is an abstract class. To create a sprite instance, you have to override
		* at least the draw() method.
		*/
		class Sprite
		{
			friend class SpriteEngine;

			public:
				/** Sprite engine owning this sprite. 0 if the sprite is not owned by an
				 *  engine and therefore still entirely under the programmer's
				 *  control. */
				SpriteEngine *owner;

				/** Colour which will be used to draw the sprite. A sprite's pixels will
				 *  be multiplied by this colour when drawing the sprite. */
				Color color;

			public:
				/** Sprite position. */
				Vector2D pos;

				/** Sprite dimensions (bounding box). */
				Vector2D dimensions;

				/** Sprite hotspot position. This is the point of the sprite which is displayed
				 *  add the position specified by the X and Y coordinate fields. */
				Vector2D hotspot;

				/** When set to <code>false</code>, the sprite is not drawn. */
				bool visible;

				/** Collision testing with a sprite is only performed when this is set to
				 *  <code>true</code>. */
				bool canCollide;

				/** Blend mode used by this sprite. By default this is bmDefault, meaning that the
				 *  blend mode set in the sprite engine will be used. */
				BlendMode blendMode;

				/** Specifies the sprite's outer bounding box. Outside this box, no pixels are
				 *  "set". Two sprites can only collide if their outer bounding boxes overlap. */
				RectInt outerBoundingBox;

				/** Specifies the sprite's inner bounding box. Within this box, all pixels are
				 *  considered to be "set". When two sprites' inner bounding boxes overlap, they
				 *  are colliding. */
				RectInt innerBoundingBox;

				/**
				* Creates a new Sprite instance.
				*
				* @param pos     Initial position.
				* @param visible Initial visibility.
				*/
				Sprite(const Vector2D pos=Vector2D(0,0),const bool visible=false);

				/**
				* Frees a sprite.
				*/
				virtual ~Sprite();

				/**
				 * Sets the sprite's colour.
				 *
				 * @param color New colour.
				 */
				void setColor(const Color &color)
				{
					this->color=color;
				}

				/**
				 * Returns the sprite's colour.
				 *
				 * @return Colour.
				 */
				const Color &getColor()
				{
					return(color);
				}

				/**
				 * Sets the correct blend mode and draws the sprite.
				 */
				void draw();

				/**
				 * Draws the actual sprite. This is an abstract method that has to be
				 * overridden for a usable Sprite class. This method does not have to
				 * care about the sprite's blend mode, since draw() correctly handles
				 * setting the required OpenGL state.
				 */
				virtual void drawSelf()=0;

				/**
				 * Performs sprite actions. The default implementation does nothing.
				 *
				 * @param ticks Number of ticks for which the sprite actions should be performed.
				 */
				virtual void perform(GLfloat ticks);

				/**
				 * Returns the sprite's bitmask.
				 *
				 * @return Pointer to the sprite's bitmask, or 0 if none.
				 */
				virtual mysdl::gfx::Bitmask *getBitmask() const;

				/**
				 * Checks for an outer bounding box collision with another sprite.
				 *
				 * @param spr Second sprite.
				 *
				 * @return <code>true</code> if the sprites' outer bounding boxes overlap.
				 */
				bool checkCollisionOuterBoundingBox(const Sprite *spr);

				/**
				 * Checks for an inner bounding box collision with another sprite.
				 *
				 * @param spr Second sprite.
				 *
				 * @return <code>true</code> if the sprites' inner bounding boxes overlap.
				 */
				bool checkCollisionInnerBoundingBox(const Sprite *spr);

				/**
				 * Checks for bitmask collision with another sprite.
				 *
				 * @param spr Second sprite.
				 *
				 * @return <code>true</code> if the sprites' bitmasks overlap.
				 */
				bool checkCollisionBitmask(const Sprite *spr) const;

				/**
				 * Checks for collision with another sprite. First a bounding
				 * box collision is performed. If this test was positive, a
				 * bitmask collision is performed (only if both sprites do
				 * have a bitmask).
				 *
				 * @param spr Second sprite.
				 *
				 * @return <code>true</code> if the sprites are colliding.
				 */
				virtual bool checkCollision(const Sprite *spr);

				/**
				 * When doing collision testing on a range of sprites through a SpriteEngine,
				 * this method is called whenever a collision between this sprite and another
				 * sprite is detected. The default implementation does nothing.
				 *
				 * @param spr Colliding sprite.
				 */
				virtual void onCollision(Sprite *spr);
		};

		/**
		 * An image-based Sprite class. Every sprite is at each point in time represented by
		 * one Image. For bitmask collisions, the bitmask of the sprite's image is used. The
		 * sprite's dimensions are automatically set to those of the image.
		 */
		class ImageSprite : public Sprite
		{
			protected:
				/**
				 * Clears the sprite's image, automatically releasing any references that might have been created
				 * when setting the image.
				 */
				void autoclearImage();

				/**
				 * Reads the sprite's image data from its current image. Called when the sprite's
				 * image changes to cache this data and avoid repeated calls to the required methods.
				 */
				void getImageData();

			public:
				/** The image currently assigned to the sprite. */
				mysdl::gfx::Image *image;

				/** Vertex coordinates of the quadrilateral displaying the image. These are set
				 *  automatically by setImage(). */
				GLfloat vertexCoordinates[4][2];

				/** Texture coordinates for the quadrilateral displaying the image. The default
				 *  coordinates project the entire image on the full quadrilateral. */
				GLfloat vertexTextureCoordinates[4][2];

				/** Colours of the four quadrilateral corners. By default, they are set to
				 *  opaque white, resulting in non-transparent display of the image in its
				 *  original colours. */
				GLfloat vertexColors[4][4];

				/**
				 * Creates a new image sprite.
				 *
				 * @param pos     Initial position.
				 * @param visible Initial visibility.
				 * @param image   Sprite image.
				 */
				ImageSprite(const Vector2D pos=Vector2D(0,0),const bool visible=false,mysdl::gfx::Image *const image=0);

				/**
				 * Deletes an image sprite.
				 */
				~ImageSprite();

				/**
				 * Sets the sprite's "image" to an animation, automatically generating an animation reference for it.
				 * This reference does not have to be released by the user, as the sprite handles this automatically
				 * when it no longer needs the animation.
				 *
				 * @param animation The animation that will be used to draw the sprite.
				 * @param delay     Delay factor for the animation. TODO: ?
				 *
				 * @return The new animation reference which connects the sprite to the animation.
				 */
				mysdl::gfx::AnimationReference *setAnimation(mysdl::gfx::Animation *const animation,GLfloat delay=1);

				/**
				 * Draws the sprite, using the current image and the current vertexCoordinates,
				 * vertexTextureCoordinates and vertexColors.
				 */
				virtual void drawSelf();

				/**
				 * Returns the image currently assigned to the sprite.
				 *
				 * @return Image pointer.
				 */
				const mysdl::gfx::ImageReference *getImage();

				/**
				 * Returns the sprite's bitmask, i.e. that of the current image.
				 *
				 * @return Bitmask pointer.
				 */
				mysdl::gfx::Bitmask *getBitmask() const;

				/**
				 * Sets the sprite's current image.
				 *
				 * @param image Pointer to new image.
				 */
				void setImage(mysdl::gfx::Image *const image);

				/**
				 * Performs sprite actions. This automatically advances the animation and updates
				 * the current image if necessary.
				 *
				 * @param ticks Number of ticks for which to perform sprite actions.
				 */
				void perform(GLfloat ticks);
		};

		/** A list of sprite pointers. */
		typedef std::list<Sprite *> SpriteList;

		/**
		 * A sprite engine that manages a list of sprites, including
		 * resource management, drawing and updating the sprites and
		 * performing collision detection.
		 * <p>
		 * When a sprite is added to a sprite engine, the engine takes
		 * over management of that sprite's resources. That means that
		 * as long as the programmer does not detach the sprite from
		 * the engine again, the sprite must not be released. The engine's
		 * destructor automatically releases all sprites attached to
		 * the engine when it is called.
		 */
		class SpriteEngine
		{
			public:
				/** Sprites managed by this collection. */
				SpriteList sprites;

				/** The blend mode to be used for sprites which don't specify their
				 *  own custom blend mode. */
				BlendMode blendMode;

				/**
				 * Creates a new sprite engine.
				 */
				SpriteEngine();

				/**
				 * Releases the sprite engine, including all sprites
				 * attached to it.
				 */
				~SpriteEngine();

				/**
				 * Draws all sprites managed by the engine.
				 */
				void draw();

				/**
				 * Adds a sprite to the back of the list.
				 *
				 * @param sprite Sprite to be added to the list. If the sprite is currently
				 *               owned by a different engine, it is removed from that engine
				 *               first.
				 */
				void addBack(Sprite *sprite);

				/**
				 * Adds a sprite to the front of the list.
				 *
				 * @param sprite Sprite to be added to the list. If the sprite is currently
				 *               owned by a different engine, it is removed from that engine
				 *               first.
				 */
				void addFront(Sprite *sprite);

				/**
				 * Removes a sprite from the list.
				 *
				 * @param sprite Sprite to be removed.
				 */
				void remove(Sprite *sprite);

				/**
				 * Removes all sprite from the list.
				 *
				 * TODO: Should this delete the sprites??
				 */
				void clear();

				/**
				 * Returns the number of sprites stored in the list.
				 *
				 * @return Number of sprites.
				 */
				int size();

				/**
				 * Calls the perform() methods for all sprites managed by
				 * the engine.
				 *
				 * @param ticks Number of ticks for which to perform sprite actions.
				 */
				void perform(GLfloat ticks);

				/**
				 * Tests each sprite in the list for collisions with all the other
				 * sprites in the list. If a collision between two sprites is
				 * detected, for sprites' onCollide() methods are called.
				 */
				void testCollisions();

				/**
				 * Tests each sprite in the list for collisions with all sprites
				 * in the list of a second engine. If a collision between two sprites is
				 * detected, for sprites' onCollide() methods are called.
				 *
				 * @param e2 Second engine.
				 */
				void testCollisions(SpriteEngine *e2);
		};
	};
};

#endif
