/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MYSDL_XML_H
#define MYSDL_XML_H

#include <list>
#include <map>
#include <string>

namespace mysdl
{
	namespace xml
	{
		/**
		* Base class for XML handlers. An XML handler is a helper class used when parsing
		* XML files through an XMLReader.
		*/
		class XMLHandler
		{
			public:
				virtual ~XMLHandler() {};

				/**
				* Called when the XMLReader finds a new XML element. The default
				* implementation does nothing, override it if you need to react to
				* this kind of parsing event.
				*
				* @param name  Name of the XML element.
				* @param attrs Array of XML attribute name/value pairs. The even
				*              array entries are the name strings, the odd entries
				*              are the corresponding value strings.
				*/
				virtual void startElement(const char *name,const char **attrs) {};

				/**
				* Called when the XMLReader finds the end of an XML element. The
				* default implementation does nothing, override it if you need to
				* react to this kind of parsing event.
				*
				* @param name Name of the XML element.
				*/
				virtual void endElement(const char *name) {};
		};

		/**
		* A class for reading XML files. You will need to create your own
		* custom handler class derived from XMLHandler to use this class.
		* <p>
		* The current implementation of XMLReader uses the expat parser
		* library for parsing XML files.
		*/
		class XMLReader
		{
			private:
				/** Handler class. */
				XMLHandler *handler;

			public:
				/**
				* Creates a new XMLReader and associates it with the specified
				* handler. When reading an XML file, this handler's methods are
				* called according to the contents of the input file.
				*
				* @param handler Handler object.
				*/
				XMLReader(XMLHandler *handler);

				/**
				* Reads the specified XML file, using the handler to interpret
				* the contents. If an error occurs, std::string exceptions are
				* thrown.
				*
				* @param filename Name of the XML file.
				*/
				void readFile(std::string filename);
		};

		class Node;

		/**
		 * A list of DOM Node pointers.
		 */
		typedef std::list<Node *> Nodes;

		/**
		 * A list of attributes, mapping names to values.
		 */
		typedef std::map<std::string,std::string> Attributes;

		/**
		 * A DOM node. Every DOM node has a name, a list of attributes, and
		 * a list of children. A DOM node along with all its children represents
		 * a DOM tree.
		 */
		class Node
		{
			public:
				/** Parent node. 0 if this is the root node. */
				Node *parent;

				/** Node name. */
				std::string name;

				/** Node attributes. */
				Attributes attributes;

				/** Node children. */
				Nodes children;

				/**
				 * Creates a new, empty DOM node.
				 */
				Node();

				/**
				 * Creates a new DOM node using the specified values.
				 * Can be used to create nodes directly from the data
				 * provided by expat's start and end node parser functions.
				 *
				 * @param name 			 Node name.
				 * @param attributes An array of attribute string pairs.
				 *                   All odd entries are attribute names, while the even nodes
				 *                   are the corresponding values. The array must be terminated
				 *                   by a null pointer.
				 */
				Node(const char *name,const char **attributes=0);

				/**
				 * Frees the node and all its children.
				 */
				~Node();

				/**
				 * Adds a new child node.
				 *
				 * @param node New child node.
				 */
				void operator+=(Node *node);

				/**
				 * Removes a child node.
				 *
				 * @param node Child node which will be removed.
				 */
				void operator-=(Node *node);

				/**
				 * Returns a reference to the specified attribute. If the
				 * attribute doesn't exist, it will be created and an
				 * empty string will be assigned as its value.
				 *
				 * @param name Attribute name.
				 *
				 * @return Attribute value reference.
				 */
				std::string &operator[](const std::string name);

				/**
				 * Returns a pointer to an attribute value.
				 *
				 * @param name Attribute name.
				 *
				 * @return Pointer to the corresponding value, or 0 if
				 *         the attribute doesn't exist.
				 */
				const std::string *getAttribute(const std::string name) const;

				/**
				 * Returns an attribute value string, or a default value if the
				 * attribute doesn't exist.
				 *
				 * @param name     Attribute name.
				 * @param defValue Default value.
				 *
				 * @return Attribute value (or the default value).
				 */
				std::string getAttribute(const std::string name,const std::string defValue) const;

				/**
				 * Returns an attribute value as a long number, or a default value
				 * if the attribute doesn't exist.
				 *
				 * @param name 		 Attribute name.
				 * @param defValue Default value.
				 *
				 * @return Attribute value (or the default value).
				 */
				long getAttributeLong(const std::string name,const long defValue) const;

				/**
				 * Sets an attribute values as a long number.
				 *
				 * @param name  Attribute name.
				 * @param value Attribute value.
				 */
				void setAttributeLong(const std::string name,const long value);

				/**
				 * Returns an attribute value as a floating point number, or a default value
				 * if the attribute doesn't exist.
				 *
				 * @param name 		 Attribute name.
				 * @param defValue Default value.
				 *
				 * @return Attribute value (or the default value).
				 */
				float getAttributeFloat(const std::string name,const float defValue) const;

				/**
				 * Sets an attribute values as a floating point number.
				 *
				 * @param name  Attribute name.
				 * @param value Attribute value.
				 */
				void setAttributeFloat(const std::string name,const float value);

				/**
				 * Returns an attribute value as a boolean value, or a default value
				 * if the attribute doesn't exist. Possible value strings for boolean
				 * values are: "true", "false", "t", "f", "1", "0". All other strings
				 * are interpreated as "false".
				 *
				 * @param name 		 Attribute name.
				 * @param defValue Default value.
				 *
				 * @return Attribute value (or the default value).
				 */
				bool getAttributeBool(const std::string name,const bool defValue) const;

				/**
				 * Sets an attribute values as a boolean value.
				 *
				 * @param name  Attribute name.
				 * @param value Attribute value.
				 */
				void setAttributeBool(const std::string name,const bool value);

				/**
				 * Returns an XML string representation of the node and all of its
				 * subnodes. This does <b>not</b> perform any encoding at the moment,
				 * so node/attribute names and values containing certain XML characters
				 * will cause this method to create incorrect XML code.
				 *
				 * @param indent Indent depth. Default value is 0. This is automatically
				 *               increased for recursive calls.
				 *
				 * @return XML string representation of the DOM tree.
				 */
				std::string toString(int indent=0) const;

				/**
				 * Writes the node to an XML file.
				 *
				 * @param filename File name.
				 */
				void saveXML(const char *filename);
		};

		/**
		 * Helper class for handling loops when reading resources from
		 * a DOM tree.
		 */
		class LoopData
		{
			public:
				/** Name of the loop counter variable. */
				std::string var;

				/** Current value of the loop counter. */
				int value;

				/** Destination value for the loop counter. TODO: rename, can be lower than initial value. */
				int max;

				/** Increment value for the loop counter. */
				int step;

				/** Number of digits to use when resolving the counter variable. */
				int digits;

				/**
				 * Creates a new, empty loop.
				 */
				LoopData();

				/**
				 * Creates a new loop.
				 *
				 * @param var    Name of the loop counter variable.
				 * @param value  Current value of the loop counter.
				 * @param max    Destination value for the loop counter. TODO: rename, can be lower than initial value.
				 * @param step   Increment value for the loop counter.
				 * @param digits Number of digits to use when resolving the counter variable.
				 */
				LoopData(std::string var,int value,int max,int step=1,int digits=0);

				/**
				 * Advances the loop to the next iteration by adding the step increment to the
				 * current loop counter.
				 *
				 * @return <code>true</code> if the loop had already finished. In this case, the
				 *         loop counter was unchanged.
				 */
				bool advance();

				/**
				 * Resolves the loop counter variable by transforming into a string.
				 *
				 * @return Counter value as a string, using <code>digits</code> digits,
				 *         left-padding it with zeroes if necessary.
				 */
				std::string getString();
		};

		/**
		 * Parses an XML file and constructs a DOM tree.
		 *
		 * @param filename XML file name.
		 *
		 * @return Pointer to a DOM tree, or 0 if the file did not contain data.
		 *
		 * @throw std::string When a read error occurs.
		 */
		Node *loadXML(const char *filename);
	};
};

#endif
